@@ -3,9 +3,10 @@
  * by Szabolcs Berecz <szabi@inf.elte.hu>
  * (C) 2001
  *
- * to compile tester app: gcc -Iloader/ -DTESTING -o codec-cfg codec-cfg.c
+ * to compile test application:
+ *  cc -I. -DTESTING -o codec-cfg-test codec-cfg.c mp_msg.o osdep/getch2.o -ltermcap
  * to compile CODECS2HTML:
- *   gcc -Iloader/ -DCODECS2HTML -o codecs2html codec-cfg.c mp_msg.o
+ *   gcc -DCODECS2HTML -o codecs2html codec-cfg.c mp_msg.o
  *
  * TODO: implement informat in CODECS2HTML too
  */
@@ -26,6 +27,13 @@
 
 #include "config.h"
 #include "mp_msg.h"
+#ifdef CODECS2HTML
+#ifdef __GNUC__
+#define mp_msg(t, l, m, args...) fprintf(stderr, m, ##args)
+#else
+#define mp_msg(t, l, ...) fprintf(stderr, __VA_ARGS__)
+#endif
+#endif
 
 #include "help_mp.h"
 
@@ -174,6 +182,12 @@ static int add_to_format(char *s, char *alias,unsigned int *fourcc, unsigned int
 		{"IDCT_MPEG2",IMGFMT_XVMC_IDCT_MPEG2},
 		{"MOCO_MPEG2",IMGFMT_XVMC_MOCO_MPEG2},
 
+		{"VDPAU_MPEG1",IMGFMT_VDPAU_MPEG1},
+		{"VDPAU_MPEG2",IMGFMT_VDPAU_MPEG2},
+		{"VDPAU_H264",IMGFMT_VDPAU_H264},
+		{"VDPAU_WMV3",IMGFMT_VDPAU_WMV3},
+		{"VDPAU_VC1",IMGFMT_VDPAU_VC1},
+
 		{NULL,    0}
 	};
 
@@ -277,11 +291,9 @@ static short get_driver(char *s,int audioflag)
 		"null",
 		"libmpeg2",
 		"vfw",
-		"odivx",
 		"dshow",
 		"ffmpeg",
 		"vfwex",
-		"divx4",
 		"raw",
 		"msrle",
 		"xanim",
@@ -343,14 +355,14 @@ static int validate_codec(codecs_t *c, int type)
 	}
 
 #if 0
-#warning codec->driver == 4;... <- ezt nem kellene belehegeszteni...
-#warning HOL VANNAK DEFINIALVA????????????
+#warning codec->driver == 4;... <- this should not be put in here...
+#warning Where are they defined ????????????
 	if (!c->dll && (c->driver == 4 ||
 				(c->driver == 2 && type == TYPE_VIDEO))) {
 		mp_msg(MSGT_CODECCFG,MSGL_ERR,MSGTR_CodecNeedsDLL, c->name);
 		return 0;
 	}
-#warning guid.f1 lehet 0? honnan lehet tudni, hogy nem adtak meg?
+#warning Can guid.f1 be 0? How does one know that it was not given?
 //	if (!(codec->flags & CODECS_FLAG_AUDIO) && codec->driver == 4)
 
 	if (type == TYPE_VIDEO)
@@ -372,7 +384,7 @@ static int add_comment(char *s, char **d)
 		pos = strlen(*d);
 		(*d)[pos++] = '\n';
 	}
-	if (!(*d = (char *) realloc(*d, pos + strlen(s) + 1))) {
+	if (!(*d = realloc(*d, pos + strlen(s) + 1))) {
 		mp_msg(MSGT_CODECCFG,MSGL_FATAL,MSGTR_CantAllocateComment);
 		return 0;
 	}
@@ -479,7 +491,7 @@ static codecs_t *audio_codecs=NULL;
 static int nr_vcodecs = 0;
 static int nr_acodecs = 0;
 
-int parse_codec_cfg(char *cfgfile)
+int parse_codec_cfg(const char *cfgfile)
 {
 	codecs_t *codec = NULL; // current codec
 	codecs_t **codecsp = NULL;// points to audio_codecs or to video_codecs
@@ -513,7 +525,7 @@ int parse_codec_cfg(char *cfgfile)
 		return 0;
 	}
 
-	if ((line = (char *) malloc(MAX_LINE_LEN + 1)) == NULL) {
+	if ((line = malloc(MAX_LINE_LEN + 1)) == NULL) {
 		mp_msg(MSGT_CODECCFG,MSGL_FATAL,MSGTR_CantGetMemoryForLine, strerror(errno));
 		return 0;
 	}
@@ -570,7 +582,7 @@ int parse_codec_cfg(char *cfgfile)
 				goto err_out;
 #endif
 			}
-		        if (!(*codecsp = (codecs_t *) realloc(*codecsp,
+		        if (!(*codecsp = realloc(*codecsp,
 				sizeof(codecs_t) * (*nr_codecsp + 2)))) {
 			    mp_msg(MSGT_CODECCFG,MSGL_FATAL,MSGTR_CantReallocCodecsp, strerror(errno));
 			    goto err_out;
@@ -731,17 +743,17 @@ err_out_release_num:
 static void codecs_free(codecs_t* codecs,int count) {
 	int i;
 		for ( i = 0; i < count; i++)
-			if ( (codecs[i]).name ) {
-				if( (codecs[i]).name )
-					free((codecs[i]).name);
-				if( (codecs[i]).info )
-					free((codecs[i]).info);
-				if( (codecs[i]).comment )
-					free((codecs[i]).comment);
-				if( (codecs[i]).dll )
-					free((codecs[i]).dll);
-				if( (codecs[i]).drv )
-					free((codecs[i]).drv);
+			if ( codecs[i].name ) {
+				if( codecs[i].name )
+					free(codecs[i].name);
+				if( codecs[i].info )
+					free(codecs[i].info);
+				if( codecs[i].comment )
+					free(codecs[i].comment);
+				if( codecs[i].dll )
+					free(codecs[i].dll);
+				if( codecs[i].drv )
+					free(codecs[i].drv);
 			}
 		if (codecs)
 			free(codecs);
@@ -812,36 +824,32 @@ codecs_t* find_codec(unsigned int fourcc,unsigned int *fourccmap,
 	return NULL;
 }
 
-void select_codec(char* codecname,int audioflag){
-	int i;
-	codecs_t *c;
-//	printf("select_codec('%s')\n",codecname);
-	if (audioflag) {
-		i = nr_acodecs;
-		c = audio_codecs;
-	} else {
-		i = nr_vcodecs;
-		c = video_codecs;
-	}
-	if(i)
-	for (/* NOTHING */; i--; c++)
-	    if(!strcmp(c->name,codecname))
-		c->flags|=CODECS_FLAG_SELECTED;
+void stringset_init(stringset_t *set) {
+  *set = calloc(1, sizeof(char *));
 }
 
-void codecs_reset_selection(int audioflag){
-	int i;
-	codecs_t *c;
-	if (audioflag) {
-		i = nr_acodecs;
-		c = audio_codecs;
-	} else {
-		i = nr_vcodecs;
-		c = video_codecs;
-	}
-	if(i)
-	for (/* NOTHING */; i--; c++)
-		c->flags&=(~CODECS_FLAG_SELECTED);
+void stringset_free(stringset_t *set) {
+  int count = 0;
+  while ((*set)[count]) free((*set)[count++]);
+  free(*set);
+  *set = NULL;
+}
+
+void stringset_add(stringset_t *set, const char *str) {
+  int count = 0;
+  while ((*set)[count]) count++;
+  count++;
+  *set = realloc(*set, sizeof(char *) * (count + 1));
+  (*set)[count - 1] = strdup(str);
+  (*set)[count] = NULL;
+}
+
+int stringset_test(stringset_t *set, const char *str) {
+  stringset_t s;
+  for (s = *set; *s; s++)
+    if (strcmp(*s, str) == 0)
+      return 1;
+  return 0;
 }
 
 void list_codecs(int audioflag){
@@ -881,7 +889,7 @@ void list_codecs(int audioflag){
 /*
  * Fake out GUI references when building the codecs2html utility.
  */
-#ifdef HAVE_NEW_GUI
+#ifdef CONFIG_GUI
 void gtkMessageBox( int type,char * str ) { return; }
 int use_gui = 0;
 #endif
@@ -1020,9 +1028,11 @@ int main(int argc, char* argv[])
 		nr[1] = nr_acodecs;
 		
 		printf("/* GENERATED FROM %s, DO NOT EDIT! */\n\n",argv[1]);
+		printf("#include <stddef.h>\n",argv[1]);
+		printf("#include \"codec-cfg.h\"\n\n",argv[1]);
 		
 		for (i=0; i<2; i++) {
-		  	printf("codecs_t %s[] = {\n", nm[i]);
+		  	printf("const codecs_t %s[] = {\n", nm[i]);
 			for (j = 0; j < nr[i]; j++) {
 			  	printf("{");
 
@@ -1079,7 +1089,7 @@ int main(int argc, char* argv[])
             if(d>='0' && d<='9'){
                 // begin section
                 section=d-'0';
-                printf("BEGIN %d\n",section);
+                //printf("BEGIN %d\n",section);
                 if(section>=5){
                     // audio
 		    cl = audio_codecs;
@@ -1173,8 +1183,10 @@ next:
 		    printf("info='%s'\n",c->info);
 		    printf("comment='%s'\n",c->comment);
 		    printf("dll='%s'\n",c->dll);
-		    printf("flags=%X  driver=%d status=%d cpuflags=%d\n",
-				    c->flags, c->driver, c->status, c->cpuflags);
+		    /* printf("flags=%X  driver=%d status=%d cpuflags=%d\n",
+		              c->flags, c->driver, c->status, c->cpuflags); */
+		    printf("flags=%X status=%d cpuflags=%d\n",
+				    c->flags, c->status, c->cpuflags);
 
 		    for(j=0;j<CODECS_MAX_FOURCC;j++){
 		      if(c->fourcc[j]!=0xFFFFFFFF){
