@@ -10,10 +10,10 @@ static char help_text[]=
 "기본 선택사항: (전체 목록은 man 페이지 참조)\n"
 " -vo <drv[:dev]>  비디오 출력 드라이버 및 장치 선택 (목록보기는 '-vo help')\n"
 " -ao <drv[:dev]>  오디오 출력 드라이버 및 장치 선택 (목록보기는 '-ao help')\n"
-#ifdef HAVE_VCD
+#ifdef CONFIG_VCD
 " vcd://<trackno>  (S)VCD (Super Video CD) 트랙 재생 (장치로부터, 마운트 없이)\n"
 #endif
-#ifdef USE_DVDREAD
+#ifdef CONFIG_DVDREAD
 " dvd://<titleno>  일반 파일이 아닌 장치로부터 DVD 타이틀 재생\n"
 " -alang/-slang    DVD 오디오/자막 언어 선택 (두 글자의 국가 코드)\n"
 #endif
@@ -58,7 +58,6 @@ static char help_text[]=
 #define MSGTR_NoHomeDir "홈디렉토리를 찾을 수 없습니다.\n"
 #define MSGTR_GetpathProblem "get_path(\"config\") 문제 발생\n"
 #define MSGTR_CreatingCfgFile "설정파일을 만듭니다.: %s\n"
-#define MSGTR_CopyCodecsConf "((MPlayer 소스 트리의) etc/codecs.conf를 ~/.mplayer/codecs.conf로 복사 또는 링크하세요.)\n"
 #define MSGTR_BuiltinCodecsConf "내장된 기본 codecs.conf를 사용합니다.\n"
 #define MSGTR_CantLoadFont "폰트를 읽어 들일 수 없습니다.: %s\n"
 #define MSGTR_CantLoadSub "자막을 읽어 들일 수 없습니다.: %s\n"
@@ -144,7 +143,7 @@ static char help_text[]=
 
 // mencoder.c:
 
-#define MSGTR_UsingPass3ControllFile "pass3 컨트롤 파일을 사용합니다.: %s\n"
+#define MSGTR_UsingPass3ControlFile "pass3 컨트롤 파일을 사용합니다.: %s\n"
 #define MSGTR_MissingFilename "\n파일이름이 없습니다.\n\n"
 #define MSGTR_CannotOpenFile_Device "파일/장치를 열 수 없습니다.\n"
 #define MSGTR_CannotOpenDemuxer "해석기를 열 수 없습니다.\n"
@@ -153,15 +152,12 @@ static char help_text[]=
 #define MSGTR_CannotOpenOutputFile "출력 파일 '%s'을(를) 열 수 없습니다.\n"
 #define MSGTR_EncoderOpenFailed "인코더 열기에 실패했습니다.\n"
 #define MSGTR_ForcingOutputFourcc "fourcc를 %x [%.4s](으)로 강제출력합니다.\n"
-#define MSGTR_WritingAVIHeader "AVI 해더를 쓰고 있습니다...\n"
 #define MSGTR_DuplicateFrames "\n%d 프레임(들)이 중복되었습니다!\n"
 #define MSGTR_SkipFrame "\n프레임을 건너 뜁니다!\n"
 #define MSGTR_ErrorWritingFile "%s: 파일 쓰기 오류가 발생했습니다.\n"
-#define MSGTR_WritingAVIIndex "\nAVI 인덱스를 쓰고 있습니다...\n"
-#define MSGTR_FixupAVIHeader "AVI 해더를 고치고 있습니다...\n"
 #define MSGTR_RecommendedVideoBitrate "%s CD용으로 추천할 만한 비디오 주사율: %d\n"
-#define MSGTR_VideoStreamResult "\n비디오 스트림: %8.3f kbit/s  (%d B/s)  크기: %d 바이트  %5.3f 초  %d 프레임\n"
-#define MSGTR_AudioStreamResult "\n오디오 스트림: %8.3f kbit/s  (%d B/s)  크기: %d 바이트  %5.3f 초\n"
+#define MSGTR_VideoStreamResult "\n비디오 스트림: %8.3f kbit/s  (%d B/s)  크기: %"PRIu64" 바이트  %5.3f 초  %d 프레임\n"
+#define MSGTR_AudioStreamResult "\n오디오 스트림: %8.3f kbit/s  (%d B/s)  크기: %"PRIu64" 바이트  %5.3f 초\n"
 
 // cfg-mencoder.h:
 
@@ -225,8 +221,7 @@ static char help_text[]=
 #define MSGTR_SMBFileNotFound "lan으로 부터 열 수 없습니다.: '%s'\n"
 #define MSGTR_SMBNotCompiled "MPlayer가 SMB읽기를 할 수 있도록 컴파일되지 않았습니다.\n"
 
-#define MSGTR_CantOpenDVD "DVD 장치를 열 수 없습니다.: %s\n"
-#define MSGTR_DVDwait "디스크 구조를 읽고있습니다. 잠시만 기다려 주세요...\n"
+#define MSGTR_CantOpenDVD "DVD 장치를 열 수 없습니다.: %s (%s)\n"
 #define MSGTR_DVDnumTitles "이 DVD에는 %d개의 타이틀이 있습니다.\n"
 #define MSGTR_DVDinvalidTitle "유효하지 않은 DVD 타이틀 번호입니다.: %d\n"
 #define MSGTR_DVDnumChapters "이 DVD 타이틀에는 %d개의 챕터가 있습니다.\n"
@@ -235,7 +230,6 @@ static char help_text[]=
 #define MSGTR_DVDinvalidAngle "유효하지 않은 DVD 앵글 번호입니다.: %d\n"
 #define MSGTR_DVDnoIFO "DVD 타이틀 %d를 위한 IFO파일을 열 수 없습니다.\n"
 #define MSGTR_DVDnoVOBs "타이틀 VOBS (VTS_%02d_1.VOB)를 열 수 없습니다.\n"
-#define MSGTR_DVDopenOk "성공적으로 DVD를 열었습니다.\n"
 
 // demuxer.c, demux_*.c:
 #define MSGTR_AudioStreamRedefined "경고: 오디오 스트림 헤더 %d가 재정의되었습니다.\n"
@@ -266,8 +260,6 @@ static char help_text[]=
 #define MSGTR_CantSeekRawAVI "raw AVI 스트림에서는 탐색할 수 없습니다. (인덱스가 필요합니다. -idx 스위치로 시도해보세요.)  \n"
 #define MSGTR_CantSeekFile "이 파일에서는 탐색할 수 없습니다.\n"
 
-#define MSGTR_EncryptedVOB "암호화된 VOB 파일입니다! DOCS/HTML/en/dvd.html을 참조하세요.\n"
-
 #define MSGTR_MOVcomprhdr "MOV: 압축된 헤더는 (아직) 지원되지않습니다.\n"
 #define MSGTR_MOVvariableFourCC "MOV: 경고: 가변적인 FOURCC 발견!?\n"
 #define MSGTR_MOVtooManyTrk "MOV: 경고: 트랙이 너무 많습니다."
@@ -281,7 +273,6 @@ static char help_text[]=
 #define MSGTR_OpeningAudioDemuxerFailed "오디오 해석기를 여는데 실패했습니다.: %s\n"
 #define MSGTR_OpeningSubtitlesDemuxerFailed "자막 해석기를 여는데 실패했습니다.: %s\n"
 #define MSGTR_TVInputNotSeekable "TV 입력을 찾을 수 없습니다! (채널을 바꾸고 하면 될수도 있습니다. ;)\n"
-#define MSGTR_DemuxerInfoAlreadyPresent "해석기 정보 %s(이)가 이미 존재합니다!\n"
 #define MSGTR_ClipInfo "클립 정보: \n"
 
 #define MSGTR_LeaveTelecineMode "\ndemux_mpg: 30fps NTSC 항목을 감지하여, 프레임속도를 바꿉니다.\n"
@@ -323,7 +314,6 @@ static char help_text[]=
 
 // LIRC:
 #define MSGTR_SettingUpLIRC "LIRC 지원을 시작합니다...\n"
-#define MSGTR_LIRCdisabled "리모콘을 사용할 수 없습니다.\n"
 #define MSGTR_LIRCopenfailed "LIRC 지원 시작을 실패했습니다.\n"
 #define MSGTR_LIRCcfgerr "LIRC 설정파일 %s를 읽는데 실패했습니다.\n"
 
@@ -346,7 +336,7 @@ static char help_text[]=
 
 // ====================== GUI messages/buttons ========================
 
-#ifdef HAVE_NEW_GUI
+#ifdef CONFIG_GUI
 
 // --- labels ---
 #define MSGTR_About "정보"
@@ -380,7 +370,6 @@ static char help_text[]=
 #define MSGTR_NEMDB "죄송합니다. 그리기 버퍼를 위한 충분한 메모리가 없습니다."
 #define MSGTR_NEMFMR "죄송합니다. 메뉴 렌더링을 위한 충분한 메모리가 없습니다."
 #define MSGTR_IDFGCVD "죄송합니다. GUI 호환 비디오 출력 드라이버를 찾지 못했습니다."
-#define MSGTR_NEEDLAVCFAME "죄송합니다. MPEG가 아닌 파일을 DXR3/H+ 장치로 재생하려면 다시 인코딩해야만 합니다.\nDXR3/H+ 설정 대화상자에서 lavc나 frame이 가능하도록 하세요."
 
 // --- skin loader error messages
 #define MSGTR_SKIN_ERRORMESSAGE "[스킨] 스킨 설정파일의 %d번째 줄에 오류가 있습니다.: %s" 
@@ -394,7 +383,7 @@ static char help_text[]=
 #define MSGTR_SKIN_BITMAP_PNGReadError "PNG 읽기 오류입니다. (%s)\n"
 #define MSGTR_SKIN_BITMAP_RLENotSupported "RLE로 압축된 TGA는 지원되지 않습니다. (%s)\n"
 #define MSGTR_SKIN_BITMAP_UnknownFileType "알 수 없는 파일 형식입니다. (%s)\n"
-#define MSGTR_SKIN_BITMAP_ConvertError "24 비트에서 32 비트로 전환 오류 (%s)\n"
+#define MSGTR_SKIN_BITMAP_ConversionError "24 비트에서 32 비트로 전환 오류 (%s)\n"
 #define MSGTR_SKIN_BITMAP_UnknownMessage "알 수 없는 메세지입니다.: %s\n"
 #define MSGTR_SKIN_FONT_NotEnoughtMemory "메모리가 부족합니다.\n"
 #define MSGTR_SKIN_FONT_TooManyFontsDeclared "폰트가 너무 많이 선언되어 있습니다.\n"
@@ -482,7 +471,7 @@ static char help_text[]=
 #define MSGTR_PREFERENCES_AvailableDrivers "가능한 드라이버:"
 #define MSGTR_PREFERENCES_DoNotPlaySound "사운드 재생 안함"
 #define MSGTR_PREFERENCES_NormalizeSound "사운드 표준화"
-#define MSGTR_PREFERENCES_EnEqualizer "이퀄라이저 사용"
+#define MSGTR_PREFERENCES_EnableEqualizer "이퀄라이저 사용"
 #define MSGTR_PREFERENCES_ExtraStereo "외부 스테레오 사용"
 #define MSGTR_PREFERENCES_Coefficient "계수:"
 #define MSGTR_PREFERENCES_AudioDelay "오디오 지연:"
@@ -521,7 +510,6 @@ static char help_text[]=
 #define MSGTR_PREFERENCES_Message "선택사항들을 적용하려면 재생기를 다시 시작해야 합니다!"
 #define MSGTR_PREFERENCES_DXR3_VENC "비디오 인코더:"
 #define MSGTR_PREFERENCES_DXR3_LAVC "LAVC 사용 (FFmpeg)"
-#define MSGTR_PREFERENCES_DXR3_FAME "FAME 사용"
 #define MSGTR_PREFERENCES_FontEncoding1 "유니코드"
 #define MSGTR_PREFERENCES_FontEncoding2 "서유럽어 (ISO-8859-1)"
 #define MSGTR_PREFERENCES_FontEncoding3 "Euro 포함 서유럽어 (ISO-8859-15)"
@@ -567,9 +555,6 @@ static char help_text[]=
 #define MSGTR_PREFERENCES_ShowVideoWindow "정지 중일 때 비디오 창 보이기"
 
 #define MSGTR_ABOUT_UHU "GUI 개발 지원: UHU Linux\n"
-#define MSGTR_ABOUT_CoreTeam "한글번역: DongCheon Park (동쪽하늘)\n          <dcpark@kaist.ac.kr>\n\n   MPlayer 코어 팀:\n"
-#define MSGTR_ABOUT_AdditionalCoders "   그 외의 코더:\n"
-#define MSGTR_ABOUT_MainTesters "   메인 테스터:\n"
 
 // --- messagebox
 #define MSGTR_MSGBOX_LABEL_FatalError "치명적 오류!"
