@@ -1,35 +1,52 @@
 /*
-  ao_alsa9/1.x - ALSA-0.9.x-1.x output plugin for MPlayer
-
-  (C) Alex Beregszaszi
-  
-  modified for real alsa-0.9.0-support by Zsolt Barat <joy@streamminister.de>
-  additional AC3 passthrough support by Andy Lo A Foe <andy@alsaplayer.org>  
-  08/22/2002 iec958-init rewritten and merged with common init, zsolt
-  04/13/2004 merged with ao_alsa1.x, fixes provided by Jindrich Makovicka
-  04/25/2004 printfs converted to mp_msg, Zsolt.
-  
-  Any bugreports regarding to this driver are welcome.
-*/
+ * ALSA 0.9.x-1.x audio output driver
+ *
+ * Copyright (C) 2004 Alex Beregszaszi
+ *
+ * modified for real ALSA 0.9.0 support by Zsolt Barat <joy@streamminister.de>
+ * additional AC-3 passthrough support by Andy Lo A Foe <andy@alsaplayer.org>
+ * 08/22/2002 iec958-init rewritten and merged with common init, zsolt
+ * 04/13/2004 merged with ao_alsa1.x, fixes provided by Jindrich Makovicka
+ * 04/25/2004 printfs converted to mp_msg, Zsolt.
+ *
+ * This file is part of MPlayer.
+ *
+ * MPlayer is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * MPlayer is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with MPlayer; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #include <errno.h>
 #include <sys/time.h>
 #include <stdlib.h>
 #include <stdarg.h>
+#include <ctype.h>
 #include <math.h>
 #include <string.h>
+#include <alloca.h>
 
 #include "config.h"
 #include "subopt-helper.h"
 #include "mixer.h"
 #include "mp_msg.h"
+#include "help_mp.h"
 
 #define ALSA_PCM_NEW_HW_PARAMS_API
 #define ALSA_PCM_NEW_SW_PARAMS_API
 
-#if HAVE_SYS_ASOUNDLIB_H
+#ifdef HAVE_SYS_ASOUNDLIB_H
 #include <sys/asoundlib.h>
-#elif HAVE_ALSA_ASOUNDLIB_H
+#elif defined(HAVE_ALSA_ASOUNDLIB_H)
 #include <alsa/asoundlib.h>
 #else
 #error "asoundlib.h is not in sys/ or alsa/ - please bugreport"
@@ -85,10 +102,10 @@ static void alsa_error_handler(const char *file, int line, const char *function,
   tmp[sizeof tmp - 1] = '\0';
 
   if (err)
-    mp_msg(MSGT_AO, MSGL_ERR, "alsa-lib: %s:%i:(%s) %s: %s\n",
+    mp_msg(MSGT_AO, MSGL_ERR, "[AO_ALSA] alsa-lib: %s:%i:(%s) %s: %s\n",
 	   file, line, function, tmp, snd_strerror(err));
   else
-    mp_msg(MSGT_AO, MSGL_ERR, "alsa-lib: %s:%i:(%s) %s\n",
+    mp_msg(MSGT_AO, MSGL_ERR, "[AO_ALSA] alsa-lib: %s:%i:(%s) %s\n",
 	   file, line, function, tmp);
 }
 
@@ -98,7 +115,6 @@ static int control(int cmd, void *arg)
   switch(cmd) {
   case AOCONTROL_QUERY_FORMAT:
     return CONTROL_TRUE;
-#ifndef WORDS_BIGENDIAN 
   case AOCONTROL_GET_VOLUME:
   case AOCONTROL_SET_VOLUME:
     {
@@ -117,6 +133,9 @@ static int control(int cmd, void *arg)
       long get_vol, set_vol;
       float f_multi;
 
+      if(ao_data.format == AF_FORMAT_AC3)
+	return CONTROL_TRUE;
+
       if(mixer_channel) {
 	 char *test_mix_index;
 
@@ -128,16 +147,13 @@ static int control(int cmd, void *arg)
 
 		if (*test_mix_index){
 		  mp_msg(MSGT_AO,MSGL_ERR,
-		    "alsa-control: invalid mixer index. Defaulting to 0\n");
+		    MSGTR_AO_ALSA_InvalidMixerIndexDefaultingToZero);
 		  mix_index = 0 ;
 		}
 	 }
       }
       if(mixer_device) card = mixer_device;
 
-      if(ao_data.format == AF_FORMAT_AC3)
-	return CONTROL_TRUE;
-
       //allocate simple id
       snd_mixer_selem_id_alloca(&sid);
 	
@@ -151,32 +167,32 @@ static int control(int cmd, void *arg)
       }
 
       if ((err = snd_mixer_open(&handle, 0)) < 0) {
-	mp_msg(MSGT_AO,MSGL_ERR,"alsa-control: mixer open error: %s\n", snd_strerror(err));
+	mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_MixerOpenError, snd_strerror(err));
 	return CONTROL_ERROR;
       }
 
       if ((err = snd_mixer_attach(handle, card)) < 0) {
-	mp_msg(MSGT_AO,MSGL_ERR,"alsa-control: mixer attach %s error: %s\n", 
+	mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_MixerAttachError, 
 	       card, snd_strerror(err));
 	snd_mixer_close(handle);
 	return CONTROL_ERROR;
       }
 
       if ((err = snd_mixer_selem_register(handle, NULL, NULL)) < 0) {
-	mp_msg(MSGT_AO,MSGL_ERR,"alsa-control: mixer register error: %s\n", snd_strerror(err));
+	mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_MixerRegisterError, snd_strerror(err));
 	snd_mixer_close(handle);
 	return CONTROL_ERROR;
       }
       err = snd_mixer_load(handle);
       if (err < 0) {
-	mp_msg(MSGT_AO,MSGL_ERR,"alsa-control: mixer load error: %s\n", snd_strerror(err));
+	mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_MixerLoadError, snd_strerror(err));
 	snd_mixer_close(handle);
 	return CONTROL_ERROR;
       }
 
       elem = snd_mixer_find_selem(handle, sid);
       if (!elem) {
-	mp_msg(MSGT_AO,MSGL_ERR,"alsa-control: unable to find simple control '%s',%i\n",
+	mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_UnableToFindSimpleControl,
 	       snd_mixer_selem_id_get_name(sid), snd_mixer_selem_id_get_index(sid));
 	snd_mixer_close(handle);
 	return CONTROL_ERROR;
@@ -190,8 +206,8 @@ static int control(int cmd, void *arg)
 	set_vol = vol->left / f_multi + pmin + 0.5;
 
 	//setting channels
-	if ((err = snd_mixer_selem_set_playback_volume(elem, 0, set_vol)) < 0) {
-	  mp_msg(MSGT_AO,MSGL_ERR,"alsa-control: error setting left channel, %s\n", 
+	if ((err = snd_mixer_selem_set_playback_volume(elem, SND_MIXER_SCHN_FRONT_LEFT, set_vol)) < 0) {
+	  mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_ErrorSettingLeftChannel, 
 		 snd_strerror(err));
 	  return CONTROL_ERROR;
 	}
@@ -199,8 +215,8 @@ static int control(int cmd, void *arg)
 
 	set_vol = vol->right / f_multi + pmin + 0.5;
 
-	if ((err = snd_mixer_selem_set_playback_volume(elem, 1, set_vol)) < 0) {
-	  mp_msg(MSGT_AO,MSGL_ERR,"alsa-control: error setting right channel, %s\n", 
+	if ((err = snd_mixer_selem_set_playback_volume(elem, SND_MIXER_SCHN_FRONT_RIGHT, set_vol)) < 0) {
+	  mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_ErrorSettingRightChannel, 
 		 snd_strerror(err));
 	  return CONTROL_ERROR;
 	}
@@ -219,9 +235,9 @@ static int control(int cmd, void *arg)
 	}
       }
       else {
-	snd_mixer_selem_get_playback_volume(elem, 0, &get_vol);
+	snd_mixer_selem_get_playback_volume(elem, SND_MIXER_SCHN_FRONT_LEFT, &get_vol);
 	vol->left = (get_vol - pmin) * f_multi;
-	snd_mixer_selem_get_playback_volume(elem, 1, &get_vol);
+	snd_mixer_selem_get_playback_volume(elem, SND_MIXER_SCHN_FRONT_RIGHT, &get_vol);
 	vol->right = (get_vol - pmin) * f_multi;
 
 	mp_msg(MSGT_AO,MSGL_DBG2,"left=%f, right=%f\n",vol->left,vol->right);
@@ -229,10 +245,9 @@ static int control(int cmd, void *arg)
       snd_mixer_close(handle);
       return CONTROL_OK;
     }
-#endif
     
   } //end switch
-  return(CONTROL_UNKNOWN);
+  return CONTROL_UNKNOWN;
 }
 
 static void parse_device (char *dest, const char *src, int len)
@@ -249,14 +264,7 @@ static void parse_device (char *dest, const char *src, int len)
 static void print_help (void)
 {
   mp_msg (MSGT_AO, MSGL_FATAL,
-           "\n-ao alsa commandline help:\n"
-           "Example: mplayer -ao alsa:device=hw=0.3\n"
-           "  sets first card fourth hardware device\n"
-           "\nOptions:\n"
-           "  noblock\n"
-           "    Opens device in non-blocking mode\n"
-           "  device=<device-name>\n"
-           "    Sets device (change , to . and : to =)\n");
+           MSGTR_AO_ALSA_CommandlineHelp);
 }
 
 static int str_maxlen(strarg_t *str) {
@@ -265,6 +273,51 @@ static int str_maxlen(strarg_t *str) {
   return 1;
 }
 
+static int try_open_device(const char *device, int open_mode, int try_ac3)
+{
+  int err, len;
+  char *ac3_device, *args;
+
+  if (try_ac3) {
+    /* to set the non-audio bit, use AES0=6 */
+    len = strlen(device);
+    ac3_device = malloc(len + 7 + 1);
+    if (!ac3_device)
+      return -ENOMEM;
+    strcpy(ac3_device, device);
+    args = strchr(ac3_device, ':');
+    if (!args) {
+      /* no existing parameters: add it behind device name */
+      strcat(ac3_device, ":AES0=6");
+    } else {
+      do
+	++args;
+      while (isspace(*args));
+      if (*args == '\0') {
+	/* ":" but no parameters */
+	strcat(ac3_device, "AES0=6");
+      } else if (*args != '{') {
+	/* a simple list of parameters: add it at the end of the list */
+	strcat(ac3_device, ",AES0=6");
+      } else {
+	/* parameters in config syntax: add it inside the { } block */
+	do
+	  --len;
+	while (len > 0 && isspace(ac3_device[len]));
+	if (ac3_device[len] == '}')
+	  strcpy(ac3_device + len, " AES0=6}");
+      }
+    }
+    err = snd_pcm_open(&alsa_handler, ac3_device, SND_PCM_STREAM_PLAYBACK,
+		       open_mode);
+    free(ac3_device);
+  }
+  if (!try_ac3 || err < 0)
+    err = snd_pcm_open(&alsa_handler, device, SND_PCM_STREAM_PLAYBACK,
+		       open_mode);
+  return err;
+}
+
 /*
     open & setup audio device
     return: 1=success 0=fail
@@ -272,7 +325,6 @@ static int str_maxlen(strarg_t *str) {
 static int init(int rate_hz, int channels, int format, int flags)
 {
     int err;
-    int cards = -1;
     int block;
     strarg_t device;
     snd_pcm_uframes_t bufsize;
@@ -298,12 +350,6 @@ static int init(int rate_hz, int channels, int format, int flags)
 
     snd_lib_error_set_handler(alsa_error_handler);
     
-    if ((err = snd_card_next(&cards)) < 0 || cards < 0)
-    {
-      mp_msg(MSGT_AO,MSGL_ERR,"alsa-init: no soundcards found: %s\n", snd_strerror(err));
-      return(0);
-    }
-
     ao_data.samplerate = rate_hz;
     ao_data.format = format;
     ao_data.channels = channels;
@@ -374,20 +420,7 @@ static int init(int rate_hz, int channels, int format, int flags)
      * 'iec958'
      */
     if (format == AF_FORMAT_AC3) {
-      unsigned char s[4];
-
-	s[0] = IEC958_AES0_NONAUDIO | 
-	  IEC958_AES0_CON_EMPHASIS_NONE;
-	s[1] = IEC958_AES1_CON_ORIGINAL | 
-	  IEC958_AES1_CON_PCM_CODER;
-	s[2] = 0;
-	s[3] = IEC958_AES3_CON_FS_48000;
-
-	snprintf(alsa_device, ALSA_DEVICE_SIZE,
-		"iec958:{CARD 0 AES0 0x%02x AES1 0x%02x AES2 0x%02x AES3 0x%02x}", 
- 		s[0], s[1], s[2], s[3]);
-	device.str = alsa_device;
-
+	device.str = "iec958";
 	mp_msg(MSGT_AO,MSGL_V,"alsa-spdif-init: playing AC3, %i channels\n", channels);
     }
   else
@@ -417,7 +450,7 @@ static int init(int rate_hz, int channels, int format, int flags)
 	  break;
 	default:
 	  device.str = "default";
-	  mp_msg(MSGT_AO,MSGL_ERR,"alsa-init: %d channels are not supported\n",channels);
+	  mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_ChannelsNotSupported,channels);
         }
     device.len = strlen(device.str);
     if (subopt_parse(ao_subdevice, subopts) != 0) {
@@ -427,7 +460,7 @@ static int init(int rate_hz, int channels, int format, int flags)
     ao_noblock = !block;
     parse_device(alsa_device, device.str, device.len);
 
-	mp_msg(MSGT_AO,MSGL_INFO,"alsa-init: %d soundcard%s found, using: %s\n", cards+1,(cards >= 0) ? "" : "s", alsa_device);
+    mp_msg(MSGT_AO,MSGL_V,"alsa-init: using device %s\n", alsa_device);
 
     //setting modes for block or nonblock-mode
     if (ao_noblock) {
@@ -474,24 +507,24 @@ static int init(int rate_hz, int channels, int format, int flags)
 
     if (!alsa_handler) {
       //modes = 0, SND_PCM_NONBLOCK, SND_PCM_ASYNC
-      if ((err = snd_pcm_open(&alsa_handler, alsa_device, SND_PCM_STREAM_PLAYBACK, open_mode)) < 0)
+      if ((err = try_open_device(alsa_device, open_mode, format == AF_FORMAT_AC3)) < 0)
 	{
 	  if (err != -EBUSY && ao_noblock) {
-	    mp_msg(MSGT_AO,MSGL_INFO,"alsa-init: open in nonblock-mode failed, trying to open in block-mode\n");
-	    if ((err = snd_pcm_open(&alsa_handler, alsa_device, SND_PCM_STREAM_PLAYBACK, 0)) < 0) {
-	      mp_msg(MSGT_AO,MSGL_ERR,"alsa-init: playback open error: %s\n", snd_strerror(err));
-	      return(0);
+	    mp_msg(MSGT_AO,MSGL_INFO,MSGTR_AO_ALSA_OpenInNonblockModeFailed);
+	    if ((err = try_open_device(alsa_device, 0, format == AF_FORMAT_AC3)) < 0) {
+	      mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_PlaybackOpenError, snd_strerror(err));
+	      return 0;
 	    }
 	  } else {
-	    mp_msg(MSGT_AO,MSGL_ERR,"alsa-init: playback open error: %s\n", snd_strerror(err));
-	    return(0);
+	    mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_PlaybackOpenError, snd_strerror(err));
+	    return 0;
 	  }
 	}
 
       if ((err = snd_pcm_nonblock(alsa_handler, 0)) < 0) {
-         mp_msg(MSGT_AO,MSGL_ERR,"alsa-init: error set block-mode %s\n", snd_strerror(err));
+         mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_ErrorSetBlockMode, snd_strerror(err));
       } else {
-	mp_msg(MSGT_AO,MSGL_V,"alsa-init: pcm opend in blocking mode\n");
+	mp_msg(MSGT_AO,MSGL_V,"alsa-init: pcm opened in blocking mode\n");
       }
 
       snd_pcm_hw_params_alloca(&alsa_hwparams);
@@ -500,17 +533,17 @@ static int init(int rate_hz, int channels, int format, int flags)
       // setting hw-parameters
       if ((err = snd_pcm_hw_params_any(alsa_handler, alsa_hwparams)) < 0)
 	{
-	  mp_msg(MSGT_AO,MSGL_ERR,"alsa-init: unable to get initial parameters: %s\n",
+	  mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_UnableToGetInitialParameters,
 		 snd_strerror(err));
-	  return(0);
+	  return 0;
 	}
     
       err = snd_pcm_hw_params_set_access(alsa_handler, alsa_hwparams,
 					 SND_PCM_ACCESS_RW_INTERLEAVED);
       if (err < 0) {
-	mp_msg(MSGT_AO,MSGL_ERR,"alsa-init: unable to set access type: %s\n", 
+	mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_UnableToSetAccessType, 
 	       snd_strerror(err));
-	return (0);
+	return 0;
       }
 
       /* workaround for nonsupported formats
@@ -519,7 +552,7 @@ static int init(int rate_hz, int channels, int format, int flags)
                                              alsa_format)) < 0)
       {
          mp_msg(MSGT_AO,MSGL_INFO,
-		"alsa-init: format %s are not supported by hardware, trying default\n", af_fmt2str_short(format));
+		MSGTR_AO_ALSA_FormatNotSupportedByHardware, af_fmt2str_short(format));
          alsa_format = SND_PCM_FORMAT_S16_LE;
          ao_data.format = AF_FORMAT_S16_LE;
       }
@@ -527,25 +560,37 @@ static int init(int rate_hz, int channels, int format, int flags)
       if ((err = snd_pcm_hw_params_set_format(alsa_handler, alsa_hwparams,
 					      alsa_format)) < 0)
 	{
-	  mp_msg(MSGT_AO,MSGL_ERR,"alsa-init: unable to set format: %s\n",
+	  mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_UnableToSetFormat,
 		 snd_strerror(err));
-	  return(0);
+	  return 0;
 	}
 
       if ((err = snd_pcm_hw_params_set_channels_near(alsa_handler, alsa_hwparams,
 						     &ao_data.channels)) < 0)
 	{
-	  mp_msg(MSGT_AO,MSGL_ERR,"alsa-init: unable to set channels: %s\n",
+	  mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_UnableToSetChannels,
 		 snd_strerror(err));
-	  return(0);
+	  return 0;
 	}
 
+      /* workaround for buggy rate plugin (should be fixed in ALSA 1.0.11)
+         prefer our own resampler */
+#if SND_LIB_VERSION >= 0x010009
+      if ((err = snd_pcm_hw_params_set_rate_resample(alsa_handler, alsa_hwparams,
+						     0)) < 0)
+	{
+	  mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_UnableToDisableResampling,
+		 snd_strerror(err));
+	  return 0;
+	}
+#endif
+
       if ((err = snd_pcm_hw_params_set_rate_near(alsa_handler, alsa_hwparams, 
 						 &ao_data.samplerate, NULL)) < 0) 
         {
-	  mp_msg(MSGT_AO,MSGL_ERR,"alsa-init: unable to set samplerate-2: %s\n",
+	  mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_UnableToSetSamplerate2,
 		 snd_strerror(err));
-	  return(0);
+	  return 0;
         }
 
       bytes_per_sample = snd_pcm_format_physical_width(alsa_format) / 8;
@@ -560,9 +605,9 @@ static int init(int rate_hz, int channels, int format, int flags)
 	if ((err = snd_pcm_hw_params_set_buffer_time_near(alsa_handler, alsa_hwparams, 
 							  &alsa_buffer_time, NULL)) < 0)
 	  {
-	    mp_msg(MSGT_AO,MSGL_ERR,"alsa-init: unable to set buffer time near: %s\n",
+	    mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_UnableToSetBufferTimeNear,
 		   snd_strerror(err));
-	    return(0);
+	    return 0;
 	  } else
 	    alsa_buffer_time = err;
 
@@ -570,10 +615,11 @@ static int init(int rate_hz, int channels, int format, int flags)
 							  &alsa_period_time, NULL)) < 0)
 	  /* original: alsa_buffer_time/ao_data.bps */
 	  {
-	    mp_msg(MSGT_AO,MSGL_ERR,"alsa-init: unable to set period time: %s\n",
+	    mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_UnableToSetPeriodTime,
 		   snd_strerror(err));
+	    return 0;
 	  }
-	mp_msg(MSGT_AO,MSGL_INFO,"alsa-init: buffer_time: %d, period_time :%d\n",
+	mp_msg(MSGT_AO,MSGL_INFO,MSGTR_AO_ALSA_BufferTimePeriodTime,
 	       alsa_buffer_time, err);
       } 
 #endif//end SET_BUFFERTIME
@@ -584,16 +630,18 @@ static int init(int rate_hz, int channels, int format, int flags)
 	if ((err = snd_pcm_hw_params_set_period_size_near(alsa_handler, alsa_hwparams, 
 							  &chunk_size, NULL)) < 0)
 	  {
-	    mp_msg(MSGT_AO,MSGL_ERR,"alsa-init: unable to set periodsize(%ld): %s\n",
+	    mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_UnableToSetPeriodSize,
 			    chunk_size, snd_strerror(err));
+	    return 0;
 	  }
 	else {
 	  mp_msg(MSGT_AO,MSGL_V,"alsa-init: chunksize set to %li\n", chunk_size);
 	}
 	if ((err = snd_pcm_hw_params_set_periods_near(alsa_handler, alsa_hwparams,
 						      &alsa_fragcount, NULL)) < 0) {
-	  mp_msg(MSGT_AO,MSGL_ERR,"alsa-init: unable to set periods: %s\n", 
+	  mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_UnableToSetPeriods, 
 		 snd_strerror(err));
+	  return 0;
 	}
 	else {
 	  mp_msg(MSGT_AO,MSGL_V,"alsa-init: fragcount=%i\n", alsa_fragcount);
@@ -604,8 +652,9 @@ static int init(int rate_hz, int channels, int format, int flags)
       /* finally install hardware parameters */
       if ((err = snd_pcm_hw_params(alsa_handler, alsa_hwparams)) < 0)
 	{
-	  mp_msg(MSGT_AO,MSGL_ERR,"alsa-init: unable to set hw-parameters: %s\n",
+	  mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_UnableToSetHwParameters,
 		 snd_strerror(err));
+	  return 0;
 	}
       // end setting hw-params
 
@@ -613,7 +662,8 @@ static int init(int rate_hz, int channels, int format, int flags)
       // gets buffersize for control
       if ((err = snd_pcm_hw_params_get_buffer_size(alsa_hwparams, &bufsize)) < 0)
 	{
-	  mp_msg(MSGT_AO,MSGL_ERR,"alsa-init: unable to get buffersize: %s\n", snd_strerror(err));
+	  mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_UnableToGetBufferSize, snd_strerror(err));
+	  return 0;
 	}
       else {
 	ao_data.buffersize = bufsize * bytes_per_sample;
@@ -621,7 +671,8 @@ static int init(int rate_hz, int channels, int format, int flags)
       }
 
       if ((err = snd_pcm_hw_params_get_period_size(alsa_hwparams, &chunk_size, NULL)) < 0) {
-	mp_msg(MSGT_AO,MSGL_ERR,"alsa-init: unable to get period size: %s\n", snd_strerror(err));
+	mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_UnableToGetPeriodSize, snd_strerror(err));
+	return 0;
       } else {
 	mp_msg(MSGT_AO,MSGL_V,"alsa-init: got period size %li\n", chunk_size);
       }
@@ -629,47 +680,53 @@ static int init(int rate_hz, int channels, int format, int flags)
 
       /* setting software parameters */
       if ((err = snd_pcm_sw_params_current(alsa_handler, alsa_swparams)) < 0) {
-	mp_msg(MSGT_AO,MSGL_ERR,"alsa-init: unable to get sw-parameters: %s\n",
+	mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_UnableToGetSwParameters,
 	       snd_strerror(err));
 	return 0;
       }
+#if SND_LIB_VERSION >= 0x000901
       if ((err = snd_pcm_sw_params_get_boundary(alsa_swparams, &boundary)) < 0) {
-	mp_msg(MSGT_AO,MSGL_ERR,"alsa-init: unable to get boundary: %s\n",
+	mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_UnableToGetBoundary,
 	       snd_strerror(err));
 	return 0;
       }
+#else
+      boundary = 0x7fffffff;
+#endif
       /* start playing when one period has been written */
       if ((err = snd_pcm_sw_params_set_start_threshold(alsa_handler, alsa_swparams, chunk_size)) < 0) {
-	mp_msg(MSGT_AO,MSGL_ERR,"alsa-init: unable to set start threshold: %s\n",
+	mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_UnableToSetStartThreshold,
 	       snd_strerror(err));
 	return 0;
       }
       /* disable underrun reporting */
       if ((err = snd_pcm_sw_params_set_stop_threshold(alsa_handler, alsa_swparams, boundary)) < 0) {
-	mp_msg(MSGT_AO,MSGL_ERR,"alsa-init: unable to set stop threshold: %s\n",
+	mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_UnableToSetStopThreshold,
 	       snd_strerror(err));
 	return 0;
       }
+#if SND_LIB_VERSION >= 0x000901
       /* play silence when there is an underrun */
       if ((err = snd_pcm_sw_params_set_silence_size(alsa_handler, alsa_swparams, boundary)) < 0) {
-	mp_msg(MSGT_AO,MSGL_ERR,"alsa-init: unable to set silence size: %s\n",
+	mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_UnableToSetSilenceSize,
 	       snd_strerror(err));
 	return 0;
       }
+#endif
       if ((err = snd_pcm_sw_params(alsa_handler, alsa_swparams)) < 0) {
-	mp_msg(MSGT_AO,MSGL_ERR,"alsa-init: unable to set sw-parameters: %s\n",
+	mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_UnableToGetSwParameters,
 	       snd_strerror(err));
 	return 0;
       }
       /* end setting sw-params */
 
-      mp_msg(MSGT_AO,MSGL_INFO,"alsa: %d Hz/%d channels/%d bpf/%d bytes buffer/%s\n",
-	     ao_data.samplerate, ao_data.channels, bytes_per_sample, ao_data.buffersize,
+      mp_msg(MSGT_AO,MSGL_V,"alsa: %d Hz/%d channels/%d bpf/%d bytes buffer/%s\n",
+	     ao_data.samplerate, ao_data.channels, (int)bytes_per_sample, ao_data.buffersize,
 	     snd_pcm_format_description(alsa_format));
 
     } // end switch alsa_handler (spdif)
     alsa_can_pause = snd_pcm_hw_params_can_pause(alsa_hwparams);
-    return(1);
+    return 1;
 } // end init
 
 
@@ -685,16 +742,16 @@ static void uninit(int immed)
 
     if ((err = snd_pcm_close(alsa_handler)) < 0)
       {
-	mp_msg(MSGT_AO,MSGL_ERR,"alsa-uninit: pcm close error: %s\n", snd_strerror(err));
+	mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_PcmCloseError, snd_strerror(err));
 	return;
       }
     else {
       alsa_handler = NULL;
-      mp_msg(MSGT_AO,MSGL_INFO,"alsa-uninit: pcm closed\n");
+      mp_msg(MSGT_AO,MSGL_V,"alsa-uninit: pcm closed\n");
     }
   }
   else {
-    mp_msg(MSGT_AO,MSGL_ERR,"alsa-uninit: no handler defined!\n");
+    mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_NoHandlerDefined);
   }
 }
 
@@ -705,14 +762,14 @@ static void audio_pause(void)
     if (alsa_can_pause) {
         if ((err = snd_pcm_pause(alsa_handler, 1)) < 0)
         {
-            mp_msg(MSGT_AO,MSGL_ERR,"alsa-pause: pcm pause error: %s\n", snd_strerror(err));
+            mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_PcmPauseError, snd_strerror(err));
             return;
         }
           mp_msg(MSGT_AO,MSGL_V,"alsa-pause: pause supported by hardware\n");
     } else {
         if ((err = snd_pcm_drop(alsa_handler)) < 0)
         {
-            mp_msg(MSGT_AO,MSGL_ERR,"alsa-pause: pcm drop error: %s\n", snd_strerror(err));
+            mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_PcmDropError, snd_strerror(err));
             return;
         }
     }
@@ -722,17 +779,21 @@ static void audio_resume(void)
 {
     int err;
 
+    if (snd_pcm_state(alsa_handler) == SND_PCM_STATE_SUSPENDED) {
+        mp_msg(MSGT_AO,MSGL_INFO,MSGTR_AO_ALSA_PcmInSuspendModeTryingResume);
+        while ((err = snd_pcm_resume(alsa_handler)) == -EAGAIN) sleep(1);
+    }
     if (alsa_can_pause) {
         if ((err = snd_pcm_pause(alsa_handler, 0)) < 0)
         {
-            mp_msg(MSGT_AO,MSGL_ERR,"alsa-resume: pcm resume error: %s\n", snd_strerror(err));
+            mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_PcmResumeError, snd_strerror(err));
             return;
         }
           mp_msg(MSGT_AO,MSGL_V,"alsa-resume: resume supported by hardware\n");
     } else {
         if ((err = snd_pcm_prepare(alsa_handler)) < 0)
         {
-           mp_msg(MSGT_AO,MSGL_ERR,"alsa-resume: pcm prepare error: %s\n", snd_strerror(err));
+           mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_PcmPrepareError, snd_strerror(err));
             return;
         }
     }
@@ -745,12 +806,12 @@ static void reset(void)
 
     if ((err = snd_pcm_drop(alsa_handler)) < 0)
     {
-	mp_msg(MSGT_AO,MSGL_ERR,"alsa-reset: pcm drop error: %s\n", snd_strerror(err));
+	mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_PcmPrepareError, snd_strerror(err));
 	return;
     }
     if ((err = snd_pcm_prepare(alsa_handler)) < 0)
     {
-	mp_msg(MSGT_AO,MSGL_ERR,"alsa-reset: pcm prepare error: %s\n", snd_strerror(err));
+	mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_PcmPrepareError, snd_strerror(err));
 	return;
     }
     return;
@@ -771,7 +832,7 @@ static int play(void* data, int len, int flags)
   //mp_msg(MSGT_AO,MSGL_ERR,"alsa-play: frames=%i, len=%i\n",num_frames,len);
 
   if (!alsa_handler) {
-    mp_msg(MSGT_AO,MSGL_ERR,"alsa-play: device configuration error");
+    mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_DeviceConfigurationError);
     return 0;
   }
 
@@ -786,16 +847,16 @@ static int play(void* data, int len, int flags)
 	res = 0;
       }
       else if (res == -ESTRPIPE) {	/* suspend */
-	mp_msg(MSGT_AO,MSGL_INFO,"alsa-play: pcm in suspend mode. trying to resume\n");
+	mp_msg(MSGT_AO,MSGL_INFO,MSGTR_AO_ALSA_PcmInSuspendModeTryingResume);
 	while ((res = snd_pcm_resume(alsa_handler)) == -EAGAIN)
 	  sleep(1);
       }
       if (res < 0) {
-	mp_msg(MSGT_AO,MSGL_ERR,"alsa-play: write error: %s\n", snd_strerror(res));
-	mp_msg(MSGT_AO,MSGL_INFO,"alsa-play: trying to reset soundcard\n");
+	mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_WriteError, snd_strerror(res));
+	mp_msg(MSGT_AO,MSGL_INFO,MSGTR_AO_ALSA_TryingToResetSoundcard);
 	if ((res = snd_pcm_prepare(alsa_handler)) < 0) {
-	  mp_msg(MSGT_AO,MSGL_ERR,"alsa-play: pcm prepare error: %s\n", snd_strerror(res));
-	  return(0);
+	  mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_PcmPrepareError, snd_strerror(res));
+	  return 0;
 	  break;
 	}
       }
@@ -814,14 +875,14 @@ static int get_space(void)
     
     if ((ret = snd_pcm_status(alsa_handler, status)) < 0)
     {
-	mp_msg(MSGT_AO,MSGL_ERR,"alsa-space: cannot get pcm status: %s\n", snd_strerror(ret));
-	return(0);
+	mp_msg(MSGT_AO,MSGL_ERR,MSGTR_AO_ALSA_CannotGetPcmStatus, snd_strerror(ret));
+	return 0;
     }
     
     ret = snd_pcm_status_get_avail(status) * bytes_per_sample;
-    if (ret > MAX_OUTBURST)
-	    ret = MAX_OUTBURST;
-    return(ret);
+    if (ret > ao_data.buffersize)  // Buffer underrun?
+	ret = ao_data.buffersize;
+    return ret;
 }
 
 /* delay in seconds between first and last sample in buffer */
@@ -842,6 +903,6 @@ static float get_delay(void)
     }
     return (float)delay / (float)ao_data.samplerate;
   } else {
-    return(0);
+    return 0;
   }
 }
