@@ -1,11 +1,24 @@
-/* 
- * ao_jack.c - libao2 JACK Audio Output Driver for MPlayer
+/*
+ * JACK audio output driver for MPlayer
  *
- * This driver is under the same license as MPlayer.
- * (http://www.mplayerhq.hu)
+ * Copyleft 2001 by Felix Bünemann (atmosfear@users.sf.net)
+ * and Reimar Döffinger (Reimar.Doeffinger@stud.uni-karlsruhe.de)
  *
- * Copyleft 2001 by Felix Bnemann (atmosfear@users.sf.net)
- * and Reimar Dffinger (Reimar.Doeffinger@stud.uni-karlsruhe.de)
+ * This file is part of MPlayer.
+ *
+ * MPlayer is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * MPlayer is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * along with MPlayer; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <stdio.h>
@@ -31,7 +44,7 @@ static ao_info_t info =
 {
   "JACK audio output",
   "jack",
-  "Reimar Dffinger <Reimar.Doeffinger@stud.uni-karlsruhe.de>",
+  "Reimar Döffinger <Reimar.Doeffinger@stud.uni-karlsruhe.de>",
   "based on ao_sdl.c"
 };
 
@@ -77,7 +90,7 @@ static volatile int write_pos;
  * return value may change between immediately following two calls,
  * and the real number of free bytes might be larger!
  */
-static int buf_free() {
+static int buf_free(void) {
   int free = read_pos - write_pos - CHUNK_SIZE;
   if (free < 0) free += BUFFSIZE;
   return free;
@@ -91,7 +104,7 @@ static int buf_free() {
  * return value may change between immediately following two calls,
  * and the real number of buffered bytes might be larger!
  */
-static int buf_used() {
+static int buf_used(void) {
   int used = write_pos - read_pos;
   if (used < 0) used += BUFFSIZE;
   return used;
@@ -111,15 +124,17 @@ static int write_buffer(unsigned char* data, int len) {
   if (len > free) len = free;
   if (first_len > len) first_len = len;
   // till end of buffer
-  memcpy (&buffer[write_pos], data, first_len);
+  fast_memcpy (&buffer[write_pos], data, first_len);
   if (len > first_len) { // we have to wrap around
     // remaining part from beginning of buffer
-    memcpy (buffer, &data[first_len], len - first_len);
+    fast_memcpy (buffer, &data[first_len], len - first_len);
   }
   write_pos = (write_pos + len) % BUFFSIZE;
   return len;
 }
 
+static void silence(float **bufs, int cnt, int num_bufs);
+
 /**
  * \brief read data from buffer and splitting it into channels
  * \param bufs num_bufs float buffers, each will contain the data of one channel
@@ -130,16 +145,19 @@ static int write_buffer(unsigned char* data, int len) {
  *
  * Assumes the data in the buffer is of type float, the number of bytes
  * read is res * num_bufs * sizeof(float), where res is the return value.
+ * If there is not enough data in the buffer remaining parts will be filled
+ * with silence.
  */
 static int read_buffer(float **bufs, int cnt, int num_bufs) {
-  int first_len = BUFFSIZE - read_pos;
   int buffered = buf_used();
   int i, j;
-  if (cnt * sizeof(float) * num_bufs > buffered)
+  if (cnt * sizeof(float) * num_bufs > buffered) {
+    silence(bufs, cnt, num_bufs);
     cnt = buffered / sizeof(float) / num_bufs;
+  }
   for (i = 0; i < cnt; i++) {
     for (j = 0; j < num_bufs; j++) {
-      bufs[j][i] = *((float *)(&buffer[read_pos]));
+      bufs[j][i] = *(float *)&buffer[read_pos];
       read_pos = (read_pos + sizeof(float)) % BUFFSIZE;
     }
   }
@@ -159,10 +177,9 @@ static int control(int cmd, void *arg) {
  * \param num_bufs number of buffers
  */
 static void silence(float **bufs, int cnt, int num_bufs) {
-  int i, j;
-  for (i = 0; i < cnt; i++)
-    for (j = 0; j < num_bufs; j++)
-      bufs[j][i] = 0;
+  int i;
+  for (i = 0; i < num_bufs; i++)
+    memset(bufs[i], 0, cnt * sizeof(float));
 }
 
 /**
@@ -178,11 +195,11 @@ static int outputaudio(jack_nframes_t nframes, void *arg) {
   int i;
   for (i = 0; i < num_ports; i++)
     bufs[i] = jack_port_get_buffer(ports[i], nframes);
-  if (!paused && !underrun)
-    if (read_buffer(bufs, nframes, num_ports) < nframes)
-      underrun = 1;
   if (paused || underrun)
     silence(bufs, nframes, num_ports);
+  else
+    if (read_buffer(bufs, nframes, num_ports) < nframes)
+      underrun = 1;
   if (estimate) {
     float now = (float)GetTimer() / 1000000.0;
     float diff = callback_time + callback_interval - now;
@@ -198,7 +215,7 @@ static int outputaudio(jack_nframes_t nframes, void *arg) {
 /**
  * \brief print suboption usage help
  */
-static void print_help ()
+static void print_help (void)
 {
   mp_msg (MSGT_AO, MSGL_FATAL,
            "\n-ao jack commandline help:\n"
@@ -210,19 +227,25 @@ static void print_help ()
            "  name=<client name>\n"
            "    Client name to pass to JACK\n"
            "  estimate\n"
-           "    Estimates the amount of data in buffers (experimental)\n");
+           "    Estimates the amount of data in buffers (experimental)\n"
+           "  autostart\n"
+           "    Automatically start JACK server if necessary\n"
+         );
 }
 
 static int init(int rate, int channels, int format, int flags) {
   const char **matching_ports = NULL;
   char *port_name = NULL;
   char *client_name = NULL;
+  int autostart = 0;
   opt_t subopts[] = {
     {"port", OPT_ARG_MSTRZ, &port_name, NULL},
     {"name", OPT_ARG_MSTRZ, &client_name, NULL},
     {"estimate", OPT_ARG_BOOL, &estimate, NULL},
+    {"autostart", OPT_ARG_BOOL, &autostart, NULL},
     {NULL}
   };
+  jack_options_t open_options = JackUseExactName;
   int port_flags = JackPortIsInput;
   int i;
   estimate = 1;
@@ -235,10 +258,12 @@ static int init(int rate, int channels, int format, int flags) {
     goto err_out;
   }
   if (!client_name) {
-    client_name = (char *)malloc(40);
-  sprintf(client_name, "MPlayer [%d]", getpid());
+    client_name = malloc(40);
+    sprintf(client_name, "MPlayer [%d]", getpid());
   }
-  client = jack_client_new(client_name);
+  if (!autostart)
+    open_options |= JackNoStartServer;
+  client = jack_client_open(client_name, open_options, NULL);
   if (!client) {
     mp_msg(MSGT_AO, MSGL_FATAL, "[JACK] cannot open server\n");
     goto err_out;
@@ -250,12 +275,13 @@ static int init(int rate, int channels, int format, int flags) {
   if (!port_name)
     port_flags |= JackPortIsPhysical;
   matching_ports = jack_get_ports(client, port_name, NULL, port_flags);
-  for (num_ports = 0; matching_ports && matching_ports[num_ports]; num_ports++) ;
-  if (!num_ports) {
+  if (!matching_ports || !matching_ports[0]) {
     mp_msg(MSGT_AO, MSGL_FATAL, "[JACK] no physical ports available\n");
     goto err_out;
   }
-  if (channels > num_ports) channels = num_ports;
+  i = 1;
+  while (matching_ports[i]) i++;
+  if (channels > i) channels = i;
   num_ports = channels;
 
   // create out output ports
@@ -282,7 +308,7 @@ static int init(int rate, int channels, int format, int flags) {
   jack_latency = (float)(jack_port_get_total_latency(client, ports[0]) +
                          jack_get_buffer_size(client)) / (float)rate;
   callback_interval = 0;
-  buffer = (unsigned char *) malloc(BUFFSIZE);
+  buffer = malloc(BUFFSIZE);
 
   ao_data.channels = channels;
   ao_data.samplerate = rate;
@@ -321,7 +347,7 @@ static void uninit(int immed) {
 /**
  * \brief stop playing and empty buffers (for seeking/pause)
  */
-static void reset() {
+static void reset(void) {
   paused = 1;
   read_pos = 0;
   write_pos = 0;
@@ -331,18 +357,18 @@ static void reset() {
 /**
  * \brief stop playing, keep buffers (for pause)
  */
-static void audio_pause() {
+static void audio_pause(void) {
   paused = 1;
 }
 
 /**
  * \brief resume playing, after audio_pause()
  */
-static void audio_resume() {
+static void audio_resume(void) {
   paused = 0;
 }
 
-static int get_space() {
+static int get_space(void) {
   return buf_free();
 }
 
@@ -350,12 +376,13 @@ static int get_space() {
  * \brief write data into buffer and reset underrun flag
  */
 static int play(void *data, int len, int flags) {
-  len -= len % ao_data.outburst;
+  if (!(flags & AOPLAY_FINAL_CHUNK))
+    len -= len % ao_data.outburst;
   underrun = 0;
   return write_buffer(data, len);
 }
 
-static float get_delay() {
+static float get_delay(void) {
   int buffered = BUFFSIZE - CHUNK_SIZE - buf_free(); // could be less
   float in_jack = jack_latency;
   if (estimate && callback_interval > 0) {
