@@ -13,13 +13,13 @@
 
 #include "config.h"
 #include "mp_msg.h"
-#include "bswap.h"
+#include "mpbswap.h"
 
-#include "../libvo/fastmemcpy.h"
+#include "libvo/fastmemcpy.h"
 
 #include "libmpdemux/nuppelvideo.h" 
-#include "RTjpegN.h"
-#include "minilzo.h"
+#include "rtjpegn.h"
+#include "libavutil/lzo.h"
 
 #define KEEP_BUFFER
 
@@ -27,13 +27,12 @@ void decode_nuv( unsigned char *encoded, int encoded_size,
 		unsigned char *decoded, int width, int height)
 {
 	int r;
-	unsigned int out_len;
+	unsigned int out_len = width * height + ( width * height ) / 2;
 	struct rtframeheader *encodedh = ( struct rtframeheader* ) encoded;
 	static unsigned char *buffer = 0; /* for RTJpeg with LZO decompress */
 #ifdef KEEP_BUFFER
 	static unsigned char *previous_buffer = 0; /* to support Last-frame-copy */
 #endif
-	static int is_lzo_inited = 0;
 
 //	printf("frametype: %c, comtype: %c, encoded_size: %d, width: %d, height: %d\n",
 //	    encodedh->frametype, encodedh->comptype, encoded_size, width, height);
@@ -54,41 +53,30 @@ void decode_nuv( unsigned char *encoded, int encoded_size,
 	    }
 	    case 'V':
 	    {
+		int in_len = encodedh->packetlength;
 #ifdef KEEP_BUFFER		
 		if (!previous_buffer) 
-			previous_buffer = ( unsigned char * ) malloc ( width * height + ( width * height ) / 2 );
+			previous_buffer = ( unsigned char * ) malloc ( out_len + AV_LZO_OUTPUT_PADDING );
 #endif
 
-		if (((encodedh->comptype == '2') ||
-		    (encodedh->comptype == '3')) && !is_lzo_inited)
-		{
-		    /* frame using lzo, init lzo first if not inited */
-		    if ( lzo_init() != LZO_E_OK ) 
-		    {
-			mp_msg(MSGT_DECVIDEO, MSGL_ERR, "LZO init failed\n");
-			return;
-		    }
-		    is_lzo_inited = 1;
-		}
-
 		switch(encodedh->comptype)
 		{
 		    case '0': /* raw YUV420 */
-			memcpy(decoded, encoded + 12, width*height*3/2);
+			fast_memcpy(decoded, encoded + 12, out_len);
 			break;
 		    case '1': /* RTJpeg */
 			RTjpeg_decompressYUV420 ( ( __s8 * ) encoded + 12, decoded );
 			break;
 		    case '2': /* RTJpeg with LZO */
 			if (!buffer) 
-			    buffer = ( unsigned char * ) malloc ( width * height + ( width * height ) / 2 );
+			    buffer = ( unsigned char * ) malloc ( out_len + AV_LZO_OUTPUT_PADDING );
 			if (!buffer)
 			{
 			    mp_msg(MSGT_DECVIDEO, MSGL_ERR, "Nuppelvideo: error decompressing\n");
 			    break;
 			}
-			r = lzo1x_decompress ( encoded + 12, encodedh->packetlength, buffer, &out_len, NULL );
-			if ( r != LZO_E_OK ) 
+			r = av_lzo1x_decode ( buffer, &out_len, encoded + 12, &in_len );
+			if ( r ) 
 			{
 			    mp_msg(MSGT_DECVIDEO, MSGL_ERR, "Nuppelvideo: error decompressing\n");
 			    break;
@@ -96,8 +84,8 @@ void decode_nuv( unsigned char *encoded, int encoded_size,
 			RTjpeg_decompressYUV420 ( ( __s8 * ) buffer, decoded );
 			break;
 		    case '3': /* raw YUV420 with LZO */
-			r = lzo1x_decompress ( encoded + 12, encodedh->packetlength, decoded, &out_len, NULL );
-			if ( r != LZO_E_OK ) 
+			r = av_lzo1x_decode ( decoded, &out_len, encoded + 12, &in_len );
+			if ( r ) 
 			{
 			    mp_msg(MSGT_DECVIDEO, MSGL_ERR, "Nuppelvideo: error decompressing\n");
 			    break;
@@ -109,13 +97,13 @@ void decode_nuv( unsigned char *encoded, int encoded_size,
 			break;
 		    case 'L': /* copy last frame */
 #ifdef KEEP_BUFFER
-			memcpy ( decoded, previous_buffer, width*height*3/2);
+			fast_memcpy ( decoded, previous_buffer, width*height*3/2);
 #endif
 			break;
 		}
 
 #ifdef KEEP_BUFFER
-		memcpy(previous_buffer, decoded, width*height*3/2);
+		fast_memcpy(previous_buffer, decoded, width*height*3/2);
 #endif
 		break;
 	    }
