@@ -2,13 +2,11 @@
 #include <stdlib.h>
 #include <string.h>
 
-#define USE_THEORA
-
 #include "config.h"
 #include "mp_msg.h"
 #include "help_mp.h"
 
-#ifdef HAVE_MALLOC_H
+#if HAVE_MALLOC_H
 #include <malloc.h>
 #endif
 
@@ -17,9 +15,9 @@
 
 #include "img_format.h"
 
-#include "stream.h"
-#include "demuxer.h"
-#include "stheader.h"
+#include "stream/stream.h"
+#include "libmpdemux/demuxer.h"
+#include "libmpdemux/stheader.h"
 #include "dec_video.h"
 
 #include "vd.h"
@@ -34,8 +32,6 @@ extern vd_functions_t mpcodecs_vd_dshow;
 extern vd_functions_t mpcodecs_vd_dmo;
 extern vd_functions_t mpcodecs_vd_vfw;
 extern vd_functions_t mpcodecs_vd_vfwex;
-extern vd_functions_t mpcodecs_vd_odivx;
-extern vd_functions_t mpcodecs_vd_divx4;
 extern vd_functions_t mpcodecs_vd_raw;
 extern vd_functions_t mpcodecs_vd_hmblck;
 extern vd_functions_t mpcodecs_vd_xanim;
@@ -53,62 +49,61 @@ extern vd_functions_t mpcodecs_vd_libdv;
 extern vd_functions_t mpcodecs_vd_lzo;
 extern vd_functions_t mpcodecs_vd_qtvideo;
 
+/* Please do not add any new decoders here. If you want to implement a new
+ * decoder, add it to libavcodec, except for wrappers around external
+ * libraries and decoders requiring binary support. */
+
 vd_functions_t* mpcodecs_vd_drivers[] = {
         &mpcodecs_vd_null,
-#ifdef USE_LIBAVCODEC
+#ifdef CONFIG_LIBAVCODEC
         &mpcodecs_vd_ffmpeg,
 #endif
-#ifdef HAVE_OGGTHEORA
+#ifdef CONFIG_OGGTHEORA
 	&mpcodecs_vd_theora,
 #endif
-#ifdef USE_WIN32DLL
-#ifdef USE_DIRECTSHOW
+#ifdef CONFIG_WIN32DLL
         &mpcodecs_vd_dshow,
         &mpcodecs_vd_dmo,
-#endif
         &mpcodecs_vd_vfw,
         &mpcodecs_vd_vfwex,
-#endif
-#ifdef USE_DIVX
-        &mpcodecs_vd_odivx,
-#ifdef NEW_DECORE
-        &mpcodecs_vd_divx4,
-#endif
 #endif
         &mpcodecs_vd_lzo,
         &mpcodecs_vd_raw,
         &mpcodecs_vd_hmblck,
         &mpcodecs_vd_nuv,
-#ifdef USE_XANIM
+#ifdef CONFIG_XANIM
         &mpcodecs_vd_xanim,
 #endif
-#ifdef HAVE_PNG
+#ifdef CONFIG_PNG
         &mpcodecs_vd_mpng,
 #endif
-#ifdef HAVE_JPEG
+#ifdef CONFIG_JPEG
 	&mpcodecs_vd_ijpg,
 #endif
 	&mpcodecs_vd_mtga,
 	&mpcodecs_vd_sgi,
-#ifdef USE_LIBMPEG2
+#ifdef CONFIG_LIBMPEG2
         &mpcodecs_vd_libmpeg2,
 #endif
         &mpcodecs_vd_mpegpes,
-#ifdef HAVE_ZR
+#ifdef CONFIG_ZR
         &mpcodecs_vd_zrmjpeg,
 #endif
-#ifdef USE_REALCODECS
+#ifdef CONFIG_REALCODECS
 	&mpcodecs_vd_realvid,
 #endif
-#if defined(HAVE_XVID3) || defined(HAVE_XVID4)
+#ifdef CONFIG_XVID4
 	&mpcodecs_vd_xvid,
 #endif
-#ifdef HAVE_LIBDV095
+#ifdef CONFIG_LIBDV095
 	&mpcodecs_vd_libdv,
 #endif
-#if defined(USE_QTX_CODECS) || defined(MACOSX)
+#ifdef CONFIG_QTX_CODECS
 	&mpcodecs_vd_qtvideo,
 #endif
+    /* Please do not add any new decoders here. If you want to implement a new
+     * decoder, add it to libavcodec, except for wrappers around external
+     * libraries and decoders requiring binary support. */
 	NULL
 };
 
@@ -127,7 +122,7 @@ int vo_flags=0;
 int vd_use_slices=1;
 
 /** global variables for gamma, brightness, contrast, saturation and hue 
-    modified by mplayer.c and Gui/mplayer/gtk/eq.c:
+    modified by mplayer.c and gui/mplayer/gtk/eq.c:
     ranges -100 - 100
     1000 if the vo default should be used
 */   
@@ -138,25 +133,22 @@ int vo_gamma_saturation = 1000;
 int vo_gamma_hue = 1000;
 
 extern vd_functions_t* mpvdec; // FIXME!
-extern int divx_quality;
+
+#define SCREEN_SIZE_X 1
+#define SCREEN_SIZE_Y 1
 
 int mpcodecs_config_vo(sh_video_t *sh, int w, int h, unsigned int preferred_outfmt){
     int i,j;
     unsigned int out_fmt=0;
     int screen_size_x=0;//SCREEN_SIZE_X;
     int screen_size_y=0;//SCREEN_SIZE_Y;
-//    vo_functions_t* video_out=sh->video_out;
     vf_instance_t* vf=sh->vfilter,*sc=NULL;
     int palette=0;
     int vocfg_flags=0;
 
-    if(!sh->disp_w || !sh->disp_h)
-        mp_msg(MSGT_DECVIDEO,MSGL_WARN, MSGTR_CodecDidNotSet);
-    /* XXX: HACK, if sh->disp_* aren't set,
-     * but we have w and h, set them :: atmos */
-    if(!sh->disp_w && w)
+    if(w)
         sh->disp_w=w;
-    if(!sh->disp_h && h)
+    if(h)
         sh->disp_h=h;
 
     if(!sh->disp_w || !sh->disp_h)
@@ -236,11 +228,11 @@ csp_again:
 	     }
 	}
 	mp_msg(MSGT_CPLAYER,MSGL_WARN,MSGTR_VOincompCodec);
-	sh->vf_inited=-1;
+	sh->vf_initialized=-1;
 	return 0;	// failed
     }
     out_fmt=sh->codec->outfmt[j];
-    mp_msg(MSGT_CPLAYER,MSGL_INFO,"VDec: using %s as output csp (no %d)\n",vo_format_name(out_fmt),j);
+    mp_msg(MSGT_CPLAYER,MSGL_INFO,MSGTR_UsingXAsOutputCspNoY,vo_format_name(out_fmt),j);
     sh->outfmtidx=j;
     sh->vfilter=vf;
 
@@ -261,6 +253,7 @@ csp_again:
     // time to do aspect ratio corrections...
 
   if(movie_aspect>-1.0) sh->aspect = movie_aspect; // cmdline overrides autodetect
+  else if(sh->stream_aspect!=0.0) sh->aspect = sh->stream_aspect;
 //  if(!sh->aspect) sh->aspect=1.0;
 
   if(opt_screen_size_x||opt_screen_size_y){
@@ -290,6 +283,7 @@ csp_again:
     if(sh->aspect>0.01){
       int w;
       mp_msg(MSGT_CPLAYER,MSGL_INFO,MSGTR_MovieAspectIsSet,sh->aspect);
+      mp_msg(MSGT_IDENTIFY, MSGL_INFO, "ID_VIDEO_ASPECT=%1.4f\n", sh->aspect);
       w=(int)((float)screen_size_y*sh->aspect); w+=w%2; // round
       // we don't like horizontal downscale || user forced width:
       if(w<screen_size_x || screen_size_xy>8){
@@ -321,11 +315,11 @@ csp_again:
 			 out_fmt)==0){
 //                      "MPlayer",out_fmt)){
 	mp_msg(MSGT_CPLAYER,MSGL_WARN,MSGTR_CannotInitVO);
-	sh->vf_inited=-1;
+	sh->vf_initialized=-1;
 	return 0;
     }
 
-    sh->vf_inited=1;
+    sh->vf_initialized=1;
 
     if (vo_gamma_gamma != 1000)
         set_video_colors(sh, "gamma", vo_gamma_gamma);
@@ -347,7 +341,7 @@ csp_again:
 // Note: buffer allocation may be moved to mpcodecs_config_vo() later...
 mp_image_t* mpcodecs_get_image(sh_video_t *sh, int mp_imgtype, int mp_imgflag, int w, int h){
   mp_image_t* mpi=vf_get_image(sh->vfilter,sh->codec->outfmt[sh->outfmtidx],mp_imgtype,mp_imgflag,w,h);
-  mpi->x=mpi->y=0;
+  if (mpi) mpi->x=mpi->y=0;
   return mpi;
 }
 
