@@ -3,7 +3,7 @@
 #include <string.h>
 
 #include "config.h"
-#ifdef HAVE_MALLOC_H
+#if HAVE_MALLOC_H
 #include <malloc.h>
 #endif
 
@@ -23,101 +23,97 @@
 
 #include "libvo/fastmemcpy.h"
 
-extern vf_info_t vf_info_vo;
-extern vf_info_t vf_info_rectangle;
-#ifndef HAVE_NO_POSIX_SELECT
-extern vf_info_t vf_info_bmovl;
-#endif
-extern vf_info_t vf_info_crop;
-extern vf_info_t vf_info_expand;
-#if defined(USE_LIBPOSTPROC) || defined(USE_LIBPOSTPROC_SO)
-extern vf_info_t vf_info_pp;
-#endif
-extern vf_info_t vf_info_scale;
-#ifdef USE_LIBFAME
-extern vf_info_t vf_info_fame;
-#endif
-extern vf_info_t vf_info_format;
-extern vf_info_t vf_info_noformat;
-extern vf_info_t vf_info_yuy2;
-extern vf_info_t vf_info_flip;
-extern vf_info_t vf_info_rgb2bgr;
-extern vf_info_t vf_info_rotate;
-extern vf_info_t vf_info_mirror;
-extern vf_info_t vf_info_palette;
-extern vf_info_t vf_info_lavc;
-extern vf_info_t vf_info_zrmjpeg;
-extern vf_info_t vf_info_dvbscale;
-extern vf_info_t vf_info_cropdetect;
-extern vf_info_t vf_info_test;
-extern vf_info_t vf_info_noise;
-extern vf_info_t vf_info_yvu9;
-extern vf_info_t vf_info_lavcdeint;
-extern vf_info_t vf_info_eq;
-extern vf_info_t vf_info_eq2;
-extern vf_info_t vf_info_halfpack;
-extern vf_info_t vf_info_dint;
-extern vf_info_t vf_info_1bpp;
-extern vf_info_t vf_info_2xsai;
-extern vf_info_t vf_info_unsharp;
-extern vf_info_t vf_info_swapuv;
-extern vf_info_t vf_info_il;
-extern vf_info_t vf_info_fil;
-extern vf_info_t vf_info_boxblur;
-extern vf_info_t vf_info_sab;
-extern vf_info_t vf_info_smartblur;
-extern vf_info_t vf_info_perspective;
-extern vf_info_t vf_info_down3dright;
-extern vf_info_t vf_info_field;
-extern vf_info_t vf_info_denoise3d;
-extern vf_info_t vf_info_hqdn3d;
-extern vf_info_t vf_info_detc;
-extern vf_info_t vf_info_telecine;
-extern vf_info_t vf_info_tinterlace;
-extern vf_info_t vf_info_tfields;
-extern vf_info_t vf_info_ivtc;
-extern vf_info_t vf_info_ilpack;
-extern vf_info_t vf_info_dsize;
-extern vf_info_t vf_info_decimate;
-extern vf_info_t vf_info_softpulldown;
-extern vf_info_t vf_info_pullup;
-extern vf_info_t vf_info_filmdint;
-extern vf_info_t vf_info_framestep;
-extern vf_info_t vf_info_tile;
-extern vf_info_t vf_info_delogo;
-extern vf_info_t vf_info_remove_logo;
-extern vf_info_t vf_info_hue;
-extern vf_info_t vf_info_spp;
-extern vf_info_t vf_info_uspp;
-extern vf_info_t vf_info_fspp;
-extern vf_info_t vf_info_pp7;
-extern vf_info_t vf_info_yuvcsp;
-extern vf_info_t vf_info_kerndeint;
-extern vf_info_t vf_info_rgbtest;
-extern vf_info_t vf_info_qp;
-extern vf_info_t vf_info_phase;
-extern vf_info_t vf_info_divtc;
-extern vf_info_t vf_info_harddup;
-extern vf_info_t vf_info_softskip;
-extern vf_info_t vf_info_screenshot;
+extern const vf_info_t vf_info_vo;
+extern const vf_info_t vf_info_rectangle;
+extern const vf_info_t vf_info_bmovl;
+extern const vf_info_t vf_info_crop;
+extern const vf_info_t vf_info_expand;
+extern const vf_info_t vf_info_pp;
+extern const vf_info_t vf_info_scale;
+extern const vf_info_t vf_info_format;
+extern const vf_info_t vf_info_noformat;
+extern const vf_info_t vf_info_yuy2;
+extern const vf_info_t vf_info_flip;
+extern const vf_info_t vf_info_rgb2bgr;
+extern const vf_info_t vf_info_rotate;
+extern const vf_info_t vf_info_mirror;
+extern const vf_info_t vf_info_palette;
+extern const vf_info_t vf_info_lavc;
+extern const vf_info_t vf_info_zrmjpeg;
+extern const vf_info_t vf_info_dvbscale;
+extern const vf_info_t vf_info_cropdetect;
+extern const vf_info_t vf_info_test;
+extern const vf_info_t vf_info_noise;
+extern const vf_info_t vf_info_yvu9;
+extern const vf_info_t vf_info_lavcdeint;
+extern const vf_info_t vf_info_eq;
+extern const vf_info_t vf_info_eq2;
+extern const vf_info_t vf_info_halfpack;
+extern const vf_info_t vf_info_dint;
+extern const vf_info_t vf_info_1bpp;
+extern const vf_info_t vf_info_2xsai;
+extern const vf_info_t vf_info_unsharp;
+extern const vf_info_t vf_info_swapuv;
+extern const vf_info_t vf_info_il;
+extern const vf_info_t vf_info_fil;
+extern const vf_info_t vf_info_boxblur;
+extern const vf_info_t vf_info_sab;
+extern const vf_info_t vf_info_smartblur;
+extern const vf_info_t vf_info_perspective;
+extern const vf_info_t vf_info_down3dright;
+extern const vf_info_t vf_info_field;
+extern const vf_info_t vf_info_denoise3d;
+extern const vf_info_t vf_info_hqdn3d;
+extern const vf_info_t vf_info_detc;
+extern const vf_info_t vf_info_telecine;
+extern const vf_info_t vf_info_tinterlace;
+extern const vf_info_t vf_info_tfields;
+extern const vf_info_t vf_info_ivtc;
+extern const vf_info_t vf_info_ilpack;
+extern const vf_info_t vf_info_dsize;
+extern const vf_info_t vf_info_decimate;
+extern const vf_info_t vf_info_softpulldown;
+extern const vf_info_t vf_info_pullup;
+extern const vf_info_t vf_info_filmdint;
+extern const vf_info_t vf_info_framestep;
+extern const vf_info_t vf_info_tile;
+extern const vf_info_t vf_info_delogo;
+extern const vf_info_t vf_info_remove_logo;
+extern const vf_info_t vf_info_hue;
+extern const vf_info_t vf_info_spp;
+extern const vf_info_t vf_info_uspp;
+extern const vf_info_t vf_info_fspp;
+extern const vf_info_t vf_info_pp7;
+extern const vf_info_t vf_info_yuvcsp;
+extern const vf_info_t vf_info_kerndeint;
+extern const vf_info_t vf_info_rgbtest;
+extern const vf_info_t vf_info_qp;
+extern const vf_info_t vf_info_phase;
+extern const vf_info_t vf_info_divtc;
+extern const vf_info_t vf_info_harddup;
+extern const vf_info_t vf_info_softskip;
+extern const vf_info_t vf_info_screenshot;
+extern const vf_info_t vf_info_ass;
+extern const vf_info_t vf_info_mcdeint;
+extern const vf_info_t vf_info_yadif;
+extern const vf_info_t vf_info_blackframe;
+extern const vf_info_t vf_info_geq;
+extern const vf_info_t vf_info_ow;
 
 // list of available filters:
-static vf_info_t* filter_list[]={
+static const vf_info_t* const filter_list[]={
     &vf_info_rectangle,
-#ifndef HAVE_NO_POSIX_SELECT
+#ifdef HAVE_POSIX_SELECT
     &vf_info_bmovl,
 #endif
     &vf_info_crop,
     &vf_info_expand,
-#if defined(USE_LIBPOSTPROC) || defined(USE_LIBPOSTPROC_SO)
+#ifdef CONFIG_LIBPOSTPROC
     &vf_info_pp,
 #endif
     &vf_info_scale,
 //    &vf_info_osd,
     &vf_info_vo,
-#ifdef USE_LIBFAME
-    &vf_info_fame,
-#endif
     &vf_info_format,
     &vf_info_noformat,
     &vf_info_yuy2,
@@ -126,11 +122,13 @@ static vf_info_t* filter_list[]={
     &vf_info_rotate,
     &vf_info_mirror,
     &vf_info_palette,
-#ifdef USE_LIBAVCODEC
+    &vf_info_pp7,
+#ifdef CONFIG_LIBAVCODEC
     &vf_info_lavc,
     &vf_info_lavcdeint,
+    &vf_info_screenshot,
 #endif
-#ifdef HAVE_ZR
+#ifdef CONFIG_ZR
     &vf_info_zrmjpeg,
 #endif
     &vf_info_dvbscale,
@@ -172,33 +170,33 @@ static vf_info_t* filter_list[]={
     &vf_info_delogo,
     &vf_info_remove_logo,
     &vf_info_hue,
-#ifdef USE_LIBAVCODEC_DSPUTIL
+#ifdef CONFIG_LIBAVCODEC_A
     &vf_info_spp,
     &vf_info_uspp,
     &vf_info_fspp,
     &vf_info_qp,
+    &vf_info_mcdeint,
+    &vf_info_geq,
 #endif
     &vf_info_yuvcsp,
     &vf_info_kerndeint,
     &vf_info_rgbtest,
-#ifdef USE_LIBAVCODEC
-    &vf_info_pp7,
-#endif
     &vf_info_phase,
     &vf_info_divtc,
     &vf_info_harddup,
     &vf_info_softskip,
-#ifdef HAVE_PNG
-    &vf_info_screenshot,
+#ifdef CONFIG_ASS
+    &vf_info_ass,
 #endif
+    &vf_info_yadif,
+    &vf_info_blackframe,
+    &vf_info_ow,
     NULL
 };
 
 // For the vf option
 m_obj_settings_t* vf_settings = NULL;
-// For the vop option
-m_obj_settings_t* vo_plugin_args = NULL;
-m_obj_list_t vf_obj_list = {
+const m_obj_list_t vf_obj_list = {
   (void**)filter_list,
   M_ST_OFF(vf_info_t,name),
   M_ST_OFF(vf_info_t,info),
@@ -255,6 +253,7 @@ void vf_mpi_clear(mp_image_t* mpi,int x0,int y0,int w,int h){
 mp_image_t* vf_get_image(vf_instance_t* vf, unsigned int outfmt, int mp_imgtype, int mp_imgflag, int w, int h){
   mp_image_t* mpi=NULL;
   int w2;
+  int number = mp_imgtype >> 16;
 
 #ifdef MP_DEBUG
   assert(w == -1 || w >= vf->w);
@@ -271,13 +270,13 @@ mp_image_t* vf_get_image(vf_instance_t* vf, unsigned int outfmt, int mp_imgtype,
   w2=(mp_imgflag&MP_IMGFLAG_ACCEPT_ALIGNED_STRIDE)?((w+15)&(~15)):w;
   
   if(vf->put_image==vf_next_put_image){
-      // passthru mode, if the plugin uses the fallback/default put_image() code
+      // passthru mode, if the filter uses the fallback/default put_image() code
       return vf_get_image(vf->next,outfmt,mp_imgtype,mp_imgflag,w,h);
   }
   
   // Note: we should call libvo first to check if it supports direct rendering
   // and if not, then fallback to software buffers:
-  switch(mp_imgtype){
+  switch(mp_imgtype & 0xff){
   case MP_IMGTYPE_EXPORT:
     if(!vf->imgctx.export_images[0]) vf->imgctx.export_images[0]=new_mp_image(w2,h);
     mpi=vf->imgctx.export_images[0];
@@ -301,6 +300,19 @@ mp_image_t* vf_get_image(vf_instance_t* vf, unsigned int outfmt, int mp_imgtype,
     mpi=vf->imgctx.static_images[vf->imgctx.static_idx];
     vf->imgctx.static_idx^=1;
     break;
+  case MP_IMGTYPE_NUMBERED:
+    if (number == -1) {
+      int i;
+      for (i = 0; i < NUM_NUMBERED_MPI; i++)
+        if (!vf->imgctx.numbered_images[i] || !(vf->imgctx.numbered_images[i]->flags & MP_IMGFLAG_IN_USE))
+          break;
+      number = i;
+    }
+    if (number < 0 || number >= NUM_NUMBERED_MPI) return NULL;
+    if (!vf->imgctx.numbered_images[number]) vf->imgctx.numbered_images[number] = new_mp_image(w2,h);
+    mpi = vf->imgctx.numbered_images[number];
+    mpi->number = number;
+    break;
   }
   if(mpi){
     mpi->type=mp_imgtype;
@@ -334,6 +346,11 @@ mp_image_t* vf_get_image(vf_instance_t* vf, unsigned int outfmt, int mp_imgtype,
 	
         if(!(mpi->flags&MP_IMGFLAG_DIRECT)){
           // non-direct and not yet allocated image. allocate it!
+          if (!mpi->bpp) { // no way we can allocate this
+              mp_msg(MSGT_DECVIDEO, MSGL_FATAL,
+                     "vf_get_image: Tried to allocate a format that can not be allocated!\n");
+              return NULL;
+          }
 	  
 	  // check if codec prefer aligned stride:  
 	  if(mp_imgflag&MP_IMGFLAG_PREFER_ALIGNED_STRIDE){
@@ -412,7 +429,9 @@ mp_image_t* vf_get_image(vf_instance_t* vf, unsigned int outfmt, int mp_imgtype,
 	    mpi->flags|=MP_IMGFLAG_TYPE_DISPLAYED;
     }
 
+  mpi->qscale = NULL;
   }
+  mpi->flags |= MP_IMGFLAG_IN_USE;
 //    printf("\rVF_MPI: %p %p %p %d %d %d    \n",
 //	mpi->planes[0],mpi->planes[1],mpi->planes[2],
 //	mpi->stride[0],mpi->stride[1],mpi->stride[2]);
@@ -427,7 +446,7 @@ static int vf_default_query_format(struct vf_instance_s* vf, unsigned int fmt){
   return vf_next_query_format(vf,fmt);
 }
 
-vf_instance_t* vf_open_plugin(vf_info_t** filter_list, vf_instance_t* next, char *name, char **args){
+vf_instance_t* vf_open_plugin(const vf_info_t* const* filter_list, vf_instance_t* next, const char *name, char **args){
     vf_instance_t* vf;
     int i;
     for(i=0;;i++){
@@ -448,7 +467,7 @@ vf_instance_t* vf_open_plugin(vf_info_t** filter_list, vf_instance_t* next, char
     vf->default_caps=VFCAP_ACCEPT_STRIDE;
     vf->default_reqs=0;
     if(vf->info->opts) { // vf_vo get some special argument
-      m_struct_t* st = vf->info->opts;
+      const m_struct_t* st = vf->info->opts;
       void* vf_priv = m_struct_alloc(st);
       int n;
       for(n = 0 ; args && args[2*n] ; n++)
@@ -466,7 +485,7 @@ vf_instance_t* vf_open_plugin(vf_info_t** filter_list, vf_instance_t* next, char
     return NULL;
 }
 
-vf_instance_t* vf_open_filter(vf_instance_t* next, char *name, char **args){
+vf_instance_t* vf_open_filter(vf_instance_t* next, const char *name, char **args){
   if(args && strcmp(args[0],"_oldargs_")) {
     int i,l = 0;
     for(i = 0 ; args && args[2*i] ; i++)
@@ -513,9 +532,9 @@ vf_instance_t* vf_add_before_vo(vf_instance_t **vf, char *name, char **args) {
 
 //============================================================================
 
-unsigned int vf_match_csp(vf_instance_t** vfp,unsigned int* list,unsigned int preferred){
+unsigned int vf_match_csp(vf_instance_t** vfp,const unsigned int* list,unsigned int preferred){
     vf_instance_t* vf=*vfp;
-    unsigned int* p;
+    const unsigned int* p;
     unsigned int best=0;
     int ret;
     if((p=list)) while(*p){
@@ -553,6 +572,44 @@ void vf_clone_mpi_attributes(mp_image_t* dst, mp_image_t* src){
 	dst->qscale= src->qscale;
     }
 }
+
+void vf_queue_frame(vf_instance_t *vf, int (*func)(vf_instance_t *))
+{
+    vf->continue_buffered_image = func;
+}
+
+// Output the next buffered image (if any) from the filter chain.
+// The queue could be kept as a simple stack/list instead avoiding the
+// looping here, but there's currently no good context variable where
+// that could be stored so this was easier to implement.
+
+int vf_output_queued_frame(vf_instance_t *vf)
+{
+    while (1) {
+	int ret;
+	vf_instance_t *current;
+	vf_instance_t *last=NULL;
+	int (*tmp)(vf_instance_t *);
+	for (current = vf; current; current = current->next)
+	    if (current->continue_buffered_image)
+		last = current;
+	if (!last)
+	    return 0;
+	tmp = last->continue_buffered_image;
+	last->continue_buffered_image = NULL;
+	ret = tmp(last);
+	if (ret > 0) {
+	    vf->control(vf, VFCTRL_DRAW_OSD, NULL);
+#ifdef CONFIG_ASS
+	    vf->control(vf, VFCTRL_DRAW_EOSD, NULL);
+#endif
+	}
+	if (ret)
+	    return ret;
+    }
+}
+
+
 /**
  * \brief Video config() function wrapper
  *
@@ -624,6 +681,14 @@ int vf_next_control(struct vf_instance_s* vf, int request, void* data){
     return vf->next->control(vf->next,request,data);
 }
 
+void vf_extra_flip(struct vf_instance_s* vf) {
+    vf_next_control(vf, VFCTRL_DRAW_OSD, NULL);
+#ifdef CONFIG_ASS
+    vf_next_control(vf, VFCTRL_DRAW_EOSD, NULL);
+#endif
+    vf_next_control(vf, VFCTRL_FLIP_PAGE, NULL);
+}
+
 int vf_next_query_format(struct vf_instance_s* vf, unsigned int fmt){
     int flags=vf->next->query_format(vf->next,fmt);
     if(flags) flags|=vf->default_caps;
@@ -662,7 +727,6 @@ vf_instance_t* append_filters(vf_instance_t* last){
   vf_instance_t* vf;
   int i; 
 
-  // -vf take precedence over -vop
   if(vf_settings) {
     // We want to add them in the 'right order'
     for(i = 0 ; vf_settings[i].name ; i++)
@@ -672,12 +736,6 @@ vf_instance_t* append_filters(vf_instance_t* last){
       vf = vf_open_filter(last,vf_settings[i].name,vf_settings[i].attribs);
       if(vf) last=vf;
     }
-  } else if(vo_plugin_args) {
-    for(i = 0 ; vo_plugin_args[i].name ; i++) {
-      vf = vf_open_filter(last,vo_plugin_args[i].name,
-			  vo_plugin_args[i].attribs);
-      if(vf) last=vf;
-    }
   }
   return last;
 }
@@ -700,11 +758,3 @@ void vf_uninit_filter_chain(vf_instance_t* vf){
 	vf=next;
     }
 }
-
-void vf_list_plugins(void){
-    int i=0;
-    while(filter_list[i]){
-        mp_msg(MSGT_VFILTER,MSGL_INFO,"\t%-10s: %s\n",filter_list[i]->name,filter_list[i]->info);
-        i++;
-    }
-}
