@@ -1,12 +1,26 @@
-#ifndef __DEMUXER_H
-#define __DEMUXER_H 1
+#ifndef MPLAYER_DEMUXER_H
+#define MPLAYER_DEMUXER_H
+
+#include <sys/types.h>
+#include <stdint.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "stream/stream.h"
+#ifdef CONFIG_ASS
+#include "libass/ass_types.h"
+#endif
+
+#ifdef HAVE_BUILTIN_EXPECT
+#define likely(x) __builtin_expect ((x) != 0, 1)
+#define unlikely(x) __builtin_expect ((x) != 0, 0)
+#else
+#define likely(x) (x)
+#define unlikely(x) (x)
+#endif
 
 #define MAX_PACKS 4096
-#ifdef HAVE_TV_BSDBT848
 #define MAX_PACK_BYTES 0x2000000
-#else
-#define MAX_PACK_BYTES 0x800000
-#endif
 
 #define DEMUXER_TYPE_UNKNOWN 0
 #define DEMUXER_TYPE_MPEG_ES 1
@@ -50,21 +64,22 @@
 #define DEMUXER_TYPE_MPC 40
 #define DEMUXER_TYPE_MPEG_PES 41
 #define DEMUXER_TYPE_MPEG_GXF 42
+#define DEMUXER_TYPE_NUT 43
+#define DEMUXER_TYPE_LAVF_PREFERRED 44
+#define DEMUXER_TYPE_RTP_NEMESI 45
+#define DEMUXER_TYPE_MNG 46
 
 // This should always match the higest demuxer type number.
 // Unless you want to disallow users to force the demuxer to some types
 #define DEMUXER_TYPE_MIN 0
-#define DEMUXER_TYPE_MAX 42
+#define DEMUXER_TYPE_MAX 46
 
 #define DEMUXER_TYPE_DEMUXERS (1<<16)
 // A virtual demuxer type for the network code
 #define DEMUXER_TYPE_PLAYLIST (2<<16)
 
 
-#define DEMUXER_TIME_NONE 0
-#define DEMUXER_TIME_PTS 1
-#define DEMUXER_TIME_FILE 2
-#define DEMUXER_TIME_BPS 3
+#define MP_NOPTS_VALUE (-1LL<<63) //both int64_t and double should be able to represent this exactly
 
 
 // DEMUXER control commands/answers
@@ -75,11 +90,22 @@
 #define DEMUXER_CTRL_GET_TIME_LENGTH 10
 #define DEMUXER_CTRL_GET_PERCENT_POS 11
 #define DEMUXER_CTRL_SWITCH_AUDIO 12
+#define DEMUXER_CTRL_RESYNC 13
+#define DEMUXER_CTRL_SWITCH_VIDEO 14
+#define DEMUXER_CTRL_IDENTIFY_PROGRAM 15
+#define DEMUXER_CTRL_CORRECT_PTS 16
+
+#define SEEK_ABSOLUTE (1 << 0)
+#define SEEK_FACTOR   (1 << 1)
+
+#define MP_INPUT_BUFFER_PADDING_SIZE 8
 
 // Holds one packet/frame/whatever
 typedef struct demux_packet_st {
   int len;
-  float pts;
+  double pts;
+  double endpts;
+  double stream_pts;
   off_t pos;  // position in index (AVI) or file (MPG)
   unsigned char* buffer;
   int flags; // keyframe, etc
@@ -92,7 +118,7 @@ typedef struct {
   int buffer_pos;          // current buffer position
   int buffer_size;         // current buffer size
   unsigned char* buffer;   // current buffer, never free() it, always use free_demux_packet(buffer_ref);
-  float pts;               // current buffer's pts
+  double pts;              // current buffer's pts
   int pts_bytes;           // number of bytes read after last pts stamp
   int eof;                 // end of demuxed stream? (true if all buffer empty)
   off_t pos;                 // position in the input stream (file)
@@ -126,9 +152,13 @@ typedef struct demuxer_info_st {
 
 #define MAX_A_STREAMS 256
 #define MAX_V_STREAMS 256
+#define MAX_S_STREAMS 256
 
 struct demuxer_st;
 
+extern int correct_pts;
+extern int user_correct_pts;
+
 /**
  * Demuxer description structure
  */
@@ -156,12 +186,28 @@ typedef struct demuxers_desc_st {
   int (*control)(struct demuxer_st *demuxer, int cmd, void *arg); ///< Optional
 } demuxer_desc_t;
 
+typedef struct demux_chapter_s
+{
+  uint64_t start, end;
+  char* name;
+} demux_chapter_t;
+
+typedef struct demux_attachment_s
+{
+  char* name;
+  char* type;
+  void* data;
+  unsigned int data_size;
+} demux_attachment_t;
+
 typedef struct demuxer_st {
-  demuxer_desc_t *desc;  ///< Demuxer description structure
+  const demuxer_desc_t *desc;  ///< Demuxer description structure
   off_t filepos; // input stream current pos.
   off_t movi_start;
   off_t movi_end;
   stream_t *stream;
+  double stream_pts;       // current stream pts, if applicable (e.g. dvd)
+  double reference_clock;
   char *filename; ///< Needed by avs_check_file
   int synced;  // stream synced (used by mpeg)
   int type;    // demuxer type: mpeg PS, mpeg ES, avi, avi-ni, avi-nini, asf
@@ -175,30 +221,43 @@ typedef struct demuxer_st {
   // stream headers:
   void* a_streams[MAX_A_STREAMS]; // audio streams (sh_audio_t)
   void* v_streams[MAX_V_STREAMS]; // video sterams (sh_video_t)
-  char s_streams[32];   // dvd subtitles (flag)
+  void *s_streams[MAX_S_STREAMS];   // dvd subtitles (flag)
+
+  demux_chapter_t* chapters;
+  int num_chapters;
   
+  demux_attachment_t* attachments;
+  int num_attachments;
+
   void* priv;  // fileformat-dependent data
   char** info;
 } demuxer_t;
 
-inline static demux_packet_t* new_demux_packet(int len){
+typedef struct {
+  int progid;        //program id
+  int aid, vid, sid; //audio, video and subtitle id
+} demux_program_t;
+
+static inline demux_packet_t* new_demux_packet(int len){
   demux_packet_t* dp=(demux_packet_t*)malloc(sizeof(demux_packet_t));
   dp->len=len;
   dp->next=NULL;
-  dp->pts=0;
+  dp->pts=MP_NOPTS_VALUE;
+  dp->endpts=MP_NOPTS_VALUE;
+  dp->stream_pts = MP_NOPTS_VALUE;
   dp->pos=0;
   dp->flags=0;
   dp->refcount=1;
   dp->master=NULL;
   dp->buffer=NULL;
-  if (len > 0 && (dp->buffer = (unsigned char *)malloc(len + 8)))
+  if (len > 0 && (dp->buffer = (unsigned char *)malloc(len + MP_INPUT_BUFFER_PADDING_SIZE)))
     memset(dp->buffer + len, 0, 8);
   else
     dp->len = 0;
   return dp;
 }
 
-inline static void resize_demux_packet(demux_packet_t* dp, int len)
+static inline void resize_demux_packet(demux_packet_t* dp, int len)
 {
   if(len > 0)
   {
@@ -216,7 +275,7 @@ inline static void resize_demux_packet(demux_packet_t* dp, int len)
      dp->len = 0;
 }
 
-inline static demux_packet_t* clone_demux_packet(demux_packet_t* pack){
+static inline demux_packet_t* clone_demux_packet(demux_packet_t* pack){
   demux_packet_t* dp=(demux_packet_t*)malloc(sizeof(demux_packet_t));
   while(pack->master) pack=pack->master; // find the master
   memcpy(dp,pack,sizeof(demux_packet_t));
@@ -227,7 +286,7 @@ inline static demux_packet_t* clone_demux_packet(demux_packet_t* pack){
   return dp;
 }
 
-inline static void free_demux_packet(demux_packet_t* dp){
+static inline void free_demux_packet(demux_packet_t* dp){
   if (dp->master==NULL){  //dp is a master packet
     dp->refcount--;
     if (dp->refcount==0){
@@ -241,27 +300,38 @@ inline static void free_demux_packet(demux_packet_t* dp){
   free(dp);
 }
 
+#ifndef SIZE_MAX
+#define SIZE_MAX ((size_t)-1)
+#endif
+
+static inline void *realloc_struct(void *ptr, size_t nmemb, size_t size) {
+  if (nmemb > SIZE_MAX / size) {
+    free(ptr);
+    return NULL;
+  }
+  return realloc(ptr, nmemb * size);
+}
+
 demux_stream_t* new_demuxer_stream(struct demuxer_st *demuxer,int id);
 demuxer_t* new_demuxer(stream_t *stream,int type,int a_id,int v_id,int s_id,char *filename);
 void free_demuxer_stream(demux_stream_t *ds);
 void free_demuxer(demuxer_t *demuxer);
 
 void ds_add_packet(demux_stream_t *ds,demux_packet_t* dp);
-void ds_read_packet(demux_stream_t *ds,stream_t *stream,int len,float pts,off_t pos,int flags);
+void ds_read_packet(demux_stream_t *ds, stream_t *stream, int len, double pts, off_t pos, int flags);
 
 int demux_fill_buffer(demuxer_t *demux,demux_stream_t *ds);
 int ds_fill_buffer(demux_stream_t *ds);
 
-inline static off_t ds_tell(demux_stream_t *ds){
+static inline off_t ds_tell(demux_stream_t *ds){
   return (ds->dpos-ds->buffer_size)+ds->buffer_pos;
 }
 
-inline static int ds_tell_pts(demux_stream_t *ds){
+static inline int ds_tell_pts(demux_stream_t *ds){
   return (ds->pts_bytes-ds->buffer_size)+ds->buffer_pos;
 }
 
 int demux_read_data(demux_stream_t *ds,unsigned char* mem,int len);
-int demux_read_data_pack(demux_stream_t *ds,unsigned char* mem,int len);
 int demux_pattern_3(demux_stream_t *ds, unsigned char *mem, int maxlen,
                     int *read, uint32_t pattern);
 
@@ -273,7 +343,7 @@ int demux_pattern_3(demux_stream_t *ds, unsigned char *mem, int maxlen,
      (likely(ds->buffer_pos<ds->buffer_size)) ? ds->buffer[ds->buffer_pos++] \
      :((unlikely(!ds_fill_buffer(ds)))? (-1) : ds->buffer[ds->buffer_pos++] ) )
 #else
-inline static int demux_getc(demux_stream_t *ds){
+static inline int demux_getc(demux_stream_t *ds){
   if(ds->buffer_pos>=ds->buffer_size){
     if(!ds_fill_buffer(ds)){
 //      printf("DEMUX_GETC: EOF reached!\n");
@@ -287,8 +357,9 @@ inline static int demux_getc(demux_stream_t *ds){
 
 void ds_free_packs(demux_stream_t *ds);
 int ds_get_packet(demux_stream_t *ds,unsigned char **start);
+int ds_get_packet_pts(demux_stream_t *ds, unsigned char **start, double *pts);
 int ds_get_packet_sub(demux_stream_t *ds,unsigned char **start);
-float ds_get_next_pts(demux_stream_t *ds);
+double ds_get_next_pts(demux_stream_t *ds);
 
 // This is defined here because demux_stream_t ins't defined in stream.h
 stream_t* new_ds_stream(demux_stream_t *ds);
@@ -296,7 +367,7 @@ stream_t* new_ds_stream(demux_stream_t *ds);
 static inline int avi_stream_id(unsigned int id){
   unsigned char *p=(unsigned char *)&id;
   unsigned char a,b;
-#if WORDS_BIGENDIAN
+#ifdef WORDS_BIGENDIAN
   a=p[3]-'0'; b=p[2]-'0';
 #else
   a=p[0]-'0'; b=p[1]-'0';
@@ -306,6 +377,7 @@ static inline int avi_stream_id(unsigned int id){
 }
 
 demuxer_t* demux_open(stream_t *stream,int file_format,int aid,int vid,int sid,char* filename);
+void demux_flush(demuxer_t *demuxer);
 int demux_seek(demuxer_t *demuxer,float rel_seek_secs,float audio_delay,int flags);
 demuxer_t*  new_demuxers_demuxer(demuxer_t* vd, demuxer_t* ad, demuxer_t* sd);
 
@@ -317,25 +389,60 @@ extern int pts_from_bps;
 
 extern int extension_parsing;
 
-int demux_info_add(demuxer_t *demuxer, char *opt, char *param);
-char* demux_info_get(demuxer_t *demuxer, char *opt);
+int demux_info_add(demuxer_t *demuxer, const char *opt, const char *param);
+char* demux_info_get(demuxer_t *demuxer, const char *opt);
 int demux_info_print(demuxer_t *demuxer);
 int demux_control(demuxer_t *demuxer, int cmd, void *arg);
 
-#ifdef HAVE_OGGVORBIS
+#ifdef CONFIG_OGGVORBIS
 /* Found in demux_ogg.c */
 int demux_ogg_num_subs(demuxer_t *demuxer);
 int demux_ogg_sub_id(demuxer_t *demuxer, int index);
-char *demux_ogg_sub_lang(demuxer_t *demuxer, int index);
 #endif
 
-#endif
+int demuxer_get_current_time(demuxer_t *demuxer);
+double demuxer_get_time_length(demuxer_t *demuxer);
+int demuxer_get_percent_pos(demuxer_t *demuxer);
+int demuxer_switch_audio(demuxer_t *demuxer, int index);
+int demuxer_switch_video(demuxer_t *demuxer, int index);
 
-extern double demuxer_get_time_length(demuxer_t *demuxer);
-extern int demuxer_get_percent_pos(demuxer_t *demuxer);
-extern int demuxer_switch_audio(demuxer_t *demuxer, int index);
+int demuxer_type_by_filename(char* filename);
 
-extern int demuxer_type_by_filename(char* filename);
+void demuxer_help(void);
+int get_demuxer_type_from_name(char *demuxer_name, int *force);
 
-extern void demuxer_help(void);
-extern int get_demuxer_type_from_name(char *demuxer_name, int *force);
+int demuxer_add_attachment(demuxer_t* demuxer, const char* name,
+                           const char* type, const void* data, size_t size);
+
+int demuxer_add_chapter(demuxer_t* demuxer, const char* name, uint64_t start, uint64_t end);
+int demuxer_seek_chapter(demuxer_t *demuxer, int chapter, int mode, float *seek_pts, int *num_chapters, char **chapter_name);
+
+/// Get current chapter index if available.
+int demuxer_get_current_chapter(demuxer_t *demuxer);
+/// Get chapter name by index if available.
+char *demuxer_chapter_name(demuxer_t *demuxer, int chapter);
+/// Get chapter display name by index.
+char *demuxer_chapter_display_name(demuxer_t *demuxer, int chapter);
+/// Get chapter start time and end time by index if available.
+float demuxer_chapter_time(demuxer_t *demuxer, int chapter, float *end);
+/// Get total chapter number.
+int demuxer_chapter_count(demuxer_t *demuxer);
+/// Get current angle index.
+int demuxer_get_current_angle(demuxer_t *demuxer);
+/// Set angle.
+int demuxer_set_angle(demuxer_t *demuxer, int angle);
+/// Get number of angles.
+int demuxer_angles_count(demuxer_t *demuxer);
+
+// get the index of a track
+// lang is a comma-separated list
+int demuxer_audio_track_by_lang(demuxer_t* demuxer, char* lang);
+int demuxer_sub_track_by_lang(demuxer_t* demuxer, char* lang);
+
+// find the default track
+// for subtitles, it is the first track with default attribute
+// for audio, additionally, the first track is selected if no track has default attribute set
+int demuxer_default_audio_track(demuxer_t* d);
+int demuxer_default_sub_track(demuxer_t* d);
+
+#endif /* MPLAYER_DEMUXER_H */
