@@ -4,42 +4,57 @@
  *
  * Artur Zaprzala <zybi@fanthom.irc.pl>
  *
- * ported inside mplayer by Jindrich Makovicka 
- * <makovick@kmlinux.fjfi.cvut.cz>
+ * ported inside MPlayer by Jindrich Makovicka <makovick@gmail.com>
  *
+ * This file is part of MPlayer.
+ *
+ * MPlayer is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * MPlayer is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with MPlayer; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "config.h"
 
-#ifdef HAVE_FREETYPE
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
 #include <string.h>
 
+#ifdef CONFIG_ICONV
 #include <iconv.h>
+#endif
 
 #include <ft2build.h>
 #include FT_FREETYPE_H
 #include FT_GLYPH_H
 
-#ifdef HAVE_FONTCONFIG
+#ifdef CONFIG_FONTCONFIG
 #include <fontconfig/fontconfig.h>
 #endif
 
-#include "bswap.h"
+#include "libavutil/common.h"
+#include "mpbswap.h"
 #include "font_load.h"
 #include "mp_msg.h"
+#include "help_mp.h"
 #include "mplayer.h"
+#include "get_path.h"
 #include "osd_font.h"
 
 #if (FREETYPE_MAJOR > 2) || (FREETYPE_MAJOR == 2 && FREETYPE_MINOR >= 1)
 #define HAVE_FREETYPE21
 #endif
 
-char *get_path(char *filename);
-
 char *subtitle_font_encoding = NULL;
 float text_font_scale_factor = 5.0;
 float osd_font_scale_factor = 6.0;
@@ -179,8 +194,8 @@ static int check_font(font_desc_t *desc, float ppem, int padding, int pic_idx,
 	else {
 	    glyph_index = FT_Get_Char_Index(face, uni_charmap ? character:code);
 	    if (glyph_index==0) {
-		WARNING("Glyph for char 0x%02x|U+%04X|%c not found.", code, character,
-			code<' '||code>255 ? '.':code);
+		WARNING("Glyph for char 0x%02lx|U+%04lX|%c not found.", code, character,
+			code<' '||code>255 ? '.':(char)code);
 		desc->font[unicode?character:code] = -1;
 		continue;
 	    }
@@ -234,7 +249,7 @@ static int check_font(font_desc_t *desc, float ppem, int padding, int pic_idx,
 }
 
 // general outline
-void outline(
+static void outline(
 	unsigned char *s,
 	unsigned char *t,
 	int width,
@@ -277,7 +292,7 @@ void outline(
 
 
 // 1 pixel outline
-void outline1(
+static void outline1(
 	unsigned char *s,
 	unsigned char *t,
 	int width,
@@ -315,7 +330,7 @@ void outline1(
 }
 
 // "0 pixel outline"
-void outline0(
+static void outline0(
 	unsigned char *s,
 	unsigned char *t,
 	int width,
@@ -453,22 +468,6 @@ void blur(
     }
 }
 
-// Gaussian matrix
-static unsigned gmatrix(unsigned char *m, int r, int w, double const A) {
-    unsigned volume = 0;		// volume under Gaussian area is exactly -pi*base/A
-    int mx, my;
-
-    for (my = 0; my<w; ++my) {
-	for (mx = 0; mx<w; ++mx) {
-	    m[mx+my*w] = (exp(A * ((mx-r)*(mx-r)+(my-r)*(my-r))) * base + .5);
-	    volume+= m[mx+my*w];
-	}
-    }
-    mp_msg(MSGT_OSD, MSGL_DBG2, "A= %f\n", A);
-    mp_msg(MSGT_OSD, MSGL_DBG2, "volume: %i; exact: %.0f; volume/exact: %.6f\n\n", volume, -M_PI*base/A, volume/(-M_PI*base/A));
-    return volume;
-}
-
 static void resample_alpha(unsigned char *abuf, unsigned char *bbuf, int width, int height, int stride, float factor)
 {
         int f=factor*256.0f;
@@ -618,9 +617,9 @@ static int prepare_font(font_desc_t *desc, FT_Face face, float ppem, int pic_idx
 
     desc->faces[pic_idx] = face;
 
-    desc->pic_a[pic_idx] = (raw_file*)malloc(sizeof(raw_file));
+    desc->pic_a[pic_idx] = malloc(sizeof(raw_file));
     if (!desc->pic_a[pic_idx]) return -1;
-    desc->pic_b[pic_idx] = (raw_file*)malloc(sizeof(raw_file));
+    desc->pic_b[pic_idx] = malloc(sizeof(raw_file));
     if (!desc->pic_b[pic_idx]) return -1;
 
     desc->pic_a[pic_idx]->bmp = NULL;
@@ -628,11 +627,11 @@ static int prepare_font(font_desc_t *desc, FT_Face face, float ppem, int pic_idx
     desc->pic_b[pic_idx]->bmp = NULL;
     desc->pic_b[pic_idx]->pal = NULL;
 
-    desc->pic_a[pic_idx]->pal = (unsigned char*)malloc(sizeof(unsigned char)*256*3);
+    desc->pic_a[pic_idx]->pal = malloc(sizeof(unsigned char)*256*3);
     if (!desc->pic_a[pic_idx]->pal) return -1;
     for (i = 0; i<768; ++i) desc->pic_a[pic_idx]->pal[i] = i/3;
 
-    desc->pic_b[pic_idx]->pal = (unsigned char*)malloc(sizeof(unsigned char)*256*3);
+    desc->pic_b[pic_idx]->pal = malloc(sizeof(unsigned char)*256*3);
     if (!desc->pic_b[pic_idx]->pal) return -1;
     for (i = 0; i<768; ++i) desc->pic_b[pic_idx]->pal[i] = i/3;
 
@@ -654,7 +653,7 @@ static int prepare_font(font_desc_t *desc, FT_Face face, float ppem, int pic_idx
     
 }
 
-int generate_tables(font_desc_t *desc, double thickness, double radius)
+static int generate_tables(font_desc_t *desc, double thickness, double radius)
 {
     int width = desc->max_height;
     int height = desc->max_width;
@@ -673,13 +672,13 @@ int generate_tables(font_desc_t *desc, double thickness, double radius)
 //    fprintf(stderr, "o_r = %d\n", desc->tables.o_r);
 
     if (desc->tables.g_r) {
-	desc->tables.g = (unsigned*)malloc(desc->tables.g_w * sizeof(unsigned));
-	desc->tables.gt2 = (unsigned*)malloc(256 * desc->tables.g_w * sizeof(unsigned));
+	desc->tables.g = malloc(desc->tables.g_w * sizeof(unsigned));
+	desc->tables.gt2 = malloc(256 * desc->tables.g_w * sizeof(unsigned));
 	if (desc->tables.g==NULL || desc->tables.gt2==NULL) {
 	    return -1;
 	}
     }
-    desc->tables.om = (unsigned*)malloc(desc->tables.o_w*desc->tables.o_w * sizeof(unsigned));
+    desc->tables.om = malloc(desc->tables.o_w*desc->tables.o_w * sizeof(unsigned));
     desc->tables.omt = malloc(desc->tables.o_size*256);
 
     omtp = desc->tables.omt;
@@ -731,14 +730,14 @@ int generate_tables(font_desc_t *desc, double thickness, double radius)
     return 0;
 }
 
-
+#ifdef CONFIG_ICONV
 /* decode from 'encoding' to unicode */
 static FT_ULong decode_char(iconv_t *cd, char c) {
     FT_ULong o;
     char *inbuf = &c;
     char *outbuf = (char*)&o;
-    int inbytesleft = 1;
-    int outbytesleft = sizeof(FT_ULong);
+    size_t inbytesleft = 1;
+    size_t outbytesleft = sizeof(FT_ULong);
 
     iconv(*cd, &inbuf, &inbytesleft, &outbuf, &outbytesleft);
 
@@ -795,9 +794,11 @@ static int prepare_charset(char *charmap, char *encoding, FT_ULong *charset, FT_
 static int prepare_charset_unicode(FT_Face face, FT_ULong *charset, FT_ULong *charcodes) {
 #ifdef HAVE_FREETYPE21
     FT_ULong  charcode;
+#else
+    int j;
 #endif
     FT_UInt   gindex;
-    int i,j;
+    int i;
 
     if (face->charmap==NULL || face->charmap->encoding!=ft_encoding_unicode) {
 	WARNING("Unicode charmap not available for this font. Very bad!");
@@ -830,6 +831,7 @@ static int prepare_charset_unicode(FT_Face face, FT_ULong *charset, FT_ULong *ch
 
     return i;
 }
+#endif
 
 static font_desc_t* init_font_desc(void)
 {
@@ -852,6 +854,7 @@ static font_desc_t* init_font_desc(void)
     desc->height = 0;
     desc->max_width = 0;
     desc->max_height = 0;
+    desc->freetype = 1;
 
     desc->tables.g = NULL;
     desc->tables.gt2 = NULL;
@@ -904,11 +907,11 @@ void free_font_desc(font_desc_t *desc)
     free(desc);
 }
 
-static int load_sub_face(char *name, FT_Face *face)
+static int load_sub_face(const char *name, int face_index, FT_Face *face)
 {
     int err = -1;
     
-    if (name) err = FT_New_Face(library, name, 0, face);
+    if (name) err = FT_New_Face(library, name, face_index, face);
 
     if (err) {
 	char *font_file = get_path("subfont.ttf");
@@ -917,7 +920,7 @@ static int load_sub_face(char *name, FT_Face *face)
 	if (err) {
 	    err = FT_New_Face(library, MPLAYER_DATADIR "/subfont.ttf", 0, face);
 	    if (err) {
-	        mp_msg(MSGT_OSD, MSGL_ERR, "New_Face failed. Maybe the font path is wrong.\nPlease supply the text font file (~/.mplayer/subfont.ttf).\n" );
+	        mp_msg(MSGT_OSD, MSGL_ERR, MSGTR_LIBVO_FONT_LOAD_FT_NewFaceFailed);
 		return -1;
 	    }
 	}
@@ -928,7 +931,7 @@ static int load_sub_face(char *name, FT_Face *face)
 static int load_osd_face(FT_Face *face)
 {
     if ( FT_New_Memory_Face(library, osd_font_pfb, sizeof(osd_font_pfb), 0, face) ) {
-	mp_msg(MSGT_OSD, MSGL_ERR, "New_Memory_Face failed..\n");
+	mp_msg(MSGT_OSD, MSGL_ERR, MSGTR_LIBVO_FONT_LOAD_FT_NewMemoryFaceFailed);
 	return -1;
     }
     return 0;
@@ -938,7 +941,7 @@ int kerning(font_desc_t *desc, int prevc, int c)
 {
     FT_Vector kern;
     
-    if (!vo_font->dynamic) return 0;
+    if (!desc->dynamic) return 0;
     if (prevc < 0 || c < 0) return 0;
     if (desc->font[prevc] != desc->font[c]) return 0;
     if (desc->font[prevc] == -1 || desc->font[c] == -1) return 0;
@@ -951,14 +954,14 @@ int kerning(font_desc_t *desc, int prevc, int c)
     return f266ToInt(kern.x);
 }
 
-font_desc_t* read_font_desc_ft(char *fname, int movie_width, int movie_height)
+font_desc_t* read_font_desc_ft(const char *fname, int face_index, int movie_width, int movie_height, float font_scale_factor)
 {
-    font_desc_t *desc;
+    font_desc_t *desc = NULL;
 
     FT_Face face;
 
-    FT_ULong my_charset[MAX_CHARSET_SIZE]; /* characters we want to render; Unicode */
-    FT_ULong my_charcodes[MAX_CHARSET_SIZE]; /* character codes in 'encoding' */
+    FT_ULong *my_charset = malloc(MAX_CHARSET_SIZE * sizeof(FT_ULong)); /* characters we want to render; Unicode */
+    FT_ULong *my_charcodes = malloc(MAX_CHARSET_SIZE * sizeof(FT_ULong)); /* character codes in 'encoding' */
 
     char *charmap = "ucs-4";
     int err;
@@ -971,6 +974,11 @@ font_desc_t* read_font_desc_ft(char *fname, int movie_width, int movie_height)
     float subtitle_font_ppem;
     float osd_font_ppem;
 
+    if (my_charset == NULL || my_charcodes == NULL) {
+	mp_msg(MSGT_OSD, MSGL_ERR, "subtitle font: malloc failed.\n");
+	goto err_out;
+    }
+
     switch (subtitle_autoscale) {
     case 1:
 	movie_size = movie_height;
@@ -986,8 +994,8 @@ font_desc_t* read_font_desc_ft(char *fname, int movie_width, int movie_height)
 	break;
     }
 
-    subtitle_font_ppem = movie_size*text_font_scale_factor/100.0;
-    osd_font_ppem = movie_size*osd_font_scale_factor/100.0;
+    subtitle_font_ppem = movie_size*font_scale_factor/100.0;
+    osd_font_ppem = movie_size*(font_scale_factor+1)/100.0;
 
     if (subtitle_font_ppem < 5) subtitle_font_ppem = 5;
     if (osd_font_ppem < 5) osd_font_ppem = 5;
@@ -1003,18 +1011,19 @@ font_desc_t* read_font_desc_ft(char *fname, int movie_width, int movie_height)
     }
 
     desc = init_font_desc();
-    if(!desc) return NULL;
+    if(!desc) goto err_out;
 
 //    t=GetTimer();
 
     /* generate the subtitle font */
-    err = load_sub_face(fname, &face);
+    err = load_sub_face(fname, face_index, &face);
     if (err) {
-	mp_msg(MSGT_OSD, MSGL_WARN, "subtitle font: load_sub_face failed.\n");
+	mp_msg(MSGT_OSD, MSGL_WARN, MSGTR_LIBVO_FONT_LOAD_FT_SubFaceFailed);
 	goto gen_osd;
     }
     desc->face_cnt++;
 
+#ifdef CONFIG_ICONV
     if (unicode) {
 	charset_size = prepare_charset_unicode(face, my_charset, my_charcodes);
     } else {
@@ -1026,10 +1035,12 @@ font_desc_t* read_font_desc_ft(char *fname, int movie_width, int movie_height)
     }
 
     if (charset_size < 0) {
-	mp_msg(MSGT_OSD, MSGL_ERR, "subtitle font: prepare_charset failed.\n");
-	free_font_desc(desc);
-	return NULL;
+	mp_msg(MSGT_OSD, MSGL_ERR, MSGTR_LIBVO_FONT_LOAD_FT_SubFontCharsetFailed);
+	goto err_out;
     }
+#else
+    goto err_out;
+#endif
 
 //    fprintf(stderr, "fg: prepare t = %lf\n", GetTimer()-t);
 
@@ -1038,9 +1049,8 @@ font_desc_t* read_font_desc_ft(char *fname, int movie_width, int movie_height)
 		       subtitle_font_thickness, subtitle_font_radius);
 
     if (err) {
-	mp_msg(MSGT_OSD, MSGL_ERR, "Cannot prepare subtitle font.\n");
-	free_font_desc(desc);
-	return NULL;
+	mp_msg(MSGT_OSD, MSGL_ERR, MSGTR_LIBVO_FONT_LOAD_FT_CannotPrepareSubtitleFont);
+	goto err_out;
     }
 
 gen_osd:
@@ -1048,8 +1058,7 @@ gen_osd:
     /* generate the OSD font */
     err = load_osd_face(&face);
     if (err) {
-	free_font_desc(desc);
-	return NULL;
+	goto err_out;
     }
     desc->face_cnt++;
 
@@ -1058,17 +1067,15 @@ gen_osd:
 		       subtitle_font_thickness, subtitle_font_radius);
     
     if (err) {
-	mp_msg(MSGT_OSD, MSGL_ERR, "Cannot prepare OSD font.\n");
-	free_font_desc(desc);
-	return NULL;
+	mp_msg(MSGT_OSD, MSGL_ERR, MSGTR_LIBVO_FONT_LOAD_FT_CannotPrepareOSDFont);
+	goto err_out;
     }
 
     err = generate_tables(desc, subtitle_font_thickness, subtitle_font_radius);
     
     if (err) {
-	mp_msg(MSGT_OSD, MSGL_ERR, "Cannot generate tables.\n");
-	free_font_desc(desc);
-	return NULL;
+	mp_msg(MSGT_OSD, MSGL_ERR, MSGTR_LIBVO_FONT_LOAD_FT_CannotGenerateTables);
+	goto err_out;
     }
 
     // final cleanup
@@ -1086,7 +1093,16 @@ gen_osd:
 	    desc->font[i] = desc->font[j];
 	}
     }
+    free(my_charset);
+    free(my_charcodes);
     return desc;
+
+err_out:
+    if (desc)
+      free_font_desc(desc);
+    free(my_charset);
+    free(my_charcodes);
+    return NULL;
 }
 
 int init_freetype(void)
@@ -1113,20 +1129,23 @@ int done_freetype(void)
     
     err = FT_Done_FreeType(library);
     if (err) {
-	mp_msg(MSGT_OSD, MSGL_ERR, "FT_Done_FreeType failed.\n");
+	mp_msg(MSGT_OSD, MSGL_ERR, MSGTR_LIBVO_FONT_LOAD_FT_DoneFreeTypeFailed);
 	return -1;
     }
 
     return 0;
 }
 
-void load_font_ft(int width, int height) 
+void load_font_ft(int width, int height, font_desc_t** fontp, const char *font_name, float font_scale_factor)
 {
-#ifdef HAVE_FONTCONFIG
+#ifdef CONFIG_FONTCONFIG
     FcPattern *fc_pattern;
+    FcPattern *fc_pattern2;
     FcChar8 *s;
+    int face_index;
     FcBool scalable;
 #endif
+    font_desc_t *vo_font = *fontp;
     vo_image_width = width;
     vo_image_height = height;
 
@@ -1135,33 +1154,35 @@ void load_font_ft(int width, int height)
 
     if (vo_font) free_font_desc(vo_font);
 
-#ifdef USE_OSD
-#ifdef HAVE_FONTCONFIG
-    if (font_fontconfig)
+#ifdef CONFIG_FONTCONFIG
+    if (font_fontconfig > 0)
     {
 	if (!font_name)
-	    font_name = "sans-serif";
+	    font_name = strdup("sans-serif");
 	FcInit();
 	fc_pattern = FcNameParse(font_name);
 	FcConfigSubstitute(0, fc_pattern, FcMatchPattern);
 	FcDefaultSubstitute(fc_pattern);
+	fc_pattern2 = fc_pattern;
 	fc_pattern = FcFontMatch(0, fc_pattern, 0);
+	FcPatternDestroy(fc_pattern2);
 	FcPatternGetBool(fc_pattern, FC_SCALABLE, 0, &scalable);
 	if (scalable != FcTrue) {
+	    FcPatternDestroy(fc_pattern);
     	    fc_pattern = FcNameParse("sans-serif");
     	    FcConfigSubstitute(0, fc_pattern, FcMatchPattern);
     	    FcDefaultSubstitute(fc_pattern);
+	    fc_pattern2 = fc_pattern;
     	    fc_pattern = FcFontMatch(0, fc_pattern, 0);
+	    FcPatternDestroy(fc_pattern2);
 	}
 	// s doesn't need to be freed according to fontconfig docs
 	FcPatternGetString(fc_pattern, FC_FILE, 0, &s);
-	vo_font=read_font_desc_ft(s, width, height);
-	free(fc_pattern);
+	FcPatternGetInteger(fc_pattern, FC_INDEX, 0, &face_index);
+	*fontp=read_font_desc_ft(s, face_index, width, height, font_scale_factor);
+	FcPatternDestroy(fc_pattern);
     }
     else
 #endif
-    vo_font=read_font_desc_ft(font_name, width, height);
-#endif
+    *fontp=read_font_desc_ft(font_name, 0, width, height, font_scale_factor);
 }
-
-#endif /* HAVE_FREETYPE */
