@@ -1,20 +1,33 @@
 /*
- *  video_out.h
+ * Copyright (C) Aaron Holtzman - Aug 1999
+ * Strongly modified, most parts rewritten: A'rpi/ESP-team - 2000-2001
+ * (C) MPlayer developers
  *
- *      Copyright (C) Aaron Holtzman - Aug 1999
- *	Strongly modified, most parts rewritten: A'rpi/ESP-team - 2000-2001
- *	(C) MPlayer Developers
+ * This file is part of MPlayer.
  *
+ * MPlayer is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * MPlayer is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with MPlayer; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
- 
-#ifndef __VIDEO_OUT_H
-#define __VIDEO_OUT_H 1
+
+#ifndef MPLAYER_VIDEO_OUT_H
+#define MPLAYER_VIDEO_OUT_H
 
 #include <inttypes.h>
 #include <stdarg.h>
 
 //#include "font_load.h"
-#include "img_format.h"
+#include "libmpcodecs/img_format.h"
 //#include "vidix/vidix.h"
 
 #define VO_EVENT_EXPOSE 1
@@ -31,7 +44,6 @@
 #define VOCTRL_GUI_NOWINDOW 19
 /* used to switch to fullscreen */
 #define VOCTRL_FULLSCREEN 5
-#define VOCTRL_SCREENSHOT 6
 /* signal a device pause */
 #define VOCTRL_PAUSE 7
 /* start/resume playback */
@@ -58,6 +70,17 @@
 #define VOCTRL_ONTOP 25
 #define VOCTRL_ROOTWIN 26
 #define VOCTRL_BORDER 27
+#define VOCTRL_DRAW_EOSD 28
+#define VOCTRL_GET_EOSD_RES 29
+typedef struct {
+  int w, h; // screen dimensions, including black borders
+  int mt, mb, ml, mr; // borders (top, bottom, left, right)
+} mp_eosd_res_t;
+
+#define VOCTRL_SET_DEINTERLACE 30
+#define VOCTRL_GET_DEINTERLACE 31
+
+#define VOCTRL_UPDATE_SCREENINFO 32
 
 // Vo can be used by xover
 #define VOCTRL_XOVERLAY_SUPPORT 22
@@ -100,7 +123,7 @@ typedef struct vo_info_s
 
 typedef struct vo_functions_s
 {
-	vo_info_t *info;
+	const vo_info_t *info;
 	/*
 	 * Preinitializes driver (real INITIALIZATION)
 	 *   arg - currently it's vo_subdevice
@@ -166,19 +189,23 @@ typedef struct vo_functions_s
 
 } vo_functions_t;
 
-char *vo_format_name(int format);
-int vo_init(void);
-
-vo_functions_t* init_best_video_out(char** vo_list);
+const vo_functions_t* init_best_video_out(char** vo_list);
+int config_video_out(const vo_functions_t *vo, uint32_t width, uint32_t height,
+                     uint32_t d_width, uint32_t d_height, uint32_t flags,
+                     char *title, uint32_t format);
 void list_video_out(void);
 
 // NULL terminated array of all drivers
-extern vo_functions_t* video_out_drivers[];
+extern const vo_functions_t* const video_out_drivers[];
 
 extern int vo_flags;
 
 extern int vo_config_count;
 
+extern int xinerama_screen;
+extern int xinerama_x;
+extern int xinerama_y;
+
 // correct resolution/bpp on screen:  (should be autodetected by vo_init())
 extern int vo_depthonscreen;
 extern int vo_screenwidth;
@@ -205,6 +232,7 @@ extern int vo_rootwin;
 extern int vo_ontop;
 extern int vo_border;
 
+extern int vo_gamma_gamma;
 extern int vo_gamma_brightness;
 extern int vo_gamma_saturation;
 extern int vo_gamma_contrast;
@@ -213,7 +241,6 @@ extern int vo_gamma_red_intensity;
 extern int vo_gamma_green_intensity;
 extern int vo_gamma_blue_intensity;
 
-extern int vo_mouse_timer_const;
 extern int vo_nomouse_input;
 
 extern int vo_pts;
@@ -225,20 +252,27 @@ extern int vo_colorkey;
 
 extern int WinID;
 
-#if defined(HAVE_FBDEV)||defined(HAVE_VESA) 
-
 typedef struct {
         float min;
 	float max;
 	} range_t;
 
-extern float range_max(range_t *r);
-extern int in_range(range_t *r, float f);
-extern range_t *str2range(char *s);
+float range_max(range_t *r);
+int in_range(range_t *r, float f);
+range_t *str2range(char *s);
 extern char *monitor_hfreq_str;
 extern char *monitor_vfreq_str;
 extern char *monitor_dotclock_str;
 
-#endif 
-		
-#endif
+struct keymap {
+  int from;
+  int to;
+};
+int lookup_keymap_table(const struct keymap *map, int key);
+struct vo_rect {
+  int left, right, top, bottom, width, height;
+};
+void calc_src_dst_rects(int src_width, int src_height, struct vo_rect *src, struct vo_rect *dst,
+                        struct vo_rect *borders, const struct vo_rect *crop);
+
+#endif /* MPLAYER_VIDEO_OUT_H */
