@@ -1,3 +1,21 @@
+/*
+ * This file is part of MPlayer.
+ *
+ * MPlayer is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * MPlayer is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with MPlayer; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
+
 /*
 	vo_macosx.m
 	by Nicolas Plourde <nicolasplourde@gmail.com>
@@ -9,7 +27,14 @@
 #import "vo_macosx.h"
 #include <sys/types.h>
 #include <sys/ipc.h>
-#include <sys/shm.h>
+#include <sys/mman.h>
+#include <unistd.h>
+#include <CoreServices/CoreServices.h>
+//special workaround for Apple bug #6267445
+//(OSServices Power API disabled in OSServices.h for 64bit systems)
+#ifndef __POWER__
+#include <CoreServices/../Frameworks/OSServices.framework/Headers/Power.h>
+#endif
 
 //MPLAYER
 #include "config.h"
@@ -19,6 +44,9 @@
 #include "aspect.h"
 #include "mp_msg.h"
 #include "m_option.h"
+#include "mp_fifo.h"
+#include "libvo/sub.h"
+#include "subopt-helper.h"
 
 #include "input/input.h"
 #include "input/mouse.h"
@@ -26,25 +54,30 @@
 #include "osdep/keycodes.h"
 
 //Cocoa
-NSProxy *mplayerosxProxy;
+NSDistantObject *mplayerosxProxy;
+id <MPlayerOSXVOProto> mplayerosxProto;
 MPlayerOpenGLView *mpGLView;
 NSAutoreleasePool *autoreleasepool;
 OSType pixelFormat;
 
 //shared memory
-int shm_id;
-struct shmid_ds shm_desc;
+int shm_fd;
 BOOL shared_buffer = false;
+#define DEFAULT_BUFFER_NAME "mplayerosx"
+static char *buffer_name;
 
 //Screen
-int screen_id;
-BOOL screen_force;
+int screen_id = -1;
 NSRect screen_frame;
 NSScreen *screen_handle;
 NSArray *screen_array;
 
 //image
 unsigned char *image_data;
+// For double buffering
+static uint8_t image_page = 0;
+static unsigned char *image_datas[2];
+
 static uint32_t image_width;
 static uint32_t image_height;
 static uint32_t image_depth;
@@ -52,21 +85,19 @@ static uint32_t image_bytes;
 static uint32_t image_format;
 
 //vo
-extern int vo_rootwin;
-extern int vo_ontop;
-extern int vo_fs;
 static int isFullscreen;
 static int isOntop;
 static int isRootwin;
 extern float monitor_aspect;
-extern int vo_keepaspect;
 extern float movie_aspect;
 static float old_movie_aspect;
-extern float vo_panscan;
+extern int enable_mouse_movements;
 
 static float winAlpha = 1;
 static int int_pause = 0;
 
+static BOOL isLeopardOrLater;
+
 static vo_info_t info = 
 {
 	"Mac OSX Core Video",
@@ -77,9 +108,6 @@ static vo_info_t info =
 
 LIBVO_EXTERN(macosx)
 
-extern void mplayer_put_key(int code);
-extern void vo_draw_text(int dxs,int dys,void (*draw_alpha)(int x0,int y0, int w,int h, unsigned char* src, unsigned char *srca, int stride));
-
 static void draw_alpha(int x0, int y0, int w, int h, unsigned char *src, unsigned char *srca, int stride)
 {
 	switch (image_format)
@@ -95,21 +123,22 @@ static void draw_alpha(int x0, int y0, int w, int h, unsigned char *src, unsigne
 
 static int config(uint32_t width, uint32_t height, uint32_t d_width, uint32_t d_height, uint32_t flags, char *title, uint32_t format)
 {
-	int i;
 	
 	//init screen
 	screen_array = [NSScreen screens];
 	if(screen_id < [screen_array count])
 	{
-		screen_handle = [screen_array objectAtIndex:screen_id];
+		screen_handle = [screen_array objectAtIndex:(screen_id < 0 ? 0 : screen_id)];
 	}
 	else
 	{
 		mp_msg(MSGT_VO, MSGL_FATAL, "Get device error: Device ID %d do not exist, falling back to main device.\n", screen_id);
 		screen_handle = [screen_array objectAtIndex:0];
-		screen_id = 0;
+		screen_id = -1;
 	}
 	screen_frame = [screen_handle frame];
+	vo_screenwidth = screen_frame.size.width;
+	vo_screenheight = screen_frame.size.height;
 
 	//misc mplayer setup
 	image_width = width;
@@ -125,10 +154,15 @@ static int config(uint32_t width, uint32_t height, uint32_t d_width, uint32_t d_
 			break;
 	}
 	image_bytes = (image_depth + 7) / 8;
-	image_data = (unsigned char*)malloc(image_width*image_height*image_bytes);
 		
 	if(!shared_buffer)
 	{		
+		image_data = malloc(image_width*image_height*image_bytes);
+		image_datas[0] = image_data;
+		if (vo_doublebuffering)
+			image_datas[1] = malloc(image_width*image_height*image_bytes);
+		image_page = 0;
+
 		monitor_aspect = (float)screen_frame.size.width/(float)screen_frame.size.height;
 		
 		//set aspect
@@ -149,25 +183,52 @@ static int config(uint32_t width, uint32_t height, uint32_t d_width, uint32_t d_
 	}
 	else
 	{
+		mp_msg(MSGT_VO, MSGL_INFO, "VO: [macosx] writing output to a shared buffer "
+				"named \"%s\".\n",buffer_name);
+		
 		movie_aspect = (float)d_width/(float)d_height;
-				
-		shm_id = shmget(9849, image_width*image_height*image_bytes, IPC_CREAT | 0666);
-		if (shm_id == -1)
+		
+		// create shared memory
+		shm_fd = shm_open(buffer_name, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR);
+		if (shm_fd == -1)
 		{
-			perror("vo_mplayer shmget: ");
+			mp_msg(MSGT_VO, MSGL_FATAL, 
+				   "vo_macosx: failed to open shared memory. Error: %s\n", strerror(errno));
 			return 1;
 		}
 		
-		image_data = shmat(shm_id, NULL, 0);
-		if (!image_data)
-		{	
-			perror("vo_mplayer shmat: ");
+		
+		if (ftruncate(shm_fd, image_width*image_height*image_bytes) == -1)
+		{
+			mp_msg(MSGT_VO, MSGL_FATAL, 
+				   "vo_macosx: failed to size shared memory, possibly already in use. Error: %s\n", strerror(errno));
+			shm_unlink(buffer_name);
 			return 1;
 		}
 		
+		image_data = mmap(NULL, image_width*image_height*image_bytes,
+					PROT_READ | PROT_WRITE, MAP_SHARED, shm_fd, 0);
+		
+		if (image_data == MAP_FAILED)
+		{
+			mp_msg(MSGT_VO, MSGL_FATAL, 
+				   "vo_macosx: failed to map shared memory. Error: %s\n", strerror(errno));
+			shm_unlink(buffer_name);
+			return 1;
+		}		
+		
 		//connnect to mplayerosx
-		mplayerosxProxy=[NSConnection rootProxyForConnectionWithRegisteredName:@"mplayerosx" host:nil];
-		[mplayerosxProxy startWithWidth: image_width withHeight: image_height withBytes: image_bytes withAspect:(int)(movie_aspect*100)];
+		mplayerosxProxy=[NSConnection rootProxyForConnectionWithRegisteredName:[NSString stringWithCString:buffer_name] host:nil];
+		if ([mplayerosxProxy conformsToProtocol:@protocol(MPlayerOSXVOProto)]) {
+			[mplayerosxProxy setProtocolForProxy:@protocol(MPlayerOSXVOProto)];
+			mplayerosxProto = (id <MPlayerOSXVOProto>)mplayerosxProxy;
+			[mplayerosxProto startWithWidth: image_width withHeight: image_height withBytes: image_bytes withAspect:(int)(movie_aspect*100)];
+		}
+		else {
+			[mplayerosxProxy release];
+			mplayerosxProxy = nil;
+			mplayerosxProto = nil;
+		}
 	}
 	return 0;
 }
@@ -185,14 +246,19 @@ static void draw_osd(void)
 static void flip_page(void)
 {
 	if(shared_buffer)
-		[mplayerosxProxy render];
-	else
+		[mplayerosxProto render];
+	else {
+		[mpGLView setCurrentTexture];
 		[mpGLView render];
+		if (vo_doublebuffering) {
+			image_page = 1 - image_page;
+			image_data = image_datas[image_page];
+		}
+	}
 }
 
 static int draw_slice(uint8_t *src[], int stride[], int w,int h,int x,int y)
 {
-	[mpGLView setCurrentTexture];
 	return 0;
 }
 
@@ -203,7 +269,7 @@ static int draw_frame(uint8_t *src[])
 	{
 		case IMGFMT_BGR32:
 		case IMGFMT_RGB32:
-			memcpy(image_data, src[0], image_width*image_height*image_bytes);
+			fast_memcpy(image_data, src[0], image_width*image_height*image_bytes);
 			break;
 
 		case IMGFMT_YUY2:
@@ -211,9 +277,6 @@ static int draw_frame(uint8_t *src[])
 			break;
 	}
 	
-	if(!shared_buffer)
-		[mpGLView setCurrentTexture];
-	
 	return 0;
 }
 
@@ -239,56 +302,91 @@ static void uninit(void)
 {
 	if(shared_buffer)
 	{
-		[mplayerosxProxy stop];
-
-		if (shmdt(image_data) == -1)
-			mp_msg(MSGT_VO, MSGL_FATAL, "uninit: shmdt failed\n");
-	
-		if (shmctl(shm_id, IPC_RMID, &shm_desc) == -1)
-			mp_msg(MSGT_VO, MSGL_FATAL, "uninit: shmctl failed\n");
+		[mplayerosxProto stop];
+		mplayerosxProto = nil;
+		[mplayerosxProxy release];
+		mplayerosxProxy = nil;
+		
+		if (munmap(image_data, image_width*image_height*image_bytes) == -1)
+			mp_msg(MSGT_VO, MSGL_FATAL, "uninit: munmap failed. Error: %s\n", strerror(errno));
+		
+		if (shm_unlink(buffer_name) == -1)
+			mp_msg(MSGT_VO, MSGL_FATAL, "uninit: shm_unlink failed. Error: %s\n", strerror(errno));
+		
 	}
 
-	SetSystemUIMode( kUIModeNormal, 0);
-	CGDisplayShowCursor(kCGDirectMainDisplay);
-	
-	[autoreleasepool release];
+    SetSystemUIMode( kUIModeNormal, 0);
+    CGDisplayShowCursor(kCGDirectMainDisplay);
+    
+    if(mpGLView)
+    {
+        NSAutoreleasePool *finalPool;
+        mpGLView = nil;
+        [autoreleasepool release];
+        finalPool = [[NSAutoreleasePool alloc] init];
+        [NSApp nextEventMatchingMask:NSAnyEventMask untilDate:nil inMode:NSDefaultRunLoopMode dequeue:YES];
+        [finalPool release];
+    }
+    if (!shared_buffer)
+    {
+        free(image_datas[0]);
+        if (vo_doublebuffering)
+            free(image_datas[1]);
+        image_datas[0] = NULL;
+        image_datas[1] = NULL;
+        image_data = NULL;
+    }
+    
+    if (buffer_name) free(buffer_name);
+    buffer_name = NULL;
 }
 
+static opt_t subopts[] = {
+{"device_id",     OPT_ARG_INT,  &screen_id,     NULL},
+{"shared_buffer", OPT_ARG_BOOL, &shared_buffer, NULL},
+{"buffer_name",   OPT_ARG_MSTRZ,&buffer_name,   NULL},
+{NULL}
+};
+
 static int preinit(const char *arg)
 {
-	int parse_err = 0;
-
-    if(arg) 
-    {
-        char *parse_pos = (char *)&arg[0];
-        while (parse_pos[0] && !parse_err) 
-		{
-			if (strncmp (parse_pos, "device_id=", 10) == 0)
-			{
-				parse_pos = &parse_pos[10];
-				screen_id = strtol(parse_pos, &parse_pos, 0);
-				screen_force = YES;
-            }
-			if (strncmp (parse_pos, "shared_buffer", 13) == 0)
-			{
-				parse_pos = &parse_pos[13];
-				shared_buffer = YES;
-            }
-            if (parse_pos[0] == ':') parse_pos = &parse_pos[1];
-            else if (parse_pos[0]) parse_err = 1;
-        }
-    }
+	
+	// set defaults
+	screen_id = -1;
+	shared_buffer = false;
+	buffer_name = NULL;
+	
+	if (subopt_parse(arg, subopts) != 0) {
+		mp_msg(MSGT_VO, MSGL_FATAL,
+				"\n-vo macosx command line help:\n"
+				"Example: mplayer -vo macosx:device_id=1:shared_buffer:buffer_name=mybuff\n"
+				"\nOptions:\n"
+				"  device_id=<0-...>\n"
+				"    Set screen device id for fullscreen.\n"
+				"  shared_buffer\n"
+				"    Write output to a shared memory buffer instead of displaying it.\n"
+				"  buffer_name=<name>\n"
+				"    Name of the shared buffer created with shm_open() as well as\n"
+				"    the name of the NSConnection MPlayer will try to open.\n"
+				"    Setting buffer_name implicitly enables shared_buffer.\n"
+				"\n" );
+		return -1;
+	}
 
 	NSApplicationLoad();
 	autoreleasepool = [[NSAutoreleasePool alloc] init];
 	NSApp = [NSApplication sharedApplication];
+	isLeopardOrLater = floor(NSAppKitVersionNumber) > 824;
+	
+	if (!buffer_name)
+		buffer_name = strdup(DEFAULT_BUFFER_NAME);
+	else
+		shared_buffer = true;
 	
 	if(!shared_buffer)
 	{
-		#if !defined (MACOSX_FINDER_SUPPORT) || !defined (HAVE_SDL)
+		#if !defined (CONFIG_MACOSX_FINDER) || !defined (CONFIG_SDL)
 		//this chunk of code is heavily based off SDL_macosx.m from SDL 
-		//it uses an Apple private function to request foreground operation
-		void CPSEnableForegroundOperation(ProcessSerialNumber* psn);
 		ProcessSerialNumber myProc, frProc;
 		Boolean sameProc;
 
@@ -298,7 +396,7 @@ static int preinit(const char *arg)
 			{
 				if (SameProcess(&frProc, &myProc, &sameProc) == noErr && !sameProc)
 				{
-					CPSEnableForegroundOperation(&myProc);
+					TransformProcessType(&myProc, kProcessTransformToForegroundApplication);
 				}
 				SetFrontProcess(&myProc);
 			}
@@ -322,12 +420,12 @@ static int control(uint32_t request, void *data, ...)
 {
 	switch (request)
 	{
-		case VOCTRL_PAUSE: return (int_pause=1);
-		case VOCTRL_RESUME: return (int_pause=0);
+		case VOCTRL_PAUSE: return int_pause = 1;
+		case VOCTRL_RESUME: return int_pause = 0;
 		case VOCTRL_QUERY_FORMAT: return query_format(*((uint32_t*)data));
-		case VOCTRL_ONTOP: vo_ontop = (!(vo_ontop)); [mpGLView ontop]; return VO_TRUE;
+		case VOCTRL_ONTOP: vo_ontop = (!(vo_ontop)); if(!shared_buffer){ [mpGLView ontop]; } else { [mplayerosxProto ontop]; } return VO_TRUE;
 		case VOCTRL_ROOTWIN: vo_rootwin = (!(vo_rootwin)); [mpGLView rootwin]; return VO_TRUE;
-		case VOCTRL_FULLSCREEN: vo_fs = (!(vo_fs)); [mpGLView fullscreen: YES]; return VO_TRUE;
+		case VOCTRL_FULLSCREEN: vo_fs = (!(vo_fs)); if(!shared_buffer){ [mpGLView fullscreen: NO]; } else { [mplayerosxProto toggleFullscreen]; } return VO_TRUE;
 		case VOCTRL_GET_PANSCAN: return VO_TRUE;
 		case VOCTRL_SET_PANSCAN: [mpGLView panscan]; return VO_TRUE;
 	}
@@ -338,7 +436,7 @@ static int control(uint32_t request, void *data, ...)
 // NSOpenGLView Subclass
 //////////////////////////////////////////////////////////////////////////
 @implementation MPlayerOpenGLView
-- (id) preinit
+- (void) preinit
 {
 	//init menu
 	[self initMenu];
@@ -359,12 +457,12 @@ static int control(uint32_t request, void *data, ...)
 	winSizeMult = 1;
 }
 
-- (id) config
+- (void) config
 {
 	uint32_t d_width;
 	uint32_t d_height;
 	
-	long swapInterval = 1;
+	GLint swapInterval = 1;
 	
 	NSRect frame;
 	CVReturn error = kCVReturnSuccess;
@@ -382,15 +480,20 @@ static int control(uint32_t request, void *data, ...)
 	[glContext setView:self];
 	[glContext makeCurrentContext];	
 	
-	error = CVPixelBufferCreateWithBytes( NULL, image_width, image_height, pixelFormat, image_data, image_width*image_bytes, NULL, NULL, NULL, &currentFrameBuffer);
+	error = CVPixelBufferCreateWithBytes(NULL, image_width, image_height, pixelFormat, image_datas[0], image_width*image_bytes, NULL, NULL, NULL, &frameBuffers[0]);
 	if(error != kCVReturnSuccess)
 		mp_msg(MSGT_VO, MSGL_ERR,"Failed to create Pixel Buffer(%d)\n", error);
+	if (vo_doublebuffering) {
+		error = CVPixelBufferCreateWithBytes(NULL, image_width, image_height, pixelFormat, image_datas[1], image_width*image_bytes, NULL, NULL, NULL, &frameBuffers[1]);
+		if(error != kCVReturnSuccess)
+			mp_msg(MSGT_VO, MSGL_ERR,"Failed to create Pixel Double Buffer(%d)\n", error);
+	}
 	
 	error = CVOpenGLTextureCacheCreate(NULL, 0, [glContext CGLContextObj], [[self pixelFormat] CGLPixelFormatObj], 0, &textureCache);
 	if(error != kCVReturnSuccess)
 		mp_msg(MSGT_VO, MSGL_ERR,"Failed to create OpenGL texture Cache(%d)\n", error);
 	
-	error = CVOpenGLTextureCacheCreateTextureFromImage(	NULL, textureCache, currentFrameBuffer, 0, &texture);
+	error = CVOpenGLTextureCacheCreateTextureFromImage(NULL, textureCache, frameBuffers[image_page], 0, &texture);
 	if(error != kCVReturnSuccess)
 		mp_msg(MSGT_VO, MSGL_ERR,"Failed to create OpenGL texture(%d)\n", error);
 	
@@ -413,7 +516,7 @@ static int control(uint32_t request, void *data, ...)
 */
 - (void)initMenu
 {
-	NSMenu *menu;
+	NSMenu *menu, *aspectMenu;
 	NSMenuItem *menuItem;
 	
 	[NSApp setMainMenu:[[NSMenu alloc] init]];
@@ -430,7 +533,6 @@ static int control(uint32_t request, void *data, ...)
 	kFullScreenCmd = menuItem;
 	menuItem = (NSMenuItem *)[NSMenuItem separatorItem]; [menu addItem:menuItem];
 	
-		NSMenu	*aspectMenu;
 		aspectMenu = [[NSMenu alloc] initWithTitle:@"Aspect Ratio"];
 		menuItem = [[NSMenuItem alloc] initWithTitle:@"Keep" action:@selector(menuAction:) keyEquivalent:@""]; [aspectMenu addItem:menuItem];
 		if(vo_keepaspect) [menuItem setState:NSOnState];
@@ -490,7 +592,7 @@ static int control(uint32_t request, void *data, ...)
 	if(sender == kHalfScreenCmd)
 	{
 		if(isFullscreen) {
-			vo_fs = (!(vo_fs)); [self fullscreen:YES];
+			vo_fs = (!(vo_fs)); [self fullscreen:NO];
 		}
 		
 		winSizeMult = 0.5;
@@ -502,7 +604,7 @@ static int control(uint32_t request, void *data, ...)
 	if(sender == kNormalScreenCmd)
 	{
 		if(isFullscreen) {
-			vo_fs = (!(vo_fs)); [self fullscreen:YES];
+			vo_fs = (!(vo_fs)); [self fullscreen:NO];
 		}
 		
 		winSizeMult = 1;
@@ -514,7 +616,7 @@ static int control(uint32_t request, void *data, ...)
 	if(sender == kDoubleScreenCmd)
 	{
 		if(isFullscreen) {
-			vo_fs = (!(vo_fs)); [self fullscreen:YES];
+			vo_fs = (!(vo_fs)); [self fullscreen:NO];
 		}
 		
 		winSizeMult = 2;
@@ -526,7 +628,7 @@ static int control(uint32_t request, void *data, ...)
 	if(sender == kFullScreenCmd)
 	{
 		vo_fs = (!(vo_fs));
-		[self fullscreen:YES];
+		[self fullscreen:NO];
 	}
 
 	if(sender == kKeepAspectCmd)
@@ -647,18 +749,20 @@ static int control(uint32_t request, void *data, ...)
 		if((d_height*aspectX)>(frame.size.height))
 		{
 			padding = (frame.size.width - d_width*aspectY)/2;
-			textureFrame = NSMakeRect(padding, 0, d_width*aspectY+padding, d_height*aspectY);
+			textureFrame = NSMakeRect(padding, 0, d_width*aspectY, d_height*aspectY);
 		}
 		else
 		{
 			padding = ((frame.size.height) - d_height*aspectX)/2;
-			textureFrame = NSMakeRect(0, padding, d_width*aspectX, d_height*aspectX+padding);
+			textureFrame = NSMakeRect(0, padding, d_width*aspectX, d_height*aspectX);
 		}
 	}
 	else
 	{
 		textureFrame = frame;
 	}
+	vo_dwidth = textureFrame.size.width;
+	vo_dheight = textureFrame.size.height;
 }
 
 /*
@@ -666,6 +770,8 @@ static int control(uint32_t request, void *data, ...)
 */ 
 - (void) render
 {
+	int curTime;
+
 	glClear(GL_COLOR_BUFFER_BIT);	
 	
 	glEnable(CVOpenGLTextureGetTarget(texture));
@@ -674,9 +780,9 @@ static int control(uint32_t request, void *data, ...)
 	glColor3f(1,1,1);
 	glBegin(GL_QUADS);
 	glTexCoord2f(upperLeft[0], upperLeft[1]); glVertex2i(	textureFrame.origin.x-(vo_panscan_x >> 1), textureFrame.origin.y-(vo_panscan_y >> 1));
-	glTexCoord2f(lowerLeft[0], lowerLeft[1]); glVertex2i(	textureFrame.origin.x-(vo_panscan_x >> 1), textureFrame.size.height+(vo_panscan_y >> 1));
-	glTexCoord2f(lowerRight[0], lowerRight[1]); glVertex2i(	textureFrame.size.width+(vo_panscan_x >> 1), textureFrame.size.height+(vo_panscan_y >> 1));
-	glTexCoord2f(upperRight[0], upperRight[1]); glVertex2i(	textureFrame.size.width+(vo_panscan_x >> 1), textureFrame.origin.y-(vo_panscan_y >> 1));
+	glTexCoord2f(lowerLeft[0], lowerLeft[1]); glVertex2i(textureFrame.origin.x-(vo_panscan_x >> 1), NSMaxY(textureFrame)+(vo_panscan_y >> 1));
+	glTexCoord2f(lowerRight[0], lowerRight[1]); glVertex2i(NSMaxX(textureFrame)+(vo_panscan_x >> 1), NSMaxY(textureFrame)+(vo_panscan_y >> 1));
+	glTexCoord2f(upperRight[0], upperRight[1]); glVertex2i(NSMaxX(textureFrame)+(vo_panscan_x >> 1), textureFrame.origin.y-(vo_panscan_y >> 1));
 	glEnd();
 	glDisable(CVOpenGLTextureGetTarget(texture));
 	
@@ -705,29 +811,25 @@ static int control(uint32_t request, void *data, ...)
 	
 	glFlush();
 	
-	//auto hide mouse cursor and futur on-screen control?
+	curTime  = TickCount()/60;
+
+	//auto hide mouse cursor (and future on-screen control?)
 	if(isFullscreen && !mouseHide && !isRootwin)
 	{
-		int curTime = TickCount()/60;
-		static int lastTime = 0;
-		
-		if( ((curTime - lastTime) >= 5) || (lastTime == 0) )
+		if( ((curTime - lastMouseHide) >= 5) || (lastMouseHide == 0) )
 		{
 			CGDisplayHideCursor(kCGDirectMainDisplay);
-			mouseHide = YES;
-			lastTime = curTime;
+			mouseHide = TRUE;
+			lastMouseHide = curTime;
 		}
 	}
 	
 	//update activity every 30 seconds to prevent
 	//screensaver from starting up.
-	int curTime = TickCount()/60;
-	static int lastTime = 0;
-		
-	if( ((curTime - lastTime) >= 30) || (lastTime == 0) )
+	if( ((curTime - lastScreensaverUpdate) >= 30) || (lastScreensaverUpdate == 0) )
 	{
 		UpdateSystemActivity(UsrActivity);
-		lastTime = curTime;
+		lastScreensaverUpdate = curTime;
 	}
 }
 
@@ -738,7 +840,7 @@ static int control(uint32_t request, void *data, ...)
 {
 	CVReturn error = kCVReturnSuccess;
 	
-	error = CVOpenGLTextureCacheCreateTextureFromImage (NULL, textureCache,  currentFrameBuffer,  0, &texture);
+	error = CVOpenGLTextureCacheCreateTextureFromImage(NULL, textureCache, frameBuffers[image_page], 0, &texture);
 	if(error != kCVReturnSuccess)
 		mp_msg(MSGT_VO, MSGL_ERR,"Failed to create OpenGL texture(%d)\n", error);
 
@@ -761,11 +863,6 @@ static int control(uint32_t request, void *data, ...)
 	static NSRect old_frame;
 	static NSRect old_view_frame;
 	
-	if(screen_force)
-		screen_frame = [screen_handle frame];
-	else
-		screen_frame = [[window screen] frame];
-
 	panscan_calc();
 			
 	//go fullscreen
@@ -779,6 +876,14 @@ static int control(uint32_t request, void *data, ...)
 		}
 		
 		old_frame = [window frame];	//save main window size & position
+		if(screen_id >= 0)
+			screen_frame = [screen_handle frame];
+		else {
+			screen_frame = [[window screen] frame];
+			vo_screenwidth = screen_frame.size.width;
+			vo_screenheight = screen_frame.size.height;
+		}
+
 		[window setFrame:screen_frame display:YES animate:animate]; //zoom-in window with nice useless sfx
 		old_view_frame = [self bounds];
 		
@@ -855,7 +960,19 @@ static int control(uint32_t request, void *data, ...)
 - (void) check_events
 {
 	event = [NSApp nextEventMatchingMask:NSAnyEventMask untilDate:[NSDate dateWithTimeIntervalSinceNow:0.0001] inMode:NSEventTrackingRunLoopMode dequeue:YES];
+	if (event == nil)
+		return;
 	[NSApp sendEvent:event];
+	// Without SDL's bootstrap code (include SDL.h in mplayer.c),
+	// on Leopard, we got trouble to get the play window auto focused
+	// when app is actived. Following code fix this problem.
+#ifndef CONFIG_SDL
+	if (isLeopardOrLater && [event type] == NSAppKitDefined
+			&& [event subtype] == NSApplicationActivatedEventType) {
+		[window makeMainWindow];
+		[window makeKeyAndOrderFront:mpGLView];
+	}
+#endif
 }
 
 /*
@@ -914,7 +1031,7 @@ static int control(uint32_t request, void *data, ...)
 		case 0x7E: key = KEY_UP; break;
 		case 0x43: key = '*'; break;
 		case 0x4B: key = '/'; break;
-		case 0x4C: key = KEY_BACKSPACE; break;
+		case 0x4C: key = KEY_KPENTER; break;
 		case 0x41: key = KEY_KPDEC; break;
 		case 0x52: key = KEY_KP0; break;
 		case 0x53: key = KEY_KP1; break;
@@ -941,6 +1058,16 @@ static int control(uint32_t request, void *data, ...)
 		CGDisplayShowCursor(kCGDirectMainDisplay);
 		mouseHide = NO;
 	}
+	if (enable_mouse_movements && !isRootwin) {
+		NSPoint p =[self convertPoint:[theEvent locationInWindow] fromView:nil];
+		if ([self mouse:p inRect:textureFrame]) {
+                	char cmdstr[40];
+                	snprintf(cmdstr, sizeof(cmdstr), "set_mouse_pos %i %i",
+			         (int)(vo_fs ? p.x : (p.x - textureFrame.origin.x)),
+			         (int)(vo_fs ? [self frame].size.height - p.y: (NSMaxY(textureFrame) - p.y)));
+                	mp_input_queue_cmd(mp_input_parse_cmd(cmdstr));
+		}
+	}
 }
 
 - (void) mouseDown: (NSEvent *) theEvent
@@ -948,16 +1075,31 @@ static int control(uint32_t request, void *data, ...)
 	[self mouseEvent: theEvent];
 }
 
+- (void) mouseUp: (NSEvent *) theEvent
+{
+	[self mouseEvent: theEvent];
+}
+
 - (void) rightMouseDown: (NSEvent *) theEvent
 {
 	[self mouseEvent: theEvent];
 }
 
+- (void) rightMouseUp: (NSEvent *) theEvent
+{
+	[self mouseEvent: theEvent];
+}
+
 - (void) otherMouseDown: (NSEvent *) theEvent
 {
 	[self mouseEvent: theEvent];
 }
 
+- (void) otherMouseUp: (NSEvent *) theEvent
+{
+	[self mouseEvent: theEvent];
+}
+
 - (void) scrollWheel: (NSEvent *) theEvent
 {
 	if([theEvent deltaY] > 0)
@@ -968,11 +1110,27 @@ static int control(uint32_t request, void *data, ...)
 
 - (void) mouseEvent: (NSEvent *) theEvent
 {
-	switch( [theEvent buttonNumber] )
-	{ 
-		case 0: mplayer_put_key(MOUSE_BTN0);break;
-		case 1: mplayer_put_key(MOUSE_BTN1);break;
-		case 2: mplayer_put_key(MOUSE_BTN2);break;
+	if ( [theEvent buttonNumber] >= 0 && [theEvent buttonNumber] <= 9 )
+	{
+		int buttonNumber = [theEvent buttonNumber];
+		// Fix to mplayer defined button order: left, middle, right
+		if (buttonNumber == 1)
+			buttonNumber = 2;
+		else if (buttonNumber == 2)
+			buttonNumber = 1;
+		switch([theEvent type])
+		{
+			case NSLeftMouseDown:
+			case NSRightMouseDown:
+			case NSOtherMouseDown:
+				mplayer_put_key((MOUSE_BTN0 + buttonNumber) | MP_KEY_DOWN);
+				break;
+			case NSLeftMouseUp:
+			case NSRightMouseUp:
+			case NSOtherMouseUp:
+				mplayer_put_key(MOUSE_BTN0 + buttonNumber);
+				break;
+		}
 	}
 }
 
@@ -996,6 +1154,7 @@ static int control(uint32_t request, void *data, ...)
 
 - (void)windowWillClose:(NSNotification *)aNotification
 {
+    mpGLView = NULL;
 	mplayer_put_key(KEY_ESC);
 }
 @end
