@@ -1,5 +1,21 @@
-/* 
- *    output through mga_vid kernel driver
+/*
+ * video output through mga_vid kernel driver
+ *
+ * This file is part of MPlayer.
+ *
+ * MPlayer is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * MPlayer is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with MPlayer; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include <stdio.h>
@@ -7,6 +23,8 @@
 #include <string.h>
 
 #include "config.h"
+#include "mp_msg.h"
+#include "help_mp.h"
 #include "video_out.h"
 #include "video_out_internal.h"
 
@@ -20,7 +38,7 @@
 #include "sub.h"
 #include "aspect.h"
 
-static vo_info_t info = 
+static const vo_info_t info = 
 {
 	"Matrox G200/G4x0/G550 overlay (/dev/mga_vid)",
 	"mga",
@@ -28,7 +46,7 @@ static vo_info_t info =
 	"Based on some code by Aaron Holtzman <aholtzma@ess.engr.uvic.ca>"
 };
 
-LIBVO_EXTERN(mga)
+const LIBVO_EXTERN(mga)
 
 #include "mga_common.c"
 
@@ -67,7 +85,7 @@ static int config(uint32_t width, uint32_t height, uint32_t d_width, uint32_t d_
 			aspect(&d_width,&d_height,A_NOZOOM);
 			vo_fs = VO_FALSE;
 		}
-		printf("vo_mga aspect(): resized to %dx%d\n",d_width,d_height);
+		mp_msg(MSGT_VO,MSGL_INFO, MSGTR_LIBVO_MGA_AspectResized,d_width,d_height);
 	}
 
 	vo_dwidth=d_width; vo_dheight=d_height;
@@ -85,7 +103,7 @@ static int config(uint32_t width, uint32_t height, uint32_t d_width, uint32_t d_
 
 static void uninit(void)
 {
-    printf("vo: uninit!\n");
+    mp_msg(MSGT_VO,MSGL_INFO, MSGTR_LIBVO_MGA_Uninit);
     mga_uninit();
 }
 
