@@ -1,3 +1,7 @@
+
+/// \file
+/// \ingroup Options
+
 #include "config.h"
 
 #include <stdlib.h>
@@ -11,14 +15,15 @@
 #include "m_option.h"
 //#include "m_config.h"
 #include "mp_msg.h"
-#include "libmpdemux/url.h"
+#include "stream/url.h"
+#include "libavutil/avstring.h"
 
 // Don't free for 'production' atm
 #ifndef MP_DEBUG
 //#define NO_FREE
 #endif
 
-m_option_t* m_option_list_find(m_option_t* list,char* name) {
+const m_option_t* m_option_list_find(const m_option_t* list,const char* name) {
   int i;
 
   for(i = 0 ; list[i].name ; i++) {
@@ -35,7 +40,7 @@ m_option_t* m_option_list_find(m_option_t* list,char* name) {
 
 // Default function that just does a memcpy
 
-static void copy_opt(m_option_t* opt,void* dst,void* src) {
+static void copy_opt(const m_option_t* opt,void* dst,void* src) {
   if(dst && src)
     memcpy(dst,src,opt->type->size);
 }
@@ -71,7 +76,7 @@ static char* dup_printf(const char *fmt, ...) {
 
 #define VAL(x) (*(int*)(x))
 
-static int parse_flag(m_option_t* opt,char *name, char *param, void* dst, int src) {
+static int parse_flag(const m_option_t* opt,const char *name, char *param, void* dst, int src) {
   if (src == M_CONFIG_FILE) {
     if(!param) return M_OPT_MISSING_PARAM;
     if (!strcasecmp(param, "yes") ||	/* any other language? */
@@ -109,14 +114,14 @@ static int parse_flag(m_option_t* opt,char *name, char *param, void* dst, int sr
   }
 }
 
-static char* print_flag(m_option_t* opt,  void* val) {
+static char* print_flag(const m_option_t* opt,  const void* val) {
   if(VAL(val) == opt->min)
     return strdup("no");
   else
     return strdup("yes");
 }
 
-m_option_type_t m_option_type_flag = {
+const m_option_type_t m_option_type_flag = {
   "Flag",
   "need yes or no in config files",
   sizeof(int),
@@ -131,7 +136,7 @@ m_option_type_t m_option_type_flag = {
 
 // Integer
 
-static int parse_int(m_option_t* opt,char *name, char *param, void* dst, int src) {
+static int parse_int(const m_option_t* opt,const char *name, char *param, void* dst, int src) {
   long tmp_int;
   char *endptr;
   src = 0;
@@ -139,6 +144,8 @@ static int parse_int(m_option_t* opt,char *name, char *param, void* dst, int src
   if (param == NULL)
     return M_OPT_MISSING_PARAM;
 
+  tmp_int = strtol(param, &endptr, 10);
+  if (*endptr)
   tmp_int = strtol(param, &endptr, 0);
   if (*endptr) {
     mp_msg(MSGT_CFGPARSER, MSGL_ERR, "The %s option must be an integer: %s\n",name, param);
@@ -160,12 +167,12 @@ static int parse_int(m_option_t* opt,char *name, char *param, void* dst, int src
   return 1;
 }
 
-static char* print_int(m_option_t* opt,  void* val) {
+static char* print_int(const m_option_t* opt,  const void* val) {
   opt = NULL;
   return dup_printf("%d",VAL(val));
 }
 
-m_option_type_t m_option_type_int = {
+const m_option_type_t m_option_type_int = {
   "Integer",
   "",
   sizeof(int),
@@ -183,7 +190,7 @@ m_option_type_t m_option_type_int = {
 #undef VAL
 #define VAL(x) (*(double*)(x))
 
-static int parse_double(m_option_t* opt,char *name, char *param, void* dst, int src) {
+static int parse_double(const m_option_t* opt,const char *name, char *param, void* dst, int src) {
   double tmp_float;
   char* endptr;
   src = 0;
@@ -232,12 +239,12 @@ static int parse_double(m_option_t* opt,char *name, char *param, void* dst, int
   return 1;
 }
 
-static char* print_double(m_option_t* opt,  void* val) {
+static char* print_double(const m_option_t* opt,  const void* val) {
   opt = NULL;
   return dup_printf("%f",VAL(val));
 }
 
-m_option_type_t m_option_type_double = {
+const m_option_type_t m_option_type_double = {
   "Double",
   "double precission floating point number or ratio (numerator[:/]denominator)",
   sizeof(double),
@@ -253,19 +260,19 @@ m_option_type_t m_option_type_double = {
 #undef VAL
 #define VAL(x) (*(float*)(x))
 
-static int parse_float(m_option_t* opt,char *name, char *param, void* dst, int src) {
+static int parse_float(const m_option_t* opt,const char *name, char *param, void* dst, int src) {
     double tmp;
     int r= parse_double(opt, name, param, &tmp, src);
     if(r==1 && dst) VAL(dst) = tmp;
     return r;
 }
 
-static char* print_float(m_option_t* opt,  void* val) {
+static char* print_float(const m_option_t* opt,  const void* val) {
   opt = NULL;
   return dup_printf("%f",VAL(val));
 }
 
-m_option_type_t m_option_type_float = {
+const m_option_type_t m_option_type_float = {
   "Float",
   "floating point number or ratio (numerator[:/]denominator)",
   sizeof(float),
@@ -282,7 +289,7 @@ m_option_type_t m_option_type_float = {
 #undef VAL
 #define VAL(x) (*(off_t*)(x))
 
-static int parse_position(m_option_t* opt,char *name, char *param, void* dst, int src) {
+static int parse_position(const m_option_t* opt,const char *name, char *param, void* dst, int src) {
   off_t tmp_off;
   char dummy;
 
@@ -315,11 +322,11 @@ static int parse_position(m_option_t* opt,char *name, char *param, void* dst, in
   return 1;
 }
 
-static char* print_position(m_option_t* opt,  void* val) {
+static char* print_position(const m_option_t* opt,  const void* val) {
   return dup_printf("%"PRId64,(int64_t)VAL(val));
 }
 
-m_option_type_t m_option_type_position = {
+const m_option_type_t m_option_type_position = {
   "Position",
   "Integer (off_t)",
   sizeof(off_t),
@@ -338,7 +345,7 @@ m_option_type_t m_option_type_position = {
 #undef VAL
 #define VAL(x) (*(char**)(x))
 
-static int parse_str(m_option_t* opt,char *name, char *param, void* dst, int src) {
+static int parse_str(const m_option_t* opt,const char *name, char *param, void* dst, int src) {
   
 
   if (param == NULL)
@@ -366,11 +373,11 @@ static int parse_str(m_option_t* opt,char *name, char *param, void* dst, int src
 
 }
 
-static char* print_str(m_option_t* opt,  void* val) {
+static char* print_str(const m_option_t* opt,  const void* val) {
   return (val && VAL(val) && strlen(VAL(val)) > 0) ? strdup(VAL(val)) : NULL;
 }
 
-static void copy_str(m_option_t* opt,void* dst, void* src) {
+static void copy_str(const m_option_t* opt,void* dst, void* src) {
   if(dst && src) {
 #ifndef NO_FREE
     if(VAL(dst)) free(VAL(dst)); //FIXME!!!
@@ -388,7 +395,7 @@ static void free_str(void* src) {
   }
 }
 
-m_option_type_t m_option_type_string = {
+const m_option_type_t m_option_type_string = {
   "String",
   "",
   sizeof(char*),
@@ -505,14 +512,27 @@ static int str_list_del(char** del, int n,void* dst) {
   return 1;
 }
   
+static char *get_nextsep(char *ptr, char sep, int modify) {
+    char *last_ptr = ptr;
+    for(;;){
+        ptr = strchr(ptr, sep);
+        if(ptr && ptr>last_ptr && ptr[-1]=='\\'){
+            if (modify) memmove(ptr-1, ptr, strlen(ptr)+1);
+            else ptr++;
+        }else
+            break;
+    }
+    return ptr;
+}
 
-static int parse_str_list(m_option_t* opt,char *name, char *param, void* dst, int src) {
+static int parse_str_list(const m_option_t* opt,const char *name, char *param, void* dst, int src) {
   int n = 0,len = strlen(opt->name);
+  char *str;
   char *ptr = param, *last_ptr, **res;
   int op = OP_NONE;
 
   if(opt->name[len-1] == '*' && ((int)strlen(name) > len - 1)) {
-    char* n = &name[len-1];
+    const char* n = &name[len-1];
     if(strcasecmp(n,"-add") == 0)
       op = OP_ADD;
     else if(strcasecmp(n,"-pre") == 0)
@@ -538,8 +558,7 @@ static int parse_str_list(m_option_t* opt,char *name, char *param, void* dst, in
 
 
   while(ptr[0] != '\0') {
-    last_ptr = ptr;
-    ptr = strchr(ptr,LIST_SEPARATOR);
+    ptr = get_nextsep(ptr, LIST_SEPARATOR, 0);
     if(!ptr) {
       n++;
       break;
@@ -556,25 +575,26 @@ static int parse_str_list(m_option_t* opt,char *name, char *param, void* dst, in
   if(!dst) return 1;
 
   res = malloc((n+2)*sizeof(char*));
-  ptr = param;
+  ptr = str = strdup(param);
   n = 0;
 
   while(1) {
     last_ptr = ptr;
-    ptr = strchr(ptr,LIST_SEPARATOR);
+    ptr = get_nextsep(ptr, LIST_SEPARATOR, 1);
     if(!ptr) {
       res[n] = strdup(last_ptr);
       n++;
       break;
     }
     len = ptr - last_ptr;
-    res[n] = (char*)malloc(len + 1);
+    res[n] = malloc(len + 1);
     if(len) strncpy(res[n],last_ptr,len);
     res[n][len] = '\0';
     ptr++;
     n++;
   }
   res[n] = NULL;
+  free(str);
 
   switch(op) {
   case OP_ADD:
@@ -592,7 +612,7 @@ static int parse_str_list(m_option_t* opt,char *name, char *param, void* dst, in
   return 1;
 }
   
-static void copy_str_list(m_option_t* opt,void* dst, void* src) {
+static void copy_str_list(const m_option_t* opt,void* dst, void* src) {
   int n;
   char **d,**s;
 
@@ -609,14 +629,14 @@ static void copy_str_list(m_option_t* opt,void* dst, void* src) {
 
   for(n = 0 ; s[n] != NULL ; n++)
     /* NOTHING */;
-  d = (char**)malloc((n+1)*sizeof(char*));
+  d = malloc((n+1)*sizeof(char*));
   for( ; n >= 0 ; n--)
     d[n] = s[n] ? strdup(s[n]) : NULL;
 
   VAL(dst) = d;
 }
 
-static char* print_str_list(m_option_t* opt, void* src) {
+static char* print_str_list(const m_option_t* opt, const void* src) {
   char **lst = NULL;
   char *ret = NULL,*last = NULL;
   int i;
@@ -636,12 +656,12 @@ static char* print_str_list(m_option_t* opt, void* src) {
   return ret;
 }
 
-m_option_type_t m_option_type_string_list = {
+const m_option_type_t m_option_type_string_list = {
   "String list",
   "A list of strings separated by ','\n"
   "Option with a name ending in an * permits using the following suffix: \n"
   "\t-add: Add the given parameters at the end of the list.\n"
-  "\t-pre: Add the given parameters at the begining of the list.\n"
+  "\t-pre: Add the given parameters at the beginning of the list.\n"
   "\t-del: Remove the entry at the given indices.\n"
   "\t-clr: Clear the list.\n"
   "e.g: -vf-add flip,mirror -vf-del 2,5\n",  
@@ -687,13 +707,13 @@ static void free_func_pf(void* src) {
 }
 
 // Parser for func_param and func_full
-static int parse_func_pf(m_option_t* opt,char *name, char *param, void* dst, int src) {
+static int parse_func_pf(const m_option_t* opt,const char *name, char *param, void* dst, int src) {
   m_func_save_t *s,*p;
 
   if(!dst)
     return 1;
 
-  s = (m_func_save_t*)calloc(1,sizeof(m_func_save_t));
+  s = calloc(1,sizeof(m_func_save_t));
   s->name = strdup(name);
   s->param = param ? strdup(param) : NULL;
 
@@ -708,7 +728,7 @@ static int parse_func_pf(m_option_t* opt,char *name, char *param, void* dst, int
   return 1;
 }
 
-static void copy_func_pf(m_option_t* opt,void* dst, void* src) {
+static void copy_func_pf(const m_option_t* opt,void* dst, void* src) {
   m_func_save_t *d = NULL, *s,* last = NULL;
 
   if(!(dst && src)) return;
@@ -718,7 +738,7 @@ static void copy_func_pf(m_option_t* opt,void* dst, void* src) {
     free_func_pf(dst);
 
   while(s) {
-    d = (m_func_save_t*)calloc(1,sizeof(m_func_save_t));
+    d = calloc(1,sizeof(m_func_save_t));
     d->name = strdup(s->name);
     d->param = s->param ? strdup(s->param) : NULL;
     if(last)
@@ -734,7 +754,7 @@ static void copy_func_pf(m_option_t* opt,void* dst, void* src) {
 
 /////////////////// Func_param
 
-static void set_func_param(m_option_t* opt, void* dst, void* src) {
+static void set_func_param(const m_option_t* opt, void* dst, void* src) {
   m_func_save_t* s;
 
   if(!src) return;
@@ -748,7 +768,7 @@ static void set_func_param(m_option_t* opt, void* dst, void* src) {
     ((m_opt_func_param_t) opt->p)(opt,s->param);
 }
 
-m_option_type_t m_option_type_func_param = {
+const m_option_type_t m_option_type_func_param = {
   "Func param",
   "",
   sizeof(m_func_save_t*),
@@ -763,7 +783,7 @@ m_option_type_t m_option_type_func_param = {
 
 /////////////////// Func_full
 
-static void set_func_full(m_option_t* opt, void* dst, void* src) {
+static void set_func_full(const m_option_t* opt, void* dst, void* src) {
   m_func_save_t* s;
 
   if(!src) return;
@@ -775,7 +795,7 @@ static void set_func_full(m_option_t* opt, void* dst, void* src) {
   }
 }
 
-m_option_type_t m_option_type_func_full = {
+const m_option_type_t m_option_type_func_full = {
   "Func full",
   "",
   sizeof(m_func_save_t*),
@@ -793,20 +813,20 @@ m_option_type_t m_option_type_func_full = {
 #undef VAL
 #define VAL(x) (*(int*)(x))
 
-static int parse_func(m_option_t* opt,char *name, char *param, void* dst, int src) {
+static int parse_func(const m_option_t* opt,const char *name, char *param, void* dst, int src) {
   if(dst)
     VAL(dst) += 1;
   return 0;
 }
 
-static void set_func(m_option_t* opt,void* dst, void* src) {
+static void set_func(const m_option_t* opt,void* dst, void* src) {
   int i;
   if(opt->priv) ((m_opt_default_func_t)opt->priv)(opt,opt->name);
   for(i = 0 ; i < VAL(src) ; i++)
     ((m_opt_func_t) opt->p)(opt);
 }
 
-m_option_type_t m_option_type_func = {
+const m_option_type_t m_option_type_func = {
   "Func",
   "",
   sizeof(int),
@@ -821,7 +841,7 @@ m_option_type_t m_option_type_func = {
 
 /////////////////// Print
 
-static int parse_print(m_option_t* opt,char *name, char *param, void* dst, int src) {
+static int parse_print(const m_option_t* opt,const char *name, char *param, void* dst, int src) {
   if(opt->type == CONF_TYPE_PRINT_INDIRECT) 
     mp_msg(MSGT_CFGPARSER, MSGL_INFO, "%s", *(char **) opt->p);
   else if(opt->type == CONF_TYPE_PRINT_FUNC)
@@ -834,7 +854,7 @@ static int parse_print(m_option_t* opt,char *name, char *param, void* dst, int s
   return 1;
 }
 
-m_option_type_t m_option_type_print = {
+const m_option_type_t m_option_type_print = {
   "Print",
   "",
   0,
@@ -847,7 +867,7 @@ m_option_type_t m_option_type_print = {
   NULL
 };
 
-m_option_type_t m_option_type_print_indirect = {
+const m_option_type_t m_option_type_print_indirect = {
   "Print",
   "",
   0,
@@ -860,7 +880,7 @@ m_option_type_t m_option_type_print_indirect = {
   NULL
 };
 
-m_option_type_t m_option_type_print_func = {
+const m_option_type_t m_option_type_print_func = {
   "Print",
   "",
   0,
@@ -878,12 +898,12 @@ m_option_type_t m_option_type_print_func = {
 #undef VAL
 #define VAL(x) (*(char***)(x))
 
-static int parse_subconf(m_option_t* opt,char *name, char *param, void* dst, int src) {
+static int parse_subconf(const m_option_t* opt,const char *name, char *param, void* dst, int src) {
   char *subparam;
   char *subopt;
   int nr = 0,i,r;
-  m_option_t *subopts;
-  char *p;
+  const m_option_t *subopts;
+  const char *p;
   char** lst = NULL;
 
   if (param == NULL || strlen(param) == 0)
@@ -901,7 +921,7 @@ static int parse_subconf(m_option_t* opt,char *name, char *param, void* dst, int
       int optlen = strcspn(p, ":=");
       /* clear out */
       subopt[0] = subparam[0] = 0;
-      strlcpy(subopt, p, optlen + 1);
+      av_strlcpy(subopt, p, optlen + 1);
       p = &p[optlen];
       if (p[0] == '=') {
         sscanf_ret = 2;
@@ -909,7 +929,7 @@ static int parse_subconf(m_option_t* opt,char *name, char *param, void* dst, int
         if (p[0] == '"') {
           p = &p[1];
           optlen = strcspn(p, "\"");
-          strlcpy(subparam, p, optlen + 1);
+          av_strlcpy(subparam, p, optlen + 1);
           p = &p[optlen];
           if (p[0] != '"') {
             mp_msg(MSGT_CFGPARSER, MSGL_ERR, "Terminating '\"' missing for '%s'\n", subopt);
@@ -918,17 +938,17 @@ static int parse_subconf(m_option_t* opt,char *name, char *param, void* dst, int
           p = &p[1];
         } else if (p[0] == '%') {
           p = &p[1];
-          optlen = (int)strtol(p, &p, 0);
+          optlen = (int)strtol(p, (char**)&p, 0);
           if (!p || p[0] != '%' || (optlen > strlen(p) - 1)) {
             mp_msg(MSGT_CFGPARSER, MSGL_ERR, "Invalid length %i for '%s'\n", optlen, subopt);
             return M_OPT_INVALID;
           }
           p = &p[1];
-          strlcpy(subparam, p, optlen + 1);
+          av_strlcpy(subparam, p, optlen + 1);
           p = &p[optlen];
         } else {
           optlen = strcspn(p, ":");
-          strlcpy(subparam, p, optlen + 1);
+          av_strlcpy(subparam, p, optlen + 1);
           p = &p[optlen];
         }
       }
@@ -973,7 +993,7 @@ static int parse_subconf(m_option_t* opt,char *name, char *param, void* dst, int
   return 1;
 }
 
-m_option_type_t m_option_type_subconfig = {
+const m_option_type_t m_option_type_subconfig = {
   "Subconfig",
   "The syntax is -option opt1=foo:flag:opt2=blah",
   sizeof(int),
@@ -990,7 +1010,7 @@ m_option_type_t m_option_type_subconfig = {
 
 /* FIXME: snyc with img_format.h */
 static struct {
-  char* name;
+  const char* name;
   unsigned int fmt;
 } mp_imgfmt_list[] = {
   {"444p", IMGFMT_444P},
@@ -1029,10 +1049,12 @@ static struct {
   {"argb", IMGFMT_ARGB},
   {"bgra", IMGFMT_BGRA},
   {"abgr", IMGFMT_ABGR},
+  {"mjpeg", IMGFMT_MJPEG},
+  {"mjpg", IMGFMT_MJPEG},
   { NULL, 0 }
 };
 
-static int parse_imgfmt(m_option_t* opt,char *name, char *param, void* dst, int src) {
+static int parse_imgfmt(const m_option_t* opt,const char *name, char *param, void* dst, int src) {
   uint32_t fmt = 0;
   int i;
 
@@ -1067,7 +1089,7 @@ static int parse_imgfmt(m_option_t* opt,char *name, char *param, void* dst, int
   return 1;
 }
 
-m_option_type_t m_option_type_imgfmt = {
+const m_option_type_t m_option_type_imgfmt = {
   "Image format",
   "Please report any missing colorspaces.",
   sizeof(uint32_t),
@@ -1084,7 +1106,7 @@ m_option_type_t m_option_type_imgfmt = {
 
 /* FIXME: snyc with af_format.h */
 static struct {
-  char* name;
+  const char* name;
   unsigned int fmt;
 } mp_afmt_list[] = {
   // SPECIAL
@@ -1120,7 +1142,7 @@ static struct {
   { NULL, 0 }
 };
 
-static int parse_afmt(m_option_t* opt,char *name, char *param, void* dst, int src) {
+static int parse_afmt(const m_option_t* opt,const char *name, char *param, void* dst, int src) {
   uint32_t fmt = 0;
   int i;
 
@@ -1155,7 +1177,7 @@ static int parse_afmt(m_option_t* opt,char *name, char *param, void* dst, int sr
   return 1;
 }
 
-m_option_type_t m_option_type_afmt = {
+const m_option_type_t m_option_type_afmt = {
   "Audio format",
   "Please report any missing formats.",
   sizeof(uint32_t),
@@ -1169,6 +1191,114 @@ m_option_type_t m_option_type_afmt = {
 };
 
 
+static double parse_timestring(const char *str)
+{
+  int a, b;
+  double d;
+  if (sscanf(str, "%d:%d:%lf", &a, &b, &d) == 3)
+    return 3600*a + 60*b + d;
+  else if (sscanf(str, "%d:%lf", &a, &d) == 2)
+    return 60*a + d;
+  else if (sscanf(str, "%lf", &d) == 1)
+    return d;
+  return -1e100;
+}
+    
+
+static int parse_time(const m_option_t* opt,const char *name, char *param, void* dst, int src)
+{
+  double time;
+
+  if (param == NULL || strlen(param) == 0)
+    return M_OPT_MISSING_PARAM;
+  
+  time = parse_timestring(param);
+  if (time == -1e100) {
+    mp_msg(MSGT_CFGPARSER, MSGL_ERR, "Option %s: invalid time: '%s'\n",
+           name,param);
+    return M_OPT_INVALID;
+  }
+  
+  if (dst)
+    *(double *)dst = time;
+  return 1;
+}
+
+const m_option_type_t m_option_type_time = {
+  "Time",
+  "",
+  sizeof(double),
+  0,
+  parse_time,
+  print_double,
+  copy_opt,
+  copy_opt,
+  NULL,
+  NULL
+};
+
+
+// Time or size (-endpos)
+
+static int parse_time_size(const m_option_t* opt,const char *name, char *param, void* dst, int src) {
+  m_time_size_t ts;
+  char unit[4];
+  double end_at;
+
+  if (param == NULL || strlen(param) == 0)
+    return M_OPT_MISSING_PARAM;
+  
+  ts.pos=0;
+  /* End at size parsing */
+  if(sscanf(param, "%lf%3s", &end_at, unit) == 2) {
+    ts.type = END_AT_SIZE;
+    if(!strcasecmp(unit, "b"))
+      ;
+    else if(!strcasecmp(unit, "kb"))
+      end_at *= 1024;
+    else if(!strcasecmp(unit, "mb"))
+      end_at *= 1024*1024;
+    else if(!strcasecmp(unit, "gb"))
+      end_at *= 1024*1024*1024;
+    else
+      ts.type = END_AT_NONE;
+
+    if (ts.type == END_AT_SIZE) {
+      ts.pos  = end_at;
+      goto out;
+    }
+  }
+
+  /* End at time parsing. This has to be last because the parsing accepts
+   * even a number followed by garbage */
+  if ((end_at = parse_timestring(param)) == -1e100) {
+    mp_msg(MSGT_CFGPARSER, MSGL_ERR, "Option %s: invalid time or size: '%s'\n",
+           name,param);
+    return M_OPT_INVALID;
+  }
+  
+  ts.type = END_AT_TIME;
+  ts.pos  = end_at;
+out:
+  if(dst)
+    *(m_time_size_t *)dst = ts;
+  return 1;
+}
+
+const m_option_type_t m_option_type_time_size = {
+  "Time or size",
+  "",
+  sizeof(m_time_size_t),
+  0,
+  parse_time_size,
+  NULL,
+  copy_opt,
+  copy_opt,
+  NULL,
+  NULL
+};
+
+ 
 //// Objects (i.e. filters, etc) settings
 
 #include "m_struct.h"
@@ -1176,7 +1306,7 @@ m_option_type_t m_option_type_afmt = {
 #undef VAL
 #define VAL(x) (*(m_obj_settings_t**)(x))
 
-static int find_obj_desc(char* name,m_obj_list_t* l,m_struct_t** ret) {
+static int find_obj_desc(const char* name,const m_obj_list_t* l,const m_struct_t** ret) {
   int i;
   char* n;
 
@@ -1190,10 +1320,10 @@ static int find_obj_desc(char* name,m_obj_list_t* l,m_struct_t** ret) {
   return 0;
 }
 
-static int get_obj_param(char* opt_name,char* obj_name, m_struct_t* desc,
+static int get_obj_param(const char* opt_name,const char* obj_name, const m_struct_t* desc,
 			 char* str,int* nold,int oldmax,char** dst) {
-  char* eq,param;
-  m_option_t* opt;
+  char* eq;
+  const m_option_t* opt;
   int r;
 
   eq = strchr(str,'=');
@@ -1243,10 +1373,10 @@ static int get_obj_param(char* opt_name,char* obj_name, m_struct_t* desc,
   return 1;
 }
 
-static int get_obj_params(char* opt_name, char* name,char* params,
-			  m_struct_t* desc,char separator, char*** _ret) {
+static int get_obj_params(const char* opt_name, const char* name,char* params,
+			  const m_struct_t* desc,char separator, char*** _ret) {
   int n = 0,nold = 0, nopts,r;
-  char* ptr,*last_ptr = params,*eq;
+  char* ptr,*last_ptr = params;
   char** ret;
 
   if(!strcmp(params,"help")) { // Help
@@ -1257,7 +1387,7 @@ static int get_obj_params(char* opt_name, char* name,char* params,
     }
     printf("\n Name                 Type            Min        Max\n\n");
     for(n = 0 ; desc->fields[n].name ; n++) {
-      m_option_t* opt = &desc->fields[n];
+      const m_option_t* opt = &desc->fields[n];
       if(opt->type->flags & M_OPT_TYPE_HAS_CHILD) continue;
       if(opt->flags & M_OPT_MIN)
 	sprintf(min,"%-8.0f",opt->min);
@@ -1340,19 +1470,21 @@ static int get_obj_params(char* opt_name, char* name,char* params,
   return 1;
 }
 
-static int parse_obj_params(m_option_t* opt,char *name,
+static int parse_obj_params(const m_option_t* opt,const char *name,
 			    char *param, void* dst, int src) {
   char** opts;
   int r;
   m_obj_params_t* p = opt->priv;
-  m_struct_t* desc = p->desc;
-  char* cpy = strdup(param);
+  const m_struct_t* desc;
+  char* cpy;
   
   // We need the object desc
   if(!p)
     return M_OPT_INVALID;
   
-  r = get_obj_params(name,desc->name,cpy,desc,p->separator,&opts);
+  desc = p->desc;
+  cpy = strdup(param);
+  r = get_obj_params(name,desc->name,cpy,desc,p->separator,dst ? &opts : NULL);
   free(cpy);
   if(r < 0)
     return r;
@@ -1368,7 +1500,7 @@ static int parse_obj_params(m_option_t* opt,char *name,
 }
 
 
-m_option_type_t m_option_type_obj_params = {
+const m_option_type_t m_option_type_obj_params = {
   "Object params",
   "",
   0,
@@ -1384,28 +1516,28 @@ m_option_type_t m_option_type_obj_params = {
 /// Some predefined types as a definition would be quite lengthy
 
 /// Span arguments
-static m_span_t m_span_params_dflts = { -1, -1 };
-static m_option_t m_span_params_fields[] = {
+static const m_span_t m_span_params_dflts = { -1, -1 };
+static const m_option_t m_span_params_fields[] = {
   {"start", M_ST_OFF(m_span_t,start), CONF_TYPE_INT, M_OPT_MIN, 1 ,0, NULL},
   {"end", M_ST_OFF(m_span_t,end), CONF_TYPE_INT, M_OPT_MIN , 1 ,0, NULL},
   { NULL, NULL, 0, 0, 0, 0,  NULL }
 };
-static struct m_struct_st m_span_opts = {
+static const struct m_struct_st m_span_opts = {
   "m_span",
   sizeof(m_span_t),
   &m_span_params_dflts,
   m_span_params_fields
 };
-m_obj_params_t m_span_params_def = {
+const m_obj_params_t m_span_params_def = {
   &m_span_opts,
   '-'
 };
 
-static int parse_obj_settings(char* opt,char* str,m_obj_list_t* list,
+static int parse_obj_settings(const char* opt,char* str,const m_obj_list_t* list,
 			      m_obj_settings_t **_ret, int ret_n) {
   int r;
   char *param,**plist = NULL;
-  m_struct_t* desc;
+  const m_struct_t* desc;
   m_obj_settings_t *ret = _ret ? *_ret : NULL;
   
 
@@ -1453,11 +1585,11 @@ static int parse_obj_settings(char* opt,char* str,m_obj_list_t* list,
 
 static void free_obj_settings_list(void* dst);
 
-static int obj_settings_list_del(char *opt_name,char *param,void* dst, int src) {
+static int obj_settings_list_del(const char *opt_name,char *param,void* dst, int src) {
   char** str_list = NULL;
   int r,i,idx_max = 0;
   char* rem_id = "_removed_marker_";
-  m_option_t list_opt = {opt_name , NULL, CONF_TYPE_STRING_LIST,
+  const m_option_t list_opt = {opt_name , NULL, CONF_TYPE_STRING_LIST,
 			   0, 0, 0, NULL };
   m_obj_settings_t* obj_list = dst ? VAL(dst) : NULL;
 
@@ -1511,7 +1643,7 @@ static int obj_settings_list_del(char *opt_name,char *param,void* dst, int src)
   return 1;
 }
 
-static int parse_obj_settings_list(m_option_t* opt,char *name,
+static int parse_obj_settings_list(const m_option_t* opt,const char *name,
 				   char *param, void* dst, int src) {
   int n = 0,r,len = strlen(opt->name);
   char *str;
@@ -1524,7 +1656,7 @@ static int parse_obj_settings_list(m_option_t* opt,char *name,
     return M_OPT_INVALID;
 
   if(opt->name[len-1] == '*' && ((int)strlen(name) > len - 1)) {
-    char* n = &name[len-1];
+    const char* n = &name[len-1];
     if(strcasecmp(n,"-add") == 0)
       op = OP_ADD;
     else if(strcasecmp(n,"-pre") == 0)
@@ -1584,8 +1716,7 @@ static int parse_obj_settings_list(m_option_t* opt,char *name,
   if(!strcmp(param,"help")) {
     m_obj_list_t* ol = opt->priv;
     mp_msg(MSGT_VFILTER,MSGL_INFO,"Available video filters:\n");
-    if (identify)
-      mp_msg(MSGT_GLOBAL, MSGL_INFO, "ID_VIDEO_FILTERS\n");
+    mp_msg(MSGT_IDENTIFY, MSGL_INFO, "ID_VIDEO_FILTERS\n");
     for(n = 0 ; ol->list[n] ; n++)
       mp_msg(MSGT_VFILTER,MSGL_INFO,"  %-15s: %s\n",
 	     M_ST_MB(char*,ol->list[n],ol->name_off),
@@ -1597,7 +1728,8 @@ static int parse_obj_settings_list(m_option_t* opt,char *name,
 
   while(ptr[0] != '\0') {
     last_ptr = ptr;
-    ptr = strchr(ptr,LIST_SEPARATOR);
+    ptr = get_nextsep(ptr, LIST_SEPARATOR, 1);
+
     if(!ptr) {
       r = parse_obj_settings(name,last_ptr,opt->priv,dst ? &res : NULL,n);
       if(r < 0) {
@@ -1665,7 +1797,7 @@ static void free_obj_settings_list(void* dst) {
   VAL(dst) = NULL;
 }
 
-static void copy_obj_settings_list(m_option_t* opt,void* dst, void* src) {
+static void copy_obj_settings_list(const m_option_t* opt,void* dst, void* src) {
   m_obj_settings_t *d,*s;
   int n;
 
@@ -1693,7 +1825,7 @@ static void copy_obj_settings_list(m_option_t* opt,void* dst, void* src) {
   VAL(dst) = d;
 }
 
-m_option_type_t m_option_type_obj_settings_list = {
+const m_option_type_t m_option_type_obj_settings_list = {
   "Object settings list",
   "",
   sizeof(m_obj_settings_t*),
@@ -1708,12 +1840,12 @@ m_option_type_t m_option_type_obj_settings_list = {
 
 
 
-static int parse_obj_presets(m_option_t* opt,char *name,
+static int parse_obj_presets(const m_option_t* opt,const char *name,
 			    char *param, void* dst, int src) {
   m_obj_presets_t* obj_p = (m_obj_presets_t*)opt->priv;
   m_struct_t *in_desc,*out_desc;
   int s,i;
-  unsigned char* pre = obj_p->presets;
+  unsigned char* pre;
   char* pre_name = NULL;
 
   if(!obj_p) {
@@ -1724,6 +1856,7 @@ static int parse_obj_presets(m_option_t* opt,char *name,
   if(!param)
     return M_OPT_MISSING_PARAM;
 
+  pre = obj_p->presets;
   in_desc = obj_p->in_desc;
   out_desc = obj_p->out_desc ? obj_p->out_desc : obj_p->in_desc;
   s = in_desc->size;
@@ -1754,7 +1887,7 @@ static int parse_obj_presets(m_option_t* opt,char *name,
   if(!dst) return 1;
   
   for(i = 0 ; in_desc->fields[i].name ; i++) {
-    m_option_t* out_opt = m_option_list_find(out_desc->fields,
+    const m_option_t* out_opt = m_option_list_find(out_desc->fields,
 					     in_desc->fields[i].name);
     if(!out_opt) {
       mp_msg(MSGT_CFGPARSER, MSGL_ERR, "Option %s: Unable to find the target option for field %s.\nPlease report this to the developers.\n",name,in_desc->fields[i].name);
@@ -1766,7 +1899,7 @@ static int parse_obj_presets(m_option_t* opt,char *name,
 }
 
 
-m_option_type_t m_option_type_obj_presets = {
+const m_option_type_t m_option_type_obj_presets = {
   "Object presets",
   "",
   0,
@@ -1779,7 +1912,7 @@ m_option_type_t m_option_type_obj_presets = {
   NULL
 };
 
-static int parse_custom_url(m_option_t* opt,char *name,
+static int parse_custom_url(const m_option_t* opt,const char *name,
 			    char *url, void* dst, int src) {
   int pos1, pos2, r, v6addr = 0;
   char *ptr1=NULL, *ptr2=NULL, *ptr3=NULL, *ptr4=NULL;
@@ -1832,7 +1965,6 @@ static int parse_custom_url(m_option_t* opt,char *name,
   if( ptr2!=NULL ) {
     
     // We got something, at least a username...
-    int len = ptr2-ptr1;
     if(!m_option_list_find(desc->fields,"username")) {
       mp_msg(MSGT_CFGPARSER, MSGL_WARN, "Option %s: This URL doesn't have a username part.\n",name);
       // skip
@@ -1840,7 +1972,6 @@ static int parse_custom_url(m_option_t* opt,char *name,
       ptr3 = strstr(ptr1, ":");
       if( ptr3!=NULL && ptr3<ptr2 ) {
 	// We also have a password
-	int len2 = ptr2-ptr3-1;
 	if(!m_option_list_find(desc->fields,"password")) {
 	  mp_msg(MSGT_CFGPARSER, MSGL_WARN, "Option %s: This URL doesn't have a password part.\n",name);
 	  // skip
@@ -1978,7 +2109,7 @@ static int parse_custom_url(m_option_t* opt,char *name,
 }
 
 /// TODO : Write the other needed funcs for 'normal' options
-m_option_type_t m_option_type_custom_url = {
+const m_option_type_t m_option_type_custom_url = {
   "Custom URL",
   "",
   0,
